/****************************************************************************
#	 	GspcaGui:  Gspca/Spca5xx Grabber                            #
# 		Copyright (C) 2004 2005 2006 Michel Xhaard                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#                                                                           #
****************************************************************************/ 
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "../colors/color.h"
/********* Image_format know by the encoder *********************/
/* Native YUV Packet */
#define  FOUR_ZERO_ZERO 0	// Grey scale Y00 ...
#define  FOUR_TWO_ZERO 1	//Y00 Y01 Y10 Y11 Cb Cr
#define  FOUR_TWO_TWO 2		//Y00 Cb Y01 Cr
#define  FOUR_FOUR_FOUR 3	//Y00 Cb Cr Y01 Cb Cr
/*****************************************************************/

/* translate RGB24 to YUV444 packed output */
unsigned int
rgb24toPyuv444(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B;
	int Y, Cb, Cr;
	size = image_width * image_height;
	for (i = size; i > 0; i--) {
		B = input_ptr[0];
		G = input_ptr[1];
		R = input_ptr[2];
		Y = RGB24_TO_Y(R,G,B);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*output_ptr++ = (unsigned char) Y;
		*output_ptr++ = (unsigned char) Cb;
		*output_ptr++ = (unsigned char) Cr;
	}
	return FOUR_FOUR_FOUR;
}


/* translate RGB24 to YUV422 packed output */
unsigned int
rgb24toPyuv422(unsigned char * input_ptr,unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B, R1, G1, B1;
	int Y, Yp, Cb, Cr;
	unsigned char *inbuf = input_ptr;
	size = image_width * image_height/2;
	for (i = size; i > 0; i--) {
		B = inbuf[0];
		G = inbuf[1];
		R = inbuf[2];
		B1 = inbuf[3];
		G1 = inbuf[4];
		R1 = inbuf[5];
		inbuf += 6;
		Y = RGB24_TO_Y(R,G,B);
		Yp = RGB24_TO_Y(R1,G1,B1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*output_ptr++ = (unsigned char) Y;
		*output_ptr++ = (unsigned char) Cb;
		*output_ptr++ = (unsigned char) Yp;
		*output_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_TWO;
}

/* translate RGB565 to YUV422 packed output */
unsigned int
rgb565toPyuv422(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B, R1, G1, B1;
	int Y, Yp, Cb, Cr;
	Myrgb16 *inbuf = (Myrgb16 *) input_ptr;
	size = image_width * image_height/2;
	for (i = size; i > 0; i--) {
	#if 0
		B = inbuf[0].red << 3;
		G = inbuf[0].green << 2;
		R = inbuf[0].blue << 3;
		B1 = inbuf[1].red << 3;
		G1 = inbuf[1].green << 2;
		R1 = inbuf[1].blue << 3;
	#endif
	        B = inbuf[0].blue << 3;
		G = inbuf[0].green << 2;
		R = inbuf[0].red << 3;
		B1 = inbuf[1].blue << 3;
		G1 = inbuf[1].green << 2;
		R1 = inbuf[1].red << 3;
		inbuf += 2;
		Y = RGB24_TO_Y(R,G,B);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		Yp = RGB24_TO_Y(R1,G1,B1);
		*output_ptr++ = (unsigned char) Y;
		*output_ptr++ = (unsigned char) Cb;
		*output_ptr++ = (unsigned char) Yp;
		*output_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_TWO;
}

/* translate RGB32 to YUV422 packed output */
unsigned int
rgb32toPyuv422(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B, R1, G1, B1;
	int Y, Yp, Cb, Cr;
	Myrgb32 *inbuf = (Myrgb32 *) input_ptr;
	size = image_width * image_height/2;
	for (i = size; i > 0; i--) {
		B = inbuf[0].blue;
		G = inbuf[0].green;
		R = inbuf[0].red;
		B1 = inbuf[1].blue;
		G1 = inbuf[1].green;
		R1 = inbuf[1].red;
		inbuf += 2;
		
		Y = RGB24_TO_Y(R,G,B);
		Yp = RGB24_TO_Y(R1,G1,B1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*output_ptr++ = (unsigned char) Y;
		*output_ptr++ = (unsigned char) Cb;
		*output_ptr++ = (unsigned char) Yp;
		*output_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_TWO;
}

/* translate RGB24 to YUV420 packed output */
unsigned int
rgb24toPyuv420(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, j, size;
	unsigned char R, G, B, R1, G1, B1, Rd, Gd, Bd, Rd1, Gd1, Bd1;
	int Y, Yd, Y11, Yd1, Cb, Cr;
	unsigned char *inbuf = input_ptr;
	unsigned char *inbuf1 = input_ptr + (image_width * 3);
	size = image_width * image_height >> 2;
	for (i = size, j = 0; i > 0; i--) {
		B = inbuf[0];
		G = inbuf[1];
		R = inbuf[2];
		B1 = inbuf[3];
		G1 = inbuf[4];
		R1 = inbuf[5];
		Bd = inbuf1[0];
		Gd = inbuf1[1];
		Rd = inbuf1[2];
		Bd1 = inbuf1[3];
		Gd1 = inbuf1[4];
		Rd1 = inbuf1[5];
		inbuf += 6;
		inbuf1 += 6;
		j++;
		if (j >= image_width / 2) {
			j = 0;
			inbuf += (image_width * 3);
			inbuf1 += (image_width * 3);
		}
		
		Y = RGB24_TO_Y(R,G,B);
		Y11 = RGB24_TO_Y(R1,G1,B1);
		Yd = RGB24_TO_Y(Rd,Gd,Bd);
		Yd1 = RGB24_TO_Y(Rd1,Gd1,Bd1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*output_ptr++ = (unsigned char) Y;
		*output_ptr++ = (unsigned char) Y11;
		*output_ptr++ = (unsigned char) Yd;
		*output_ptr++ = (unsigned char) Yd1;
		*output_ptr++ = (unsigned char) Cb;
		*output_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_ZERO;
}


/* translate RGB32 to YUV420 packed output */
unsigned int
rgb32toPyuv420(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, j, size;
	unsigned char R, G, B, R1, G1, B1, Rd, Gd, Bd, Rd1, Gd1, Bd1;
	int Y, Yd, Y11, Yd1, Cb, Cr;
	unsigned char *inbuf = input_ptr;
	unsigned char *inbuf1 = input_ptr + (image_width * 4);
	size = image_width * image_height >> 2;
	for (i = size, j = 0; i > 0; i--) {
		B = inbuf[0];
		G = inbuf[1];
		R = inbuf[2];
		B1 = inbuf[4];
		G1 = inbuf[5];
		R1 = inbuf[6];
		Bd = inbuf1[0];
		Gd = inbuf1[1];
		Rd = inbuf1[2];
		Bd1 = inbuf1[4];
		Gd1 = inbuf1[5];
		Rd1 = inbuf1[6];
		inbuf += 8;
		inbuf1 += 8;
		j++;
		if (j >= image_width / 2) {
			j = 0;
			inbuf += (image_width * 4);
			inbuf1 += (image_width * 4);
		}
		
		Y = RGB24_TO_Y(R,G,B);
		Y11 = RGB24_TO_Y(R1,G1,B1);
		Yd = RGB24_TO_Y(Rd,Gd,Bd);
		Yd1 = RGB24_TO_Y(Rd1,Gd1,Bd1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*output_ptr++ = (unsigned char) Y;
		*output_ptr++ = (unsigned char) Y11;
		*output_ptr++ = (unsigned char) Yd;
		*output_ptr++ = (unsigned char) Yd1;
		*output_ptr++ = (unsigned char) Cb;
		*output_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_ZERO;
}


/* translate RGB565 to YUV420 packed output */
unsigned int
rgb565toPyuv420(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, j, size;
	unsigned char R, G, B, R1, G1, B1, Rd, Gd, Bd, Rd1, Gd1, Bd1;
	int Y, Yd, Y11, Yd1, Cb, Cr;
	Myrgb16 *inbuf = (Myrgb16 *) input_ptr;
	Myrgb16 *inbuf1 = inbuf + (image_width);
	size = image_width * image_height >> 2;
	for (i = size, j = 0; i > 0; i--) {
		B = inbuf[0].blue << 3;
		G = inbuf[0].green << 2;
		R = inbuf[0].red << 3;
		B1 = inbuf[1].blue << 3;
		G1 = inbuf[1].green << 2;
		R1 = inbuf[1].red << 3;
		Bd = inbuf1[0].blue << 3;
		Gd = inbuf1[0].green << 2;
		Rd = inbuf1[0].red << 3;
		Bd1 = inbuf1[1].blue << 3;
		Gd1 = inbuf1[1].green << 2;
		Rd1 = inbuf1[1].red << 3;
		inbuf += 2;
		inbuf1 += 2;
		j++;
		if (j >= image_width / 2) {
			j = 0;
			inbuf += (image_width);
			inbuf1 += (image_width);
		}
		
		Y = RGB24_TO_Y(R,G,B);
		Y11 = RGB24_TO_Y(R1,G1,B1);
		Yd = RGB24_TO_Y(Rd,Gd,Bd);
		Yd1 = RGB24_TO_Y(Rd1,Gd1,Bd1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*output_ptr++ = (unsigned char) Y;
		*output_ptr++ = (unsigned char) Y11;
		*output_ptr++ = (unsigned char) Yd;
		*output_ptr++ = (unsigned char) Yd1;
		*output_ptr++ = (unsigned char) Cb;
		*output_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_ZERO;
}

/* translate RGB24 to YUV400 packed output */
unsigned int
rgb24toPyuv400(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B;
	int Y;
	unsigned char *inbuf = input_ptr;
	size = image_width * image_height;
	for (i = size; i > 0; i--) {
		B = inbuf[0];
		G = inbuf[1];
		R = inbuf[2];
		inbuf += 3;
		
		Y = RGB24_TO_Y(R,G,B);
		*output_ptr++ = (unsigned char) Y;
	}
	return FOUR_ZERO_ZERO;
}


/* translate YUV444P to YUV444 packed output */
unsigned int
yuv444PtoPyuv444(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char *Ytmp = NULL;
	unsigned char *Cbtmp = NULL;
	unsigned char *Crtmp = NULL;
	unsigned char *Buff = input_ptr;
	
		Ytmp = Buff;
		Cbtmp = Buff + image_width * image_height;
		Crtmp = Buff + (image_width * image_height << 1);
		size = image_width * image_height;
		for (i = size; i > 0; i--) {
			*output_ptr++ = (unsigned char) * Ytmp++;
			*output_ptr++ = (unsigned char) * Cbtmp++;
			*output_ptr++ = (unsigned char) * Crtmp++;
		}
	
	return FOUR_FOUR_FOUR;
}


/* translate YUV422P to YUV422 packed output */
unsigned int
yuv422PtoPyuv422(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char *inbuf = input_ptr;
	unsigned char *Ytmp = NULL;
	unsigned char *Cbtmp = NULL;
	unsigned char *Crtmp = NULL;
	unsigned char *Buff = input_ptr;
	
		Ytmp = Buff;
		Cbtmp = Buff + image_width * image_height;
		Crtmp = Cbtmp + (image_width * image_height >> 1);
		size = image_width * image_height / 2;
		for (i = size; i > 0; i--) {
			*output_ptr++ = (unsigned char) * Ytmp++;
			*output_ptr++ = (unsigned char) * Cbtmp++;
			*output_ptr++ = (unsigned char) * Ytmp++;
			*output_ptr++ = (unsigned char) * Crtmp++;
		}
	
	return FOUR_TWO_TWO;
}

/* translate YUV422Packed to rgb24 */
unsigned int
Pyuv422torgb24(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	int U,V;
	unsigned char Y,Y1;
	unsigned char R,G,B,R1,G1,B1;
	unsigned char *Buff = input_ptr;
		size = image_width * image_height /2;
		for (i = size; i > 0; i--) {
		/* bgr instead rgb ?? */
		Y= *Buff++ ;
		V= *Buff++ ;
		Y1 = *Buff++;
		U = *Buff++;
	
			*output_ptr++ = R_FROMYV(Y,V);
			*output_ptr++ = G_FROMYUV(Y,U,V); //b
			*output_ptr++ = B_FROMYU(Y,U); //v
			
			*output_ptr++ = R_FROMYV(Y1,V);
			*output_ptr++ = G_FROMYUV(Y1,U,V); //b
			*output_ptr++ = B_FROMYU(Y1,U); //v

		}
	
	return FOUR_TWO_TWO;
}


/* translate YUV420P to YUV420 packed output */
unsigned int
yuv420PtoPyuv420(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int x, y, size;
	unsigned char *inbuf = input_ptr;
	unsigned char *Ytmp = NULL;
	unsigned char *Y2tmp = NULL;
	unsigned char *Cbtmp = NULL;
	unsigned char *Crtmp = NULL;
	unsigned char *Buff = input_ptr;
	
		Ytmp = Buff;
		Y2tmp = Buff + image_width;
		Cbtmp = Buff + image_width * image_height;
		Crtmp = Cbtmp + (image_width * image_height >> 2);
		size = image_width * image_height >> 2;
		for (y = 0; y < image_height; y += 2) {
			for (x = 0; x < image_width; x += 2) {
				*output_ptr++ = (unsigned char) * Ytmp++;
				*output_ptr++ = (unsigned char) * Ytmp++;
				*output_ptr++ = (unsigned char) * Y2tmp++;
				*output_ptr++ = (unsigned char) * Y2tmp++;
				*output_ptr++ = (unsigned char) * Cbtmp++;
				*output_ptr++ = (unsigned char) * Crtmp++;
			}
			Ytmp += image_width;
			Y2tmp += image_width;
		}
	
	return FOUR_TWO_ZERO;
}

/* translate YUV420P to YUV422 packed output */
unsigned int
yuv420PtoPyuv422(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int x, y, size;
	unsigned char *Ytmp = NULL;
	unsigned char *Y2tmp = NULL;
	unsigned char *Cbtmp = NULL;
	unsigned char *Crtmp = NULL;
	unsigned char *Buff = input_ptr;
	MyYUV422 *out0= (MyYUV422*) output_ptr;
	MyYUV422 *out1 = out0+(image_width/2);
	
		Ytmp = Buff;
		Y2tmp = Buff + image_width;
		Cbtmp = Buff + image_width * image_height;
		Crtmp = Cbtmp + (image_width * image_height >> 2);
		size = image_width * image_height >> 2;
		
		for (y = 0; y < image_height; y += 2) {
			for (x = 0; x < image_width/2; x++) {
				out0[x].y0 = (unsigned char) * Ytmp++;
				out0[x].u = (unsigned char) * Cbtmp;
				out0[x].y1 = (unsigned char) * Ytmp++;
				out0[x].v = (unsigned char) * Crtmp;
				
				out1[x].y0= (unsigned char) * Y2tmp++;
				out1[x].u  = (unsigned char) * Cbtmp++;
				out1[x].y1 = (unsigned char) * Y2tmp++;
				out1[x].v = (unsigned char) * Crtmp++;
			}
			Ytmp += image_width;
			Y2tmp += image_width;
			out0 += image_width;
			out1 += image_width;
		}
	
	return FOUR_TWO_TWO;
}
/* convert RGB24 to RGB32 */
unsigned int
rgb24torgb32(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	Myrgb32 *bufout = (Myrgb32 *) output_ptr;
	Myrgb24 *bufin =(Myrgb24 *) input_ptr;
	size = image_width * image_height;
	for (i = size; i > 0; i--) {
	memcpy(bufout,bufin,sizeof(Myrgb24));
	bufout++;
	bufin++;	
	}
	return 0;
}

/* convert RGB24 to RGB565 */
unsigned int
rgb24torgb565(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	Myrgb16 *bufout = (Myrgb16 *) output_ptr;
	Myrgb24 *bufin =(Myrgb24 *) input_ptr;
	size = image_width * image_height;
	for (i = size; i > 0; i--) {
	bufout->blue = bufin->blue >> 3;
	bufout->green = bufin->green >> 2;
	bufout->red = bufin->red >> 3;
	bufout++;
	bufin++;	
	}
	return 0;
}

/* convert RGB24 to yuv420p */
unsigned int
rgb24toyuv420p(unsigned char * input_ptr, unsigned char * output_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int x, y, size;
	unsigned char R, G, B, R1, G1, B1, Rd, Gd, Bd, Rd1, Gd1, Bd1,Y;
	unsigned char *Ytmp = NULL;
	unsigned char *Y2tmp = NULL;
	unsigned char *Cbtmp = NULL;
	unsigned char *Crtmp = NULL;
	Myrgb24 *bufin =(Myrgb24 *) input_ptr;
	Myrgb24 *bufin1 = bufin + image_width;
	size = image_width * image_height;
	Ytmp = output_ptr;
	Y2tmp = Ytmp + image_width;
	Cbtmp = Ytmp + size;
	Crtmp = Cbtmp + (size >> 2);
	for (y =0; y < image_height; y +=2){
		for (x = 0; x < image_width; x+=2, bufin +=2, bufin1 += 2){
		B = bufin[0].blue ;
		G = bufin[0].green ;
		R = bufin[0].red ;
		B1 = bufin[1].blue ;
		G1 = bufin[1].green ;
		R1 = bufin[1].red ;
		Bd = bufin1[0].blue ;
		Gd = bufin1[0].green ;
		Rd = bufin1[0].red ;
		Bd1 = bufin1[1].blue ;
		Gd1 = bufin1[1].green ;
		Rd1 = bufin1[1].red ;
		
		Y= RGB24_TO_Y(R,G,B);
		*Ytmp++=Y;
		*Ytmp++= RGB24_TO_Y(R1,G1,B1);
		*Y2tmp++= RGB24_TO_Y(Rd,Gd,Bd);
		*Y2tmp++= RGB24_TO_Y(Rd1,Gd1,Bd1);
		*Cbtmp++ = YB_TO_U(B,Y);
		*Crtmp++ = YR_TO_V(R,Y);
		}
		bufin += image_width;
		bufin1 += image_width;
		Ytmp += image_width;
		Y2tmp += image_width;
	} //end y loop
	return 0;
}
