/****************************************************************************
#	 	GspcaGui:  Gspca/Spca5xx Grabber                            #
# 		Copyright (C) 2004 2005 2006 Michel Xhaard                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#                                                                           #
****************************************************************************/ 
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "../colors/color.h"
#include "encoder.h"

	
/* translate RGB24 to YUV444 in input */
unsigned int
rgb24trPyuv444(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B;
	int Y, Cb, Cr;
	size = image_width * image_height;
	for (i = size; i > 0; i--) {
		B = input_ptr[0];
		G = input_ptr[1];
		R = input_ptr[2];

		//input_ptr -= 3;
		Y = RGB24_TO_Y(R,G,B);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*input_ptr++ = (unsigned char) Y;
		*input_ptr++ = (unsigned char) Cb;
		*input_ptr++ = (unsigned char) Cr;
	}
	return FOUR_FOUR_FOUR;
}


/* translate RGB24 to YUV422 in input */
unsigned int
rgb24trPyuv422(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B, R1, G1, B1;
	int Y, Yp, Cb, Cr;
	unsigned char *inbuf = input_ptr;
	size = image_width * image_height/2;
	for (i = size; i > 0; i--) {
		B = inbuf[0];
		G = inbuf[1];
		R = inbuf[2];
		B1 = inbuf[3];
		G1 = inbuf[4];
		R1 = inbuf[5];
		inbuf += 6;
		Y = RGB24_TO_Y(R,G,B);
		Yp = RGB24_TO_Y(R1,G1,B1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*input_ptr++ = (unsigned char) Y;
		*input_ptr++ = (unsigned char) Cb;
		*input_ptr++ = (unsigned char) Yp;
		*input_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_TWO;
}

/* translate RGB565 to YUV422 in input */
unsigned int
rgb565trPyuv422(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B, R1, G1, B1;
	int Y, Yp, Cb, Cr;
	Myrgb16 *inbuf = (Myrgb16 *) input_ptr;
	size = image_width * image_height /2;
	for (i = size; i > 0; i--) {
		R = inbuf[0].red << 3;
		G = inbuf[0].green << 2;
		B = inbuf[0].blue << 3;
		R1 = inbuf[1].red << 3;
		G1 = inbuf[1].green << 2;
		B1 = inbuf[1].blue << 3;
		inbuf += 2;
		
		Y = RGB24_TO_Y(R,G,B);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		Yp = RGB24_TO_Y(R1,G1,B1);
		
		*input_ptr++ = (unsigned char) Y;
		*input_ptr++ = (unsigned char) Cb;
		*input_ptr++ = (unsigned char) Yp;
		*input_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_TWO;
}
unsigned int
rgb565trPyuv420(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int x, y, size, base;
	unsigned char R, G, B, R1, G1, B1, Rd, Gd, Bd, Rd1, Gd1, Bd1;
	unsigned char Y, Yd, Y11, Yd1, Cb, Cr;
	unsigned short *inbuf = (unsigned short*) input_ptr;
	unsigned short *inbuf1 =(unsigned short*) input_ptr+image_width;
	base = 0;
	for (y=0; y <image_height/2; y++){
	base += 2*image_width;
            for(x=0; x <image_width; x+=2){
	    UNPACK16(inbuf[base+x],R,G,B);
	    UNPACK16(inbuf[base+x+1],R1,G1,B1);
	    UNPACK16(inbuf1[base+x],Rd,Gd,Bd);
	    UNPACK16(inbuf1[base+x+1],Rd1,Gd1,Bd1);
	        Y = RGB24_TO_Y(R,G,B);
		Y11 = RGB24_TO_Y(R1,G1,B1);
		Yd = RGB24_TO_Y(Rd,Gd,Bd);
		Yd1 = RGB24_TO_Y(Rd1,Gd1,Bd1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*input_ptr++ = (unsigned char) Y;
		*input_ptr++ = (unsigned char) Y11;
		*input_ptr++ = (unsigned char) Yd;
		*input_ptr++ = (unsigned char) Yd1;
		*input_ptr++ = (unsigned char) Cb;
		*input_ptr++ = (unsigned char) Cr;
	    }
	
	}
	return FOUR_TWO_ZERO;
}
/* translate RGB24 to YUV422 in input */
unsigned int
rgb32trPyuv422(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B, R1, G1, B1;
	int Y, Yp, Cb, Cr;
	Myrgb32 *inbuf = (Myrgb32 *) input_ptr;
	size = image_width * image_height / 2;
	for (i = size; i > 0; i--) {
		B = inbuf[0].blue;
		G = inbuf[0].green;
		R = inbuf[0].red;
		B1 = inbuf[1].blue;
		G1 = inbuf[1].green;
		R1 = inbuf[1].red;
		inbuf += 2;
		Y = RGB24_TO_Y(R,G,B);
		Yp = RGB24_TO_Y(R1,G1,B1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*input_ptr++ = (unsigned char) Y;
		*input_ptr++ = (unsigned char) Cb;
		*input_ptr++ = (unsigned char) Yp;
		*input_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_TWO;
}

/* translate RGB24 to YUV420 in input */
unsigned int
rgb24trPyuv420(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, j, size;
	unsigned char R, G, B, R1, G1, B1, Rd, Gd, Bd, Rd1, Gd1, Bd1;
	int Y, Yd, Y11, Yd1, Cb, Cr;
	unsigned char *inbuf = input_ptr;
	unsigned char *inbuf1 = input_ptr + (image_width * 3);
	size = image_width * image_height >> 2;
	for (i = size, j = 0; i > 0; i--) {
		B = inbuf[0];
		G = inbuf[1];
		R = inbuf[2];
		B1 = inbuf[3];
		G1 = inbuf[4];
		R1 = inbuf[5];
		Bd = inbuf1[0];
		Gd = inbuf1[1];
		Rd = inbuf1[2];
		Bd1 = inbuf1[3];
		Gd1 = inbuf1[4];
		Rd1 = inbuf1[5];
		inbuf += 6;
		inbuf1 += 6;
		j++;
		if (j >= image_width / 2) {
			j = 0;
			inbuf += (image_width * 3);
			inbuf1 += (image_width * 3);
		}
		Y = RGB24_TO_Y(R,G,B);
		Y11 = RGB24_TO_Y(R1,G1,B1);
		Yd = RGB24_TO_Y(Rd,Gd,Bd);
		Yd1 = RGB24_TO_Y(Rd1,Gd1,Bd1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*input_ptr++ = (unsigned char) Y;
		*input_ptr++ = (unsigned char) Y11;
		*input_ptr++ = (unsigned char) Yd;
		*input_ptr++ = (unsigned char) Yd1;
		*input_ptr++ = (unsigned char) Cb;
		*input_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_ZERO;
}


/* translate RGB32 to YUV420 in input */
unsigned int
rgb32trPyuv420(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, j, size;
	unsigned char R, G, B, R1, G1, B1, Rd, Gd, Bd, Rd1, Gd1, Bd1;
	int Y, Yd, Y11, Yd1, Cb, Cr;
	unsigned char *inbuf = input_ptr;
	unsigned char *inbuf1 = input_ptr + (image_width * 4);
	size = image_width * image_height >> 2;
	for (i = size, j = 0; i > 0; i--) {
		B = inbuf[0];
		G = inbuf[1];
		R = inbuf[2];
		B1 = inbuf[4];
		G1 = inbuf[5];
		R1 = inbuf[6];
		Bd = inbuf1[0];
		Gd = inbuf1[1];
		Rd = inbuf1[2];
		Bd1 = inbuf1[4];
		Gd1 = inbuf1[5];
		Rd1 = inbuf1[6];
		inbuf += 8;
		inbuf1 += 8;
		j++;
		if (j >= image_width / 2) {
			j = 0;
			inbuf += (image_width * 4);
			inbuf1 += (image_width * 4);
		}
		Y = RGB24_TO_Y(R,G,B);
		Y11 = RGB24_TO_Y(R1,G1,B1);
		Yd = RGB24_TO_Y(Rd,Gd,Bd);
		Yd1 = RGB24_TO_Y(Rd1,Gd1,Bd1);
		Cb = YB_TO_U(B,Y);
		Cr = YR_TO_V(R,Y);
		*input_ptr++ = (unsigned char) Y;
		*input_ptr++ = (unsigned char) Y11;
		*input_ptr++ = (unsigned char) Yd;
		*input_ptr++ = (unsigned char) Yd1;
		*input_ptr++ = (unsigned char) Cb;
		*input_ptr++ = (unsigned char) Cr;
	}
	return FOUR_TWO_ZERO;
}


unsigned int
rgb24trPyuv400(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char R, G, B;
	int Y;
	unsigned char *inbuf = input_ptr;
	size = image_width * image_height;
	for (i = size; i > 0; i--) {
		B = inbuf[0];
		G = inbuf[1];
		R = inbuf[2];
		inbuf += 3;
		Y = RGB24_TO_Y(R,G,B);
		*input_ptr++ = (unsigned char) Y;
	}
	return FOUR_ZERO_ZERO;
}


/* translate YUV444P to YUV444 in input */
unsigned int
yuv444PtrPyuv444(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char *Ytmp = NULL;
	unsigned char *Cbtmp = NULL;
	unsigned char *Crtmp = NULL;
	unsigned char *Buff = NULL;
	Buff =
	    (unsigned char *) realloc((unsigned char *) Buff,
			      (size_t) (image_width * image_height * 3));
	if (Buff) {
		memcpy(Buff, input_ptr, image_width * image_height * 3);
		Ytmp = Buff;
		Cbtmp = Buff + image_width * image_height;
		Crtmp = Buff + (image_width * image_height << 1);
		size = image_width * image_height;
		for (i = size; i > 0; i--) {
			*input_ptr++ = (unsigned char) * Ytmp++;
			*input_ptr++ = (unsigned char) * Cbtmp++;
			*input_ptr++ = (unsigned char) * Crtmp++;
		}
		free(Buff);
		Buff = NULL;
	}
	return FOUR_FOUR_FOUR;
}


/* translate YUV422P to YUV422 in input */
unsigned int
yuv422PtrPyuv422(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int i, size;
	unsigned char *inbuf = input_ptr;
	unsigned char *Ytmp = NULL;
	unsigned char *Cbtmp = NULL;
	unsigned char *Crtmp = NULL;
	unsigned char *Buff = NULL;
	Buff =
	    (unsigned char *) realloc((unsigned char *) Buff,
			      (size_t) (image_width * image_height * 2));
	if (Buff) {
		memcpy(Buff, input_ptr, image_width * image_height * 2);
		Ytmp = Buff;
		Cbtmp = Buff + image_width * image_height;
		Crtmp = Cbtmp + (image_width * image_height >> 1);
		size = image_width * image_height /2;
		for (i = size; i > 0; i--) {
			*input_ptr++ = (unsigned char) * Ytmp++;
			*input_ptr++ = (unsigned char) * Cbtmp++;
			*input_ptr++ = (unsigned char) * Ytmp++;
			*input_ptr++ = (unsigned char) * Crtmp++;
		}
		free(Buff);
		Buff = NULL;
	}
	return FOUR_ZERO_ZERO;
}


/* translate YUV420P to YUV420 in input */
unsigned int
yuv420PtrPyuv420(unsigned char * input_ptr, unsigned int image_width, unsigned int image_height)
{
	unsigned int x, y, size;
	unsigned char *inbuf = input_ptr;
	unsigned char *Ytmp = NULL;
	unsigned char *Y2tmp = NULL;
	unsigned char *Cbtmp = NULL;
	unsigned char *Crtmp = NULL;
	unsigned char *Buff = NULL;
	Buff =
	    (unsigned char *) realloc((unsigned char *) Buff,
			      (size_t) ((image_width * image_height *
					 3) >> 1));
	if (Buff) {
		memcpy(Buff, input_ptr,
		       (image_width * image_height * 3) >> 1);
		Ytmp = Buff;
		Y2tmp = Buff + image_width;
		Cbtmp = Buff + image_width * image_height;
		Crtmp = Cbtmp + (image_width * image_height >> 2);
		size = image_width * image_height >> 2;
		for (y = 0; y < image_height; y += 2) {
			for (x = 0; x < image_width; x += 2) {
				*input_ptr++ = (unsigned char) * Ytmp++;
				*input_ptr++ = (unsigned char) * Ytmp++;
				*input_ptr++ = (unsigned char) * Y2tmp++;
				*input_ptr++ = (unsigned char) * Y2tmp++;
				*input_ptr++ = 128;//(unsigned char) * Cbtmp++;
				*input_ptr++ = 128;//(unsigned char) * Crtmp++;
			}
			Ytmp += image_width;
			Y2tmp += image_width;
		}
		free(Buff);
		Buff = NULL;
	}
	return FOUR_TWO_ZERO;
}
//dummy transform
unsigned int Pyuv400trPyuv400(unsigned char * input_ptr, unsigned int image_width,
		 unsigned int image_height)
{
return FOUR_ZERO_ZERO;
}
unsigned int Pyuv420trPyuv420(unsigned char * input_ptr, unsigned int image_width,
		 unsigned int image_height)
{
return FOUR_TWO_ZERO;
}
unsigned int Pyuv422trPyuv422(unsigned char * input_ptr, unsigned int image_width,
		 unsigned int image_height)
{
return FOUR_TWO_TWO;
}
unsigned int Pyuv444trPyuv444(unsigned char * input_ptr, unsigned int image_width,
		 unsigned int image_height)
{
return FOUR_FOUR_FOUR;
}
