/****************************************************************************
#	 	SpcaGui:  Spca5xx Grabber                                   #
# 		Copyright (C) 2004 Michel Xhaard                            #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#                                                                           #
****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <wait.h>
#include <limits.h>

#include "gui_internal.h"
#include "decoder/decoder.h"
#include "converter/cvrt.h"
#include "osd/font.h"


#define FONT 8
#define INITSP 2
#define PFONT 12
#define DEBOUNCE 50

int bpp = 3;
//#define PATHSPCA "icons/"
#define PATHSPCA "/usr/local/share/gspcagui/"

static const char curseur[] = PATHSPCA "curseur.jpg";
static const char colorpot[] = PATHSPCA "colorpot.jpg";
static const char v3pot[] = PATHSPCA "colorpotV3.jpg";

static const char photo1[] = PATHSPCA "photo.jpg";
static const char pictrec[] = PATHSPCA "pictrec.jpg";
static const char clap1[] = PATHSPCA "clap1.jpg";
static const char avi2[] = PATHSPCA "avi2.jpg";
static const char pause1[] = PATHSPCA "pause.jpg";
static const char recp[] = PATHSPCA "recp.jpg";
static const char play[] = PATHSPCA "soundoff.jpg";
static const char restart[] = PATHSPCA "soundoff.jpg";
static const char quitter[] = PATHSPCA "quitter.jpg";
static const char rgb[] = PATHSPCA "rgb.jpg";
static const char bgr[] = PATHSPCA "bgr.jpg";

static const char ctrlsize[] = PATHSPCA "ctrlsize.jpg";
static const char ctrlpal[] = PATHSPCA "ctrlpal.jpg";
static const char pvalid[] = PATHSPCA "pvalid.jpg";
static const char pactif[] = PATHSPCA "pactif.jpg";
static const char pvide[] = PATHSPCA "pvide.jpg";

static const char padcontrol[] = PATHSPCA "rotate.jpg";
static const char padup[] = PATHSPCA "rotateUp.jpg";
static const char paddown[] = PATHSPCA "rotateDown.jpg";
static const char padleft[] = PATHSPCA "rotateLeft.jpg";
static const char padright[] = PATHSPCA "rotateRight.jpg";
static const char padcenter[] = PATHSPCA "rotateCenter.jpg";

static const char togleup[] = PATHSPCA "togleUp.jpg";
static const char togledown[] = PATHSPCA "togleDown.jpg";
static const char togle[] = PATHSPCA "togle.jpg";
static const char togleall[] = PATHSPCA "togleAll.jpg";

static const char functon[] = PATHSPCA "on.jpg";
static const char player[] = PATHSPCA "player.jpg";
static const char grabber[] = PATHSPCA "grabber.jpg";
static const char client[] = PATHSPCA "client.jpg";
static const char info[] = PATHSPCA "ecran.jpg";

static const char labelsize[][5]= {
"XVGA","SVGA","VGA", "PAL","SIF","CIF","QSIF","QCIF"
};
static const char labelpal[][5] = {
"MJPG","YUYV","I420","R24","R565","R32"
};
static const char labelcont[][12] = {
"CONTRAST"
};
static const char labelbright[][12] = {
"BRIHTNESS"
};
static const char labelcol[][12] = {
"COLORS"
};
static const char labelnoise[][12] = {
"DENOISE"
};
static const char labelsharp[][12] = {
"SHARP"
};

typedef struct icon {
	int action;
	const char *filin; 
	const char *filout;
	const char (*label)[5];
	const char (*labelpot)[12];
} icon;

static icon icon_path[] = {
    {PHOTO,photo1,pictrec},
    {AVI,clap1,avi2},
    {PAUSE,pause1,recp},
    {QUIT,quitter,quitter},
    {FLIPRGB,rgb,bgr},
    {BRIGHT,colorpot,NULL,NULL,labelbright},
    {CONTRAST,colorpot,NULL,NULL,labelcont},
    {COLOR,colorpot,NULL,NULL,labelcol},
    {SIZE,ctrlsize,NULL,labelsize},
    {PALETTE,ctrlpal,NULL,labelpal},
    {SOUND,play,restart},
    {ROTATE,padcontrol,NULL},
    {DENOISE,v3pot,NULL,NULL,labelnoise},
    {SHARP,v3pot,NULL,NULL,labelsharp},
    {FRMRATE,togle,NULL},
    {QUALITY,togle,NULL,NULL},
    {AUTO,togle,NULL,NULL},
    {PLAYER,player,functon},
    {GRABBER,grabber,functon},
    {CLIENT,client,functon},
    {INFO,info,NULL},
    {0,NULL,NULL}
};

static void
draw_surface (int x, int y, SDL_Surface * surface, SDL_Surface * screen)
{
  SDL_Rect dest;
  dest.x = x;
  dest.y = y;
  dest.w = surface->w;
  dest.h = surface->h;
  SDL_BlitSurface (surface, NULL, screen, &dest);
  SDL_UpdateRect(screen,dest.x,dest.y,dest.w,dest.h);

}

static int file_size (FILE * f) 
{
int save_pos, size_of_file;
save_pos = ftell (f);
fseek (f, 0L, SEEK_END);
size_of_file = ftell (f);
fseek (f, save_pos, SEEK_SET);
return size_of_file;
}

SDL_Surface *imgload (const char* pathfile)
{
	SDL_Surface *Object;
	FILE *in_file;
	int width, height, depth, buffer_size;
	int err;
	unsigned int RM = 0;
	unsigned int GM = 0;
	unsigned int BM = 0;
	unsigned int AM = 0;
	unsigned char *inp = NULL;
	unsigned char *pp =NULL;
	in_file = fopen (pathfile , "rb");
	buffer_size = file_size (in_file);
	inp = (unsigned char *) malloc (buffer_size);
  	if (inp == NULL){
    	  error ("not enough memory");
	  fclose(in_file);
	  return NULL;
	  }
	  /* load whole file into the memory */
  	fread (inp, 1, buffer_size, in_file);
	fclose(in_file);
	width = 0; height = 0 ;
	if((err = jpeg_decode(&pp,inp,&width,&height)) < 0){
	error ("decoder jpeg fail");
	free(inp);
	if(pp) free(pp);
	return NULL;
	}
	free(inp);
	Object = SDL_CreateRGBSurface(SDL_SWSURFACE,width,height,bpp*8,RM,GM,BM,AM);
	Pyuv422torgb24(pp,Object->pixels,width,height);
	free(pp);
	return (Object);
}
void
init_info(int id,int x, int y, struct ecran *myecran)
{
	const char *release; 
	int i; int length;
	myecran->id = id;
	myecran->x = x;
	myecran->y = y;
	i = 0;
	while (icon_path[i].action != id){
	if (icon_path[i].action == 0){
	printf(" Button action not find \n");
	 return;
	 }
	 i++;
	}
	release = icon_path[i].filin;
	myecran->fond = imgload(release);
	myecran->orig = imgload(release);
	length = myecran->fond->w / FONT;
	myecran->length = length;
	myecran->stinfo = calloc(1,length);
	
}
void
draw_info (struct ecran *myecran,SDL_Surface *Screen)
{

 draw_surface (myecran->x, myecran->y, myecran->fond, Screen);
}
void
free_info(struct ecran *myecran)
{
SDL_FreeSurface (myecran->fond);
SDL_FreeSurface (myecran->orig);
free(myecran->stinfo);
}
void
process_info( char *chaine ,struct ecran *myecran,SDL_Surface *Screen)
{
	memcpy(myecran->fond->pixels,myecran->orig->pixels,myecran->fond->w*myecran->fond->h*bpp);
	snprintf(myecran->stinfo,myecran->length,"%s",chaine);
 if (myecran->stinfo && myecran->length)
 printscreen(myecran->fond->pixels,6,myecran->fond->h/2,FONT,myecran->stinfo,myecran->fond->w,myecran->fond->h,0,0);
 draw_info (myecran,Screen);
}

void
init_button (int id, int value, int x, int y, struct button *mybutt)
{
	     const char *release; 
	     const char *press;
	     int i = 0;
				/* init pot */
	while (icon_path[i].action != id){
	if (icon_path[i].action == 0){
	printf(" Button action not find \n");
	 return;
	 }
	 i++;
	}
	release = icon_path[i].filin;
	press = icon_path[i].filout;
  mybutt->id = id;
  mybutt->x = x;
  mybutt->y = y;
  mybutt->toggle = value;
  if (release)
    mybutt->iconrelease = imgload (release);
  if (press)
    mybutt->iconpress = imgload (press);
}
void
refresh_button (int id, int value, struct button *mybutt, SDL_Surface * Screen )
{ 
	mybutt->toggle = value;
	draw_button(mybutt,Screen);
}
void
free_button (struct button *mybutt)
{
  SDL_FreeSurface (mybutt->iconrelease);
  SDL_FreeSurface (mybutt->iconpress);
}

void
draw_button (struct button *mybutt, SDL_Surface * Screen)
{
  if (mybutt->toggle)
    {
      draw_surface (mybutt->x, mybutt->y, mybutt->iconpress, Screen);
    }
  else
    {
      draw_surface (mybutt->x, mybutt->y, mybutt->iconrelease, Screen);
    }

}

void
process_button (int value, struct button *mybutt, SDL_Surface * Screen,
		unsigned char *message)
{

  mybutt->toggle = !mybutt->toggle;
  SDL_Delay (DEBOUNCE);
  draw_button (mybutt, Screen);
  message[0] = mybutt->id;
  message[1] = mybutt->toggle;

  if (!value)
    {
      SDL_Delay (500);
      mybutt->toggle = !mybutt->toggle;
      draw_button (mybutt, Screen);
    }
printf("processing button %d \n",mybutt->id);
}

void
init_potentiometre (int id, int value, int x, int y,
		    struct potentiometre *mypot)
{
int val;
  int curheight ;
  int heighttot ;
 const char *fond;
 int len,k,startx,starty;
 int incy;
 int i = 0;
		/* init pot */
	while (icon_path[i].action != id){
	if (icon_path[i].action == 0){
	printf(" pot action not find \n");
	 return;
	 }
	 i++;
	}
	fond = icon_path[i].filin;	
	/* init pot */
  mypot->id = id;
  mypot->x = x;
  mypot->y = y;
  mypot->value = value;
  mypot->fond = imgload (fond);
  mypot->curseur = imgload (curseur);
 //point haut
  curheight = mypot->y+mypot->curseur->h;
 //point bas
  heighttot = mypot->fond->h+mypot->y- mypot->curseur->h;
  // Amplitude 
  val= (heighttot -curheight)*(value)/255;
  mypot->value = val;
  /* set the graphical value */
  val =  heighttot - val;  
  mypot->y0 = val;
if ( icon_path[i].labelpot){
  startx = (mypot->fond->w - PFONT)/2;
  starty = mypot->curseur->h;//2* mypot->curseur->h;
  len = strlen(icon_path[i].labelpot[0]);
  snprintf(mypot->label,12,"%s",icon_path[i].labelpot[0]);
//  printf("len %d %s\n",len,mypot->label);
  incy=((mypot->fond->h - starty) - (len*PFONT))/(len+1)+PFONT;
  for(k=0;k <len;k++){
  printchar ( mypot->fond->pixels,startx,starty,PFONT,mypot->label[k],mypot->fond->w, 0,0);
  starty += incy;//(2*PFONT);
 if(starty >(mypot->fond->h-2*PFONT)) break;
  }
}
}

void
draw_potentiometre (struct potentiometre *mypot, SDL_Surface * Screen)
{ 
  /* first draw bakground */
  draw_surface (mypot->x, mypot->y, mypot->fond, Screen);
  /* then cursor */
  draw_surface (mypot->x, mypot->y0-mypot->curseur->h/2, mypot->curseur, Screen);
}

void
refresh_potentiometre (int id, int value,struct potentiometre *mypot , SDL_Surface * Screen )
{ 
  int val;
  int curheight ;
  int heighttot ;
  heighttot = mypot->fond->h+mypot->y- mypot->curseur->h;
  curheight = mypot->y+mypot->curseur->h;
  val= (heighttot -curheight)*(value)/255;
  mypot->value = val;
  /* set the graphical value */
  val =  heighttot - val;  
  mypot->y0 = val;
  draw_potentiometre (mypot, Screen);
}

void
free_potentiometre (struct potentiometre *mypot)
{
  SDL_FreeSurface (mypot->fond);
  SDL_FreeSurface (mypot->curseur);
}



void
process_potentiometre (int yp, struct potentiometre *mypot,
		       SDL_Surface * Screen, unsigned char *message)
{ /*  */
  int val;
  int curheight;;
  int heighttot ;
  SDL_Delay (DEBOUNCE);
  /* convert graphique to value in the good range */
  /* graphique tot = heighttot-2*curheight */
  /* value 0..255 */
 //point haut
  curheight = mypot->y+mypot->curseur->h;
 //point bas
  heighttot = mypot->fond->h + mypot->y - mypot->curseur->h;
   if(yp < curheight || yp > heighttot){
   	message[0] = NOTHING;
  	message[1] = NOTHING;
  	return;
   }
   mypot->y0 = yp;
   yp -= curheight;  
  // Amplitude 
  val= ((heighttot -curheight)-yp)*255/(heighttot -curheight);
 // printf ("Oh pot id %d val %d /n",mypot->id,val);
  mypot->value = val;
  /* set the graphical value */
  draw_potentiometre (mypot, Screen);
  message[0] = mypot->id;
  message[1] = mypot->value;
}

void
init_control (int id, int value, int nb, int x, int y,
	      struct control *mycontrol)
{
  const char *myfond;

  int len;
  unsigned char masq = 0x01;
  int i = 0,k;
  int val,space;
  int nofont = 0;
  while (icon_path[i].action != id){
	if (icon_path[i].action == 0){
	printf(" pot action not find \n");
	 return;
	 }
	 i++;
	}
	myfond = icon_path[i].filin;
  mycontrol->id = id;
  mycontrol->nbpuce = nb;
  mycontrol->x = x;
  mycontrol->y = y;
  mycontrol->offset = 0;
  mycontrol->available = value;
  mycontrol->pucevalid = imgload (pvalid);
  mycontrol->puceactive = imgload (pactif);
  mycontrol->pucevide = imgload (pvide);
  mycontrol->fond = imgload (myfond);
  if( (nb * mycontrol->pucevide->h + mycontrol->offset) > mycontrol->fond->h) {
  	printf("Cannot set %d puces in control %d id\n", nb,id);
	return;
  }
  
  /*setting up the label */
  if ( icon_path[i].label){
  nofont = 1;
  for(k=0 ; k < nb; k++){ 
  snprintf(mycontrol->label[k],5,"%s",icon_path[i].label[k]);
  }
  }
  /* compute available h */
  val = mycontrol->fond->h ;
  /* compute space */
  space = (val - nb * mycontrol->pucevide->h)/(nb+1);  
  mycontrol->incpuce = space+ mycontrol->pucevide->h;
  mycontrol->offset = space;
  if(nofont) {
  k = mycontrol->incpuce;  
    for (i = 0; i < nb; i++){
    	len = strlen(mycontrol->label[i]);
	space = (mycontrol->fond->w-FONT*len)/2;
  	printscreen(mycontrol->fond->pixels,space,k+INITSP,FONT,mycontrol->label[i],mycontrol->fond->w,mycontrol->fond->h,0,0);
	k += mycontrol->incpuce;
  }
  }
  /* find a default mode */
  for (i = 0; i < 8; i++)
    {
      if ((value & masq) > 0)
	{
	  mycontrol->courant = masq;
	  //printf("courant %d\n",mycontrol->courant);
	  break;
	}
      masq = masq << 1;

    }

}

void
free_control (struct control *mycontrol)
{
  SDL_FreeSurface (mycontrol->fond);
  SDL_FreeSurface (mycontrol->pucevalid);
  SDL_FreeSurface (mycontrol->puceactive);
  SDL_FreeSurface (mycontrol->pucevide);
}

void
draw_control (struct control *mycontrol, SDL_Surface * Screen)
{
  unsigned char masq = 0x01;
  int end_h, posx, posy;
  int starty = mycontrol->offset;
  int i;
  /* first draw bakgraound */
  draw_surface (mycontrol->x, mycontrol->y, mycontrol->fond, Screen);

  posy = mycontrol->y + starty;
  end_h = mycontrol->y+mycontrol->fond->h -mycontrol->pucevide->h;	
  posx = mycontrol->x + (HOTSURFACE - mycontrol->pucevide->w)/2;
  
  for (i = 0; i < mycontrol->nbpuce; i++)
    {
      if ((masq & mycontrol->courant) > 0)
	{
	  draw_surface (posx, posy, mycontrol->puceactive, Screen);
	}
      else if ((masq & mycontrol->available) > 0)
	{
	  draw_surface (posx, posy, mycontrol->pucevalid, Screen);
	}
      else
	{
	  draw_surface (posx, posy, mycontrol->pucevide, Screen);
	}
      posy += mycontrol->incpuce;
      if (posy >= end_h)
	break;
      masq = masq << 1;

    }
}

void
process_control (int yp, struct control *mycontrol, SDL_Surface * Screen,
		 unsigned char *message)
{
  unsigned char masq = 0x01;
  unsigned char maxmasq = 0x01;

  int i, yn;
  int posy, posy1, posx;
  int starty = mycontrol->offset ;
  int nb = mycontrol->nbpuce;
  /* debounce delay */
  SDL_Delay (DEBOUNCE);
  /* initialize */
  posx = mycontrol->x + (HOTSURFACE - mycontrol->pucevide->w)/2;
  /* find courant */
  for (i = 0; i < mycontrol->nbpuce; i++)
    {
      if ((mycontrol->courant & maxmasq) > 0)
	break;
      maxmasq = maxmasq << 1;
    }
  posy = (mycontrol->y) + starty + (i * mycontrol->incpuce);
  /* test if mouse on actif return */
  yn = (yp - (mycontrol->y + starty)) / (mycontrol->incpuce);
  printf ("y value: %d \n",yn);
  posy1 = mycontrol->y + starty + (yn * mycontrol->incpuce);
  masq = masq << yn;
  /* if nothing todo return 0 */
  message[0] = NOTHING;
  message[1] = NOTHING;
  if (mycontrol->courant == masq)
    return;
  /* if mouse not actif are they valid */
  if ((mycontrol->available & masq) > 0)
    {
      /* actualize */
      mycontrol->courant = masq;
      message[0] = mycontrol->id;
      message[1] = mycontrol->courant;
      draw_surface (posx, posy, mycontrol->pucevalid, Screen);
      draw_surface (posx, posy1, mycontrol->puceactive, Screen);
    }
  else
    {
      /*nothing todo go back */
      return;
    }
}
#if 0
/* function not on even context */
void
refresh_control (int value, struct control *mycontrol, SDL_Surface * Screen,
		 unsigned char *message)
{
  unsigned char masq = 0x01;
  int i;
  mycontrol->available = value;
  if (!(mycontrol->courant & value))
    {
      /* find a new match */
      for (i = 0; i < 8; i++)
	{
	  if ((value & masq) > 0)
	    {
	      mycontrol->courant = masq;
	      //printf("courant %d\n",mycontrol->courant);
	      break;
	    }
	  masq = masq << 1;

	}
    }
  draw_control (mycontrol, Screen);
  message[0] = mycontrol->id;
  message[1] = mycontrol->courant;
}
#endif
/* function not on even context */
void
refresh_control (int value, int masq, struct control *mycontrol, SDL_Surface * Screen)
{
  mycontrol->available = value;
  mycontrol->courant = masq;
  draw_control (mycontrol, Screen);
 
}
void init_pad( int id, int value,int x,int y,
		struct pad *mypad )
{
const char *myfond;
int i = 0;
 while (icon_path[i].action != id){
	if (icon_path[i].action == 0){
	printf(" pot action not find \n");
	 return;
	 }
	 i++;
	}
	myfond = icon_path[i].filin;
  mypad->id = id;
  mypad->x = x;
  mypad->y = y;
  mypad->state = value & 0xFF;
  mypad->fond= imgload (myfond);
  mypad->padup=imgload(padup);
  mypad->paddown =imgload(paddown);
  mypad->padright=imgload(padright);
  mypad->padleft=imgload(padleft);
  mypad->padcenter=imgload(padcenter);
}
void free_pad( struct pad *mypad )
{ 
SDL_FreeSurface (mypad->fond);
SDL_FreeSurface (mypad->padup);
SDL_FreeSurface (mypad->paddown);
SDL_FreeSurface (mypad->padright);
SDL_FreeSurface (mypad->padleft);
SDL_FreeSurface (mypad->padcenter);
}
void 
draw_pad (struct pad *mypad,SDL_Surface *Screen)
{
draw_surface (mypad->x, mypad->y, mypad->fond, Screen);
}
void 
process_pad (int valx, int valy ,struct pad *mypad,
		SDL_Surface *Screen, unsigned char *message)
{
	int x,y;
	int xsample,ysample,pos;
	
	#define PSTAT 3
	x = valx - mypad->x;
	y = valy - mypad->y;
	if (x < 0 || y < 0 || x > mypad->fond->w || y > mypad->fond->h)
		goto nothing;
	/* ----------------------
	-	-	-	-
	-64	-0	-8	-
	-------------------------
	-	-	-	-
	-4	-128	-1	-
	-------------------------
	-	-2	-	-
	-32	-	-16	-
	-----------------------*/
	xsample=mypad->fond->w/PSTAT;
	ysample=mypad->fond->h/PSTAT;
	/*compute the position */
	pos = x/xsample+(y/ysample)*PSTAT;
	switch (pos){
	case 0: mypad->state = 64; break;
	case 1:mypad->state = 0;
	draw_surface (mypad->x, mypad->y, mypad->padup, Screen);
	 break;
	case 2:mypad->state = 8; break;
	case 3:mypad->state = 4;
	draw_surface (mypad->x, mypad->y, mypad->padleft, Screen);
	 break;
	case 4:mypad->state = 128; 
	draw_surface (mypad->x, mypad->y, mypad->padcenter, Screen);
	break;
	case 5:mypad->state = 1;
	draw_surface (mypad->x, mypad->y, mypad->padright, Screen); 
	break;
	case 6:mypad->state = 32; break;
	case 7:mypad->state = 2; 
	draw_surface (mypad->x, mypad->y, mypad->paddown, Screen);
	break;
	case 8:mypad->state = 16; break;
	};
	SDL_Delay (500);
	draw_pad(mypad,Screen);
  message[0] = mypad->id;
  message[1] = mypad->state;
  return;
nothing:
  message[0] = NOTHING;
  message[1] = NOTHING;
}
/* On init value bit 4 0 switch are On/off 
default up bit 0 down bit 1 
bit 4 set to 1 switch one shot */

void init_togle( int id, int value, int x,int y,
		struct toglesw *mytogle )
{
const char *myfond;
int i = 0;
 while (icon_path[i].action != id){
	if (icon_path[i].action == 0){
	printf(" pot action not find \n");
	 return;
	 }
	 i++;
	}
	myfond = icon_path[i].filin;
  mytogle->id = id;
  mytogle->x = x;
  mytogle->y = y;
  mytogle->mode = ((value & 0x10) >> 4);
  printf("toggle init id %d, mode %d\n",mytogle->id,mytogle->mode);
  if( mytogle->mode){
  mytogle->valup = (value & 0x01);
  mytogle->valdown = (value &0x02);
  } else {
  mytogle->valup = 0;
  mytogle->valdown = 0;
  }
  mytogle->fond= imgload (myfond);
  mytogle->togleup=imgload(togleup);
  mytogle->togledown =imgload(togledown);
  mytogle->togleall=imgload(togleall);
 
}
void free_togle( struct toglesw *mytogle )
{
SDL_FreeSurface (mytogle->fond);
SDL_FreeSurface (mytogle->togleup);
SDL_FreeSurface (mytogle->togledown);
SDL_FreeSurface (mytogle->togleall);
}
void 
draw_togle (struct toglesw *mytogle,SDL_Surface *Screen)
{
printf("toggle id %d, mode %d\n",mytogle->id,mytogle->mode);

if(!(mytogle->valup | mytogle->valdown)){
 draw_surface (mytogle->x, mytogle->y, mytogle->fond, Screen);
 return;
 }
 if(mytogle->valup && mytogle->valdown) {
draw_surface (mytogle->x, mytogle->y, mytogle->togleall, Screen);
return;
}
if(mytogle->valup) {
draw_surface (mytogle->x, mytogle->y, mytogle->togleup, Screen);
return;
}
if(mytogle->valdown)
draw_surface (mytogle->x, mytogle->y, mytogle->togledown, Screen);

}
void 
process_togle (int y ,struct toglesw *mytogle,
		SDL_Surface *Screen, unsigned char *message)
{
int yint, i;
int ysample;
#define TSTAT 2
yint = y - mytogle->y;
if( yint < 0 || yint > mytogle->fond->h)
goto nothing;
ysample = mytogle->fond->h/TSTAT;
if (yint > ysample) {
	if(mytogle->mode){
	if(mytogle->valdown & 0x02)
		mytogle->valdown &= 0xFD;
	else mytogle->valdown |= 0x02;
	} else {
	mytogle->valup = 0;
	mytogle->valdown =2;
	}
/* down */
} else {
/* up */
	if(mytogle->mode){
	if(mytogle->valup & 0x01)
		mytogle->valup &= 0xFE;
	else mytogle->valup |= 0x01;
	} else {
	mytogle->valup = 0x01;
	mytogle->valdown =0;
	}
}
draw_togle(mytogle,Screen);
if (! mytogle->mode){
  SDL_Delay(200);
  draw_surface (mytogle->x, mytogle->y, mytogle->fond, Screen);
}
  message[0] = mytogle->id;
  message[1] = mytogle->valup | mytogle->valdown;
  return;
nothing:
  message[0] = NOTHING;
  message[1] = NOTHING;
}
action_gui
witchButton (int x, int y)
{ 
  int pos ;
  int posx = x /HOTSURFACE;
  int posy = y /HOTSURFACE;
//  printf("Xpos %d Ypos %d \n",posx,posy);
if ( posx > HOTSURFACE*NBX  || posy > HOTSURFACE*NBY) 
	return NOTHING;
  pos = NBX * posy + posx ;
  switch (pos) {
  	case 0:
		return PHOTO;
	break;
	case 1:
		return AVI;
	break;
	case 2:
		return PAUSE;
	break;
	case 3:
		return FLIPRGB;
	break;
	case 4:
		return QUIT;
	break;
	case 5: case 10: case 15: case 20: case 25:
		return BRIGHT;
	break;
	case 6: case 11: case 16: case 21: case 26:
		return CONTRAST;
	break;
	case 7: case 12: case 17: case 22: case 27:
		return COLOR;
	break;
	case 8: case 13: case 18: case 23: case 28:
		return SIZE;
	break;
	case 9: case 14: case 19: case 24:
		return PALETTE;
	break;
	case 29:
	 	return SOUND;
	break;
	case 31: case 32: case 33: case 36: case 37: case 38:
	return ROTATE;
	break;
	case 30: case 35: case 40:
	 return DENOISE;
	break;
	case 34: case 39: case 44:
	 return SHARP;
	break;
	case 41:
	 return FRMRATE;
	break;
	case 42:
	 return QUALITY;
	break;
	case 43:
	 return AUTO;
	break;
	case 45:
	 return GRABBER;
	break;
	case 46:
	 return PLAYER;
	break;
	case 47:
	 return CLIENT;
	break;
	case 48: case 49:
	return INFO;
	break;
	default:
		return NOTHING;
	break;
  }
}

action_gui
witchKey (SDLKey key)
{
}
