/****************************************************************************
#	 	GspcaGui:  Gspca/Spca5xx Grabber                            #
# 		Copyright (C) 2004 2005 2006 Michel Xhaard                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#                                                                           #
****************************************************************************/ 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>

#include "libv4l.h"

#define __u8 unsigned char

void exit_fatal(char *messages);
static int init_v4l(videoIn_t *vd );
static int uninit_v4l(videoIn_t *vd);
static int realloc_buffer(videoIn_t *vd );
static int destroy_buffer(videoIn_t *vd );
static int GetVideoPict (videoIn_t *vd);
static int SetVideoPict (videoIn_t *vd);
static int uninit_v4l1(videoIn_t *vd);
static int init_v4l1(videoIn_t *vd );
static int alloc_v4l1_bufferIn(videoIn_t *vd);
static int dealloc_v4l1_bufferIn(videoIn_t *vd);
static int probeHardCap(videoIn_t *vd );
static int GetDepth (int format);
static int convertIsize( int width, int height);
static int convertsize( int width, int height);
static int sizeIconvert( int *width, int *height, int Isize);
static int sizeconvert( int *width, int *height, int size);
static int convertpalette( int palette);
static int paletteconvert( int code);
static int paletteIconvert( int code);
static void spcaPrintParam (int fd, struct video_param *videoparam);
static void spcaSetTimeInterval(int fd, struct video_param *videoparam, unsigned short time);
static void spcaSetQuality(int fd, struct video_param *videoparam, unsigned char index);
static int init_v4l2(videoIn_t *vd );
static int uninit_v4l2(videoIn_t *vd);;
static int nearest(int masq ,int val);

ctrlop opt[];

__u8 v4l1_control( videoIn_t * vdin, int ctrl,__u8 param){
  return opt[ctrl].fctrl(vdin,param);
}

videoIn_t *openvideo(const char *path)
{
int len;
videoIn_t *vd = NULL;
		if (!path)
			return NULL;
		vd = (videoIn_t *) malloc(sizeof(videoIn_t));
		if(!vd)
			return NULL;
		if((vd->fd = open(path,O_RDWR)) < 0) {
			fprintf(stderr,"open failed \n");
			free(vd);
			return NULL;
		}
		memset(&vd->indev,0,sizeof(vd->indev));
		len = strlen(path);
		vd->videodevice = (char *)malloc(len+1);
		snprintf(vd->videodevice,len,"%s",path);
		vd->this = vd;
		if(init_v4l( vd )<0){
		       free(vd->videodevice);
		       vd->videodevice = NULL;
		       close(vd->fd);
		       free(vd);
		       vd = NULL;
		}
		return (vd); 
} 

int closevideo(videoIn_t *vd)
{
int err;
	if((err = close(vd->fd)) < 0){
	  fprintf(stderr,"error on close %d ,errno %d \n",err,errno);
	  return err;
	}
	uninit_v4l(vd);
	if(vd->cameraname)
	       free(vd->cameraname);
	if(vd->videodevice)
	        free(vd->videodevice);
	vd->cameraname = NULL;
	vd->videodevice = NULL;
	free(vd);
	vd = NULL;
return 0;	
}
static int init_v4l(videoIn_t *vd )
{
    int ret;
    int err;
    int i;
    vd->v4l2 = &vd->indev.v4l2;
    /* try v4l2 */
    ret = ioctl(vd->fd, VIDIOC_QUERYCAP, &vd->indev.v4l2.cap);
    if (ret < 0) {
	printf("Error opening device %s: unable to query v4l2 device.\n",
	       vd->videodevice);
	vd->v4l2 = NULL;
	vd->v4l1 = &vd->indev.v4l1;
	/* try v4l1 */
	if (ioctl (vd->fd, VIDIOCGCAP, &(vd->indev.v4l1.videocap)) == -1){
		printf("Fatal Error v4l1 vidiocgcap \n");
		vd->v4l1 = NULL;
		return -EINVAL;
	} else 
		printf("get open v4l1 device  %p  , v4l2 %p \n",vd->v4l1,vd->v4l2); 
    }

if( vd->v4l2){
/* v4l2 stuff here */
	err = init_v4l2(vd);
} else {
	err = init_v4l1(vd);
	printf(" v4l1 init done \n");
}
if (err <0)
	return -EINVAL;
vd->streamon = 0;
vd->readon = 0;
/* alloc the users buffer */
for (i = 0; i < NBUFFER; i++){
	vd->framebuffer[i].data = NULL;
}
if(realloc_buffer(vd) < 0)
	return -EINVAL;
printf(" v4l1 buffer alloc done \n");

return 0;
}

static int uninit_v4l(videoIn_t *vd)
{
int err;
    streamstop(vd);
    if(destroy_buffer(vd) < 0)
	printf("destroy buf fail !!\n");
    if( vd->v4l2){
   /* v4l2 stuff here */
	err = uninit_v4l2(vd);
    } else {
	err = uninit_v4l1(vd);
    }
   return err;	
}
/* be sure to stop the reader process with all mutex unlocked */
static int realloc_buffer(videoIn_t *vd ){
int i;
int depth;
printf("enter read buffer realloc \n");
if((depth = GetDepth(vd->formatIn)) < 0)
	return -1;
	/* FIXME if streamon close stream */
if(vd->streamon || vd->readon) 
	return -1;
printf("read buffer realloc stream %d\n",vd->streamon);	
for (i = 0; i < NBUFFER; i++){
	vd->framebuffer[i].w= vd->hdrwidth;
	vd->framebuffer[i].h= vd->hdrheight;
	vd->framebuffer[i].size = (vd->hdrwidth*vd->hdrheight*depth) >> 3;
	vd->framebuffer[i].formatIn = vd->formatIn;
	printf("read buffer %d  realloc size %d , buffer (%p) \n",i,vd->framebuffer[i].size,vd->framebuffer[i].data);
	if(vd->framebuffer[i].data) {
		free(vd->framebuffer[i].data);
		vd->framebuffer[i].data = NULL;
		if(pthread_mutex_destroy(&vd->framebuffer[i].grabmutex))
			printf ("release mutex failed \n");
		}
		vd->framebuffer[i].readlck = 0;
	vd->framebuffer[i].data = (unsigned char*) malloc((size_t) vd->framebuffer[i].size);
	pthread_mutex_init (&vd->framebuffer[i].grabmutex, NULL);
	
}
printf("read buffer realloc done \n");
return 0;
}
static int destroy_buffer(videoIn_t *vd ){
int i;
/* FIXME if streamon close stream */
	if(vd->streamon || vd->readon) 
	return -1;
for (i = 0; i < NBUFFER; i++){
	vd->framebuffer[i].w= 0;
	vd->framebuffer[i].h= 0;
	vd->framebuffer[i].size = 0;
	vd->framebuffer[i].formatIn= 0;
	vd->framebuffer[i].readlck = 0;
	if(vd->framebuffer[i].data) {
		free(vd->framebuffer[i].data);
		vd->framebuffer[i].data = NULL;
		if(pthread_mutex_destroy(&vd->framebuffer[i].grabmutex))
			printf ("release mutex failed \n");
		}
}
return 0;
}
int streamstop(videoIn_t *vd)
{
    int type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    int ret = 0;
    vd->readon = 0;
    usleep(100000); //wait 100ms 
    vd->streamon = 0;
    usleep(500000); //wait 500ms 
    if( vd->v4l2){
	ret = ioctl(vd->fd, VIDIOC_STREAMOFF, &type);
	if (ret < 0) {
	printf("Unable to %s capture: %d.\n", "start", errno);
        }
    }
    return ret;
}
int streamstart (videoIn_t *vd)
{
	int type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
        int ret = 0;
	if( vd->v4l2){
	    ret = ioctl(vd->fd, VIDIOC_STREAMON, &type);
	    if (ret < 0) {
	     printf("Unable to %s capture: %d.\n", "start", errno);
            }
        }
	vd->streamon = 1;
	usleep(500000); //wait 500ms 
	vd->readon = 1;
return ret;
}
void exit_fatal(char *messages)
{
	printf("%s \n",messages);
	exit(1);
}
/* palette is spca5xx internal JPG to RGB32 */
int setPalette (videoIn_t *vd, int palette)
{     int ipal;
      int masqs;
      int cursize;
	if (palette == vd->curpalette) return palette;
	if (!(palette & vd->palette)) {
		printf ("How can we change an unavailable palette !!!");
		return vd->curpalette;
	}
	ipal = paletteMIconvert(palette);
	masqs = convertsize(vd->hdrwidth,vd->hdrheight);
	if(vd->cursize != masqs){
		printf( "setPalette error cursize did not match hdrwithxhdrheight \n");
		vd->cursize = masqs;
		}
      streamstop(vd);
      if( vd->v4l2){
      /* v4l2 stuff */
      } else {
      vd->curpalette = palette;
      vd->formatIn = paletteconvert(palette);
      vd->bppIn = GetDepth (vd->formatIn);
      vd->v4l1->videopict.palette = vd->formatIn;
      vd->v4l1->vmmap.format = vd->formatIn;
      vd->v4l1->videopict.depth = vd->bppIn;
      SetVideoPict (vd);
      /* is the cur size in this new palette */
      if(vd->sizenative[ipal] & masqs) {
        vd->framesizeIn = ((vd->hdrwidth * vd->hdrheight * vd->bppIn) >> 3);
       } else {
       /* find a new size */
        cursize = nearest (vd->sizenative[ipal],masqs);
        if(cursize < 0){
		printf("Change palette failed to find a new size \n");
		return -1;
		}
	if(changeSizeP(vd,cursize) < 0){
		printf("Change palette failed to change sizeP \n");
		return -1;
		}
       }
      }
      if(!vd->grabmethod)
       	   vd->pFramebuffer =
		(unsigned char *) realloc (vd->pFramebuffer, (size_t) vd->framesizeIn);
       if(realloc_buffer(vd )< 0){
		printf("Change palette failed buffer allocation \n");
		return -1;
		}
      streamstart(vd);
   return palette;
}

int changeSizeP (videoIn_t *vd, int size)
{
	if(sizeconvert(&vd->hdrwidth,&vd->hdrheight,size) < 0)
		return -1;
	vd->cursize = size;
	vd->framesizeIn = ((vd->hdrwidth * vd->hdrheight * vd->bppIn) >> 3);	
	if( vd->v4l2){
	/* here v4l2 stuff*/
	} else {
	if (vd->grabmethod)
    {
      vd->v4l1->vmmap.height = vd->hdrheight;
      vd->v4l1->vmmap.width = vd->hdrwidth;
      vd->v4l1->vmmap.format = vd->formatIn;

    }
  else
    {

      if (ioctl (vd->fd, VIDIOCGWIN, &vd->v4l1->videowin) < 0)
	perror ("VIDIOCGWIN failed \n");
      vd->v4l1->videowin.height = vd->hdrheight;
      vd->v4l1->videowin.width = vd->hdrwidth;
      if (ioctl (vd->fd, VIDIOCSWIN, &vd->v4l1->videowin) < 0)
	perror ("VIDIOCSWIN failed \n");
      printf ("VIDIOCGWIN height %d  width %d \n",
	      vd->v4l1->videowin.height, vd->v4l1->videowin.width);
    }
	}
}
/* size is spca5xx internal XGA to QCIF */
int changeSize (videoIn_t *vd, int size)
{
	int ipal;
	if (vd->cursize&size)
	    return size;
	ipal = paletteMIconvert(vd->curpalette);
	if(!(vd->sizenative[ipal]&size)){
	    printf("Ooops Cannot change size unknow for this palette \n");
	    return vd->cursize;
	}
	if(sizeconvert(&vd->hdrwidth,&vd->hdrheight,size) < 0)
	    return vd->cursize;
	vd->cursize = size;
	vd->framesizeIn = ((vd->hdrwidth * vd->hdrheight * vd->bppIn) >> 3);	
      streamstop(vd);
      if( vd->v4l2){
      } else {
      if (vd->grabmethod){
          vd->v4l1->vmmap.height = vd->hdrheight;
          vd->v4l1->vmmap.width = vd->hdrwidth;
          vd->v4l1->vmmap.format = vd->formatIn;
       }
      else
       {
       if (ioctl (vd->fd, VIDIOCGWIN, &vd->v4l1->videowin) < 0)
	 perror ("VIDIOCGWIN failed \n");
       vd->v4l1->videowin.height = vd->hdrheight;
       vd->v4l1->videowin.width = vd->hdrwidth;
       if (ioctl (vd->fd, VIDIOCSWIN, &vd->v4l1->videowin) < 0)
	perror ("VIDIOCSWIN failed \n");
        printf ("VIDIOCGWIN height %d  width %d \n",
	      vd->v4l1->videowin.height, vd->v4l1->videowin.width);
       }
      }
      if(!vd->grabmethod)
       	   vd->pFramebuffer =
		(unsigned char *) realloc (vd->pFramebuffer, (size_t) vd->framesizeIn);
       if(realloc_buffer(vd )< 0)
		exit_fatal("Change palette failed buffer allocation ");
      streamstart(vd);
     return size;
}

int grabone(videoIn_t *vd)
{
  int len;
  int size;
  int erreur = 0;
 unsigned char *tmp= NULL;
  if(!vd->streamon) {
   		usleep(20000); //wait 20ms 
		return 0;
		}
if( vd->v4l2) {
} else {
  if (vd->grabmethod)
    {
      vd->v4l1->vmmap.height = vd->hdrheight;
      vd->v4l1->vmmap.width = vd->hdrwidth;
      vd->v4l1->vmmap.format = vd->formatIn;
      if (ioctl (vd->fd, VIDIOCSYNC,&vd->v4l1->vmmap.frame) < 0)
	{
	  //perror ("cvsync err\n");
	  //return 0;
	}
	/* Is there someone using the frame spin until*/  
    while((vd->framebuffer[vd->framecour].readlck != 0) && vd->signalquit)
   	usleep(1000);
	pthread_mutex_lock (&vd->framebuffer[vd->framecour].grabmutex);
	tmp = vd->framebuffer[vd->framecour].data;
	memcpy (tmp, vd->pFramebuffer + vd->v4l1->videombuf.offsets[vd->v4l1->vmmap.frame] , vd->framesizeIn);       
	pthread_mutex_unlock (&vd->framebuffer[vd->framecour].grabmutex); 
	/************************************/
	
      if ((ioctl (vd->fd, VIDIOCMCAPTURE, &(vd->v4l1->vmmap))) < 0)
	{
	 // perror ("cmcapture");
	 // return 0;
	}
      vd->v4l1->vmmap.frame = (vd->v4l1->vmmap.frame + 1) % vd->v4l1->videombuf.frames;
      vd->framecour = (vd->framecour +1) % NBUFFER;
      //printf("frame nb %d\n",vd->v4l1->vmmap.frame);

    }
  else
    {
      /* read method */
      size = vd->framesizeIn;
      len = read (vd->fd, vd->pFramebuffer, size);
      if (len <= 0 )
	{
	  printf ("v4l read error\n");
	  printf ("len %d asked %d \n", len, size);
	  return 0;
	}
      /* Is there someone using the frame spin until*/
       while((vd->framebuffer[vd->framecour].readlck != 0)&& vd->signalquit)
   	usleep(1000);
	pthread_mutex_lock (&vd->framebuffer[vd->framecour].grabmutex);
	// memcpy (vd->framebuffer[vd->framecour].data, vd->pFramebuffer, vd->framesizeIn);
	vd->framecour = (vd->framecour +1) % NBUFFER;  
	pthread_mutex_unlock (&vd->framebuffer[vd->framecour].grabmutex); 
      /************************************/
     
    }
   }
  return erreur;	
}
/*****************************************************************************
* 				Private
******************************************************************************/
static int 
nearest(int masq ,int val){
	int i = 8;
	int valdwn = val;
	int valup = val ;
if (masq&val) 
	     return val&masq;
while (i--) {
     
    if(valdwn >> 1){
        valdwn = valdwn >> 1;
	if(masq&valdwn)
	  return valdwn&masq;  
	}	      
    if((valup << 1) < 256){
	valup = valup << 1;
	if(masq&valup)
	  return valup&masq;  	
        }		
}  
return  -1; 
}

static int
GetVideoPict (videoIn_t *vd)
{
  if (ioctl (vd->fd, VIDIOCGPICT, &vd->v4l1->videopict) < 0)
    return -1;
  return 0;
}

static int
SetVideoPict (videoIn_t *vd)
{
  if (ioctl (vd->fd, VIDIOCSPICT, &vd->v4l1->videopict) < 0)
    return -1;
  return 0;
}

static int
uninit_v4l1(videoIn_t *vd){
 if(vd->streamon || vd->readon)
 	printf("Damned trying to remove buffer in used ");
 dealloc_v4l1_bufferIn(vd);
 return 0;
}

static int
uninit_v4l2(videoIn_t *vd){
 if(vd->streamon || vd->readon)
 	printf("Damned trying to remove buffer in used ");
 return 0;
}

static int
init_v4l2(videoIn_t *vd )
{
if ((vd->v4l2->cap.capabilities & V4L2_CAP_VIDEO_CAPTURE) == 0) {
	printf("Error opening device %s: video capture not supported.\n",
	       vd->videodevice);
	goto fatal;;
    }
if (vd->v4l2->cap.capabilities & V4L2_CAP_STREAMING) {
	    printf("%s mmap streaming i/o\n", vd->videodevice);
	   vd->grabmethod = 1;
	}  else {
		if (!(vd->v4l2->cap.capabilities & V4L2_CAP_READWRITE)) {
	    printf("%s does not support read i/o\n", vd->videodevice);
	    goto fatal;
		}
	}
	
	return 0;
fatal:
	return -1; 

}

static int
init_v4l1(videoIn_t *vd )
{
    int len;
    int erreur = 0;
    printf(" v4l1 open init done \n");
    len = strlen(vd->v4l1->videocap.name);
     if(len) {
	vd->cameraname = (char *)malloc(len+1);
	snprintf (vd->cameraname, len, "%s", vd->v4l1->videocap.name);
	printf ("Camera %s\n",vd->cameraname);
      }
      
     if (ioctl (vd->fd, VIDIOCGCHAN, &vd->v4l1->videochan) == -1)
      {
      printf ("Hmm did not support Video_channel\n");
      vd->cameratype = UNOW;
      }
      else
      {
        if (vd->v4l1->videochan.name){
          printf ("Bridge found: %s \n", vd->v4l1->videochan.name);
          snprintf (vd->bridge, 9, "%s", vd->v4l1->videochan.name);
         }
        else
         {
          printf ("Bridge not found not a spca5xx Webcam Probing the hardware !!\n");
          vd->cameratype = UNOW;
         }
      }
      printf(" v4l1 probe .....  \n");
     if( probeHardCap(vd ) < 0) {
	  printf("could't probe video palette Abort !\n");
	  free(vd->cameraname);
	  vd->cameraname = NULL;
	 return -1; 
     }
 printf(" v4l1 probe done \n");
 erreur = GetVideoPict (vd);
  vd->v4l1->videopict.palette = vd->formatIn;
  vd->v4l1->videopict.depth = GetDepth (vd->formatIn);     
  erreur = SetVideoPict (vd);
  
  /* Double check is there a match break pwc 
  if ( erreur < 0 || vd->formatIn != vd->v4l1->videopict.palette ||
      vd->bppIn != vd->v4l1->videopict.depth)
    exit_fatal ("could't set video palette Abort !");
   */ 
  vd->fsgctrl = v4l1_control;
  /* Set the Input frame buffer */
 if(alloc_v4l1_bufferIn(vd) < 0){
        free(vd->cameraname);
	vd->cameraname = NULL;
 	return -1;
	}
	 printf(" v4l1 pframebuffer donee \n");
 return 0;
}

/* alloc Buffer in  */
static int
alloc_v4l1_bufferIn(videoIn_t *vd)
{ 
int f;
  vd->pFramebuffer = NULL; 
  vd->framesizeIn = (vd->hdrwidth * vd->hdrheight * vd->bppIn) >> 3;
if (vd->grabmethod)
    {
      printf (" grabbing method default MMAP asked \n");
      // MMAP VIDEO acquisition
      memset (&(vd->v4l1->videombuf), 0, sizeof (vd->v4l1->videombuf));
      if (ioctl (vd->fd, VIDIOCGMBUF, &(vd->v4l1->videombuf)) < 0)
	{
	  perror (" init VIDIOCGMBUF FAILED\n");
	  return -1;
	}
      printf ("VIDIOCGMBUF size %d  frames %d  offets[0]=%d offsets[1]=%d\n",
	      vd->v4l1->videombuf.size, vd->v4l1->videombuf.frames,
	      vd->v4l1->videombuf.offsets[0], vd->v4l1->videombuf.offsets[1]);
      vd->pFramebuffer =
	(unsigned char *) mmap (0, vd->v4l1->videombuf.size, PROT_READ | PROT_WRITE,
				MAP_SHARED, vd->fd, 0);
      vd->mmapsize = vd->v4l1->videombuf.size;
      vd->v4l1->vmmap.height = vd->hdrheight;
      vd->v4l1->vmmap.width = vd->hdrwidth;
      vd->v4l1->vmmap.format = vd->formatIn;
      for (f = 0; f < vd->v4l1->videombuf.frames; f++)
	{
	  vd->v4l1->vmmap.frame = f;
	  if (ioctl (vd->fd, VIDIOCMCAPTURE, &(vd->v4l1->vmmap)))
	    {
	      perror ("cmcapture");
	    }
	}
      vd->v4l1->vmmap.frame = 0;
    }
  else
    {
      /* read method */
      /* allocate the read buffer */
      vd->pFramebuffer =
	(unsigned char *) realloc (vd->pFramebuffer, (size_t) vd->framesizeIn);
      printf (" grabbing method READ asked \n");
      if (ioctl (vd->fd, VIDIOCGWIN, &(vd->v4l1->videowin)) < 0)
	perror ("VIDIOCGWIN failed \n");
      vd->v4l1->videowin.height = vd->hdrheight;
      vd->v4l1->videowin.width = vd->hdrwidth;
      if (ioctl (vd->fd, VIDIOCSWIN, &(vd->v4l1->videowin)) < 0)
	perror ("VIDIOCSWIN failed \n");
    }
  vd->framecour = 0;
  return 0;	  
}
static int
dealloc_v4l1_bufferIn(videoIn_t *vd)
{ 
if (vd->grabmethod)
    {
    munmap (vd->pFramebuffer, vd->mmapsize);
    } else {
    free(vd->pFramebuffer);
    }
    vd->pFramebuffer = NULL;
    return 0;
}

static int
probeHardCap(videoIn_t *vd ){
	struct video_window win;
	struct video_picture pict;
	struct video_mmap vmmap;
	int maxw,minw,maxh,minh;
	int w,h;
	int masqs, masqp,pal,size;
	int defaut = 1;
	vd->grabmethod = -1; //should be set by probe
	if( vd->v4l2){
	} else {
	maxw = vd->v4l1->videocap.maxwidth;
	minw = vd->v4l1->videocap.minwidth;
	maxh = vd->v4l1->videocap.maxheight;
	minh = vd->v4l1->videocap.minheight;
	maxw = convertIsize(maxw,maxh); //IXVGA = 0 to IQCIF = 7
	minw = convertIsize(minw,minh);
	vd->palette = 0; // pas de palette dispo
/* FIX the driver should not answer yes if the palette size is not available */
         printf (" try probe ioctl ( VIDIOCMCAPTURE ) \n");
	for (pal = IJPG; pal < IMAXPAL; pal++){
		vd->sizenative[pal] = 0; // pas de size dispo
		vmmap.format = paletteIconvert(pal); // a partir de index -> palette v4l1
		masqp = convertpalette(vmmap.format); // a partir de la palette -> masq
		for ( size = maxw; size <= minw; size++){
			sizeIconvert(&w,&h,size); // a partir de index -> w x h			
			vmmap.height = h;
                        vmmap.width = w;
			vmmap.frame = 0;
	  		if (ioctl (vd->fd, VIDIOCMCAPTURE, &vmmap) )
	    		{
			printf("Reject size width %d ,height %d, Palette (%d) \n",w,h,vmmap.format);
	      			//perror ("cmcapture");
	    		} else {
			 masqs = convertsize(w,h);
			printf("Accept size width %d ,height %d, Palette (%d) \n",w,h,vmmap.format);
			 if (masqs > 0) {
			   vd->sizenative[pal] |= masqs;
			   vd->palette |= masqp;
			   }
			  if(defaut) {
			  	defaut = 0;
				vd->hdrwidth = w;
				vd->hdrheight = h;
				vd->formatIn = vmmap.format;
				vd->bppIn = GetDepth (vd->formatIn);
				vd->cursize = masqs;
				vd->curpalette = masqp;
				vd->grabmethod = 1;
			  }
			}
		}
	
		
	 }
defaut =1;
	if(defaut) {
	printf (" try probe ioctl ( VIDIOCGWIN,  VIDIOCCGPICT ) \n");
	if (ioctl (vd->fd, VIDIOCGWIN, &win) < 0) {
		perror ("VIDIOCGWIN failed \n");
		return -1;
	}
	if (ioctl (vd->fd, VIDIOCGPICT, &pict) < 0)
    		{
      		perror ("Couldnt get videopict params with VIDIOCGPICT\n");
      		return -1;
   		 }
	vd->palette = 0; // pas de palette dispo
	for (pal = IJPG; pal < IMAXPAL; pal++){
		vd->sizenative[pal] = 0; // pas de size dispo	 
	 pict.palette = paletteIconvert(pal); // a partir de index -> palette v4l1;
	 pict.depth = GetDepth (pict.palette);
	 masqp = convertpalette(pict.palette); // a partir de la palette -> masq
	 sizeIconvert(&w,&h,maxw); // a partir de index -> wxh	
			win.width = w;
			win.height = h;
			if (ioctl (vd->fd, VIDIOCSWIN, &win) < 0) ;
      			 	
	 for ( size = maxw; size <= minw; size++){
	 pict.palette = paletteIconvert(pal); // a partir de index -> palette v4l1;
	 pict.depth = GetDepth (pict.palette);
	 if (ioctl (vd->fd, VIDIOCSPICT, &pict) < 0)
    		{
      		printf("Couldnt set palette first try %d \n", pict.palette);
    		}
	if (ioctl (vd->fd, VIDIOCGPICT, &pict) < 0)
    		{
      		printf("Couldnt get palette %d \n", pict.palette);    
    		}
	 if(pict.palette == paletteIconvert(pal)){	 	
			sizeIconvert(&w,&h,size); // a partir de index -> wxh	
			win.width = w;
			win.height = h;
			if (ioctl (vd->fd, VIDIOCSWIN, &win) < 0) {
      			 printf("Reject size width %d ,height %d, Palette (%d) \n",w,h,pict.palette);	    
    			} else {
			 masqs = convertsize(w,h);
			 printf("Accept size width %d ,height %d, Palette (%d) \n",w,h,pict.palette);
			 if (masqs > 0) {
			   vd->sizenative[pal] |= masqs;
			   vd->palette |= masqp;
			   }
			 if(defaut) {
			  	defaut = 0;
				vd->hdrwidth = w;
				vd->hdrheight = h;
				vd->formatIn = pict.palette;
				vd->bppIn = GetDepth (vd->formatIn);
				vd->cursize = masqs;
				vd->curpalette = masqp;
				if(vd->grabmethod == -1)
				        vd->grabmethod = 0;
			 }
			}
		}
	 }
	 printf(" palette %d size avail %d \n", masqp, vd->sizenative[pal]);
	 }
	}
	
	if(vd->grabmethod == -1){
	printf (" Sorry the probe function did not match any palette size available !!\n");
	return -1;
	}	
    }
    return 0;
}

static int
GetDepth (int format)
{
  int depth;
  switch (format)
    {
   
   case VIDEO_PALETTE_JPEG:
      {
	depth = 8;		
      }
      break;
    case VIDEO_PALETTE_RAW:
      {
	depth = 8;		
      }
      break;
    case VIDEO_PALETTE_YUV420:   
    case VIDEO_PALETTE_YUV420P:
      {
	depth = (8 * 3) >> 1;
      }
      break;
    case VIDEO_PALETTE_YUV422:
    case VIDEO_PALETTE_RGB565:
      depth = 16;
      break;
    case VIDEO_PALETTE_RGB24:
      depth = 24;
      break;
    case VIDEO_PALETTE_RGB32:
      {
	depth = 32;
      }
      break;
    default:
      depth = -1;
      break;
    }
  return depth;
}

/* return index of masq byte for the needed size */
static int convertIsize( int width, int height)
{
	switch (width){
	case 1280:
		if(height ==960)
		return IXVGA; // 0
	break;
	case 800:
		if(height ==600)
		return ISVGA;
	break;
	case 640:
		if(height == 480)
		return IVGA;
	break;
	case 384:
		if(height == 288)
		return IPAL;
	break;
	case 352:
		if(height == 288)
		return ISIF;
	break;
	case 320:
		if(height == 240)
		return ICIF;
	break;
	case 176:
		if(height == 144)
		return IQSIF;
	break;
	case 160:
		if(height == 120)
		return IQCIF;
	break;
	default:
	break;
	}
 return -1;
}
/* return masq byte for the needed size */
static int convertsize( int width, int height)
{
	switch (width){
	case 1280:
		if(height ==960)
		return XVGA; // 1
	break;
	case 800:
		if(height ==600)
		return SVGA;
	break;
	case 640:
		if(height == 480)
		return VGA;
	break;
	case 384:
		if(height == 288)
		return PAL;
	break;
	case 352:
		if(height == 288)
		return SIF;
	break;
	case 320:
		if(height == 240)
		return CIF;
	break;
	case 176:
		if(height == 144)
		return QSIF;
	break;
	case 160:
		if(height == 120)
		return QCIF;
	break;
	default:
	break;
	}
 return -1;
}
/* get masq return  width hight */
static int sizeIconvert( int *width, int *height, int Isize)
{
	switch (Isize){
	case IXVGA:
		*height = 960;
		*width = 1280;	
	break;
	case ISVGA:
		*height = 600;
		*width = 800;	
	break;
	case IVGA:
		*height = 480;
		*width = 640;		
	break;
	case IPAL:
		*height = 288;
		*width = 384;		
	break;
	case ISIF:
		*height = 288;
		*width = 352;		
	break;
	case ICIF:
		*height = 240;
		*width = 320;		
	break;
	case IQSIF:
		*height = 144;
		*width = 176;		
	break;
	case IQCIF:
		*height = 120;
		*width = 160;		
	break;
	default:
	return -1;
	break;
	}
 return 0;
}
/* get masq return  width hight */
static int sizeconvert( int *width, int *height, int size)
{
	switch (size){
	case XVGA:
		*height = 960;
		*width = 1280;	
	break;
	case SVGA:
		*height = 600;
		*width = 800;	
	break;
	case VGA:
		*height = 480;
		*width = 640;		
	break;
	case PAL:
		*height = 288;
		*width = 384;		
	break;
	case SIF:
		*height = 288;
		*width = 352;		
	break;
	case CIF:
		*height = 240;
		*width = 320;		
	break;
	case QSIF:
		*height = 144;
		*width = 176;		
	break;
	case QCIF:
		*height = 120;
		*width = 160;		
	break;
	default:
	return -1;
	break;
	}
 return 0;
}
/* get palette return masq */
static int convertpalette( int palette)
{
	switch (palette){
	case VIDEO_PALETTE_JPEG:
		return JPG;
	break;
	case VIDEO_PALETTE_YUV422:	
		return YUV422;
	break;
	case VIDEO_PALETTE_YUV420P:	
		return YUV420P;
	break;
	case VIDEO_PALETTE_RGB24:	
		return RGB24;
	break;
	case VIDEO_PALETTE_RGB565:	
		return RGB565;
	break;
	case VIDEO_PALETTE_RGB32:	
		return RGB32;
	break;
	default:
	break;
	}
 return -1;
}
/* get masq return palette */
static int paletteconvert( int code)
{
	switch (code){
	case JPG:
		return VIDEO_PALETTE_JPEG;			
	break;
	case YUV422:
		return VIDEO_PALETTE_YUV422;		
	break;
	case YUV420P:
		return VIDEO_PALETTE_YUV420P;		
	break;
	case RGB24:
		return VIDEO_PALETTE_RGB24;			
	break;
	case RGB565:
		return VIDEO_PALETTE_RGB565;		
	break;
	case RGB32:
		return VIDEO_PALETTE_RGB32;		
	break;
	default:
	return -1;
	break;
	}
 return 0;
}
/* get index of masq return palette */
static int paletteIconvert( int code)
{
	switch (code){
	case IJPG:
		return VIDEO_PALETTE_JPEG;			
	break;
	case IYUV422:
		return VIDEO_PALETTE_YUV422;		
	break;
	case IYUV420P:
		return VIDEO_PALETTE_YUV420P;		
	break;
	case IRGB24:
		return VIDEO_PALETTE_RGB24;			
	break;
	case IRGB565:
		return VIDEO_PALETTE_RGB565;		
	break;
	case IRGB32:
		return VIDEO_PALETTE_RGB32;		
	break;
	default:
	return -1;
	break;
	}
 return 0;
}
/* get masq return index of masq */
int paletteMIconvert( int masq)
{
	switch (masq){
	case JPG:
		return IJPG;			
	break;
	case YUV422:
		return IYUV422;
	break;
	case YUV420P:
		return IYUV420P;		
	break;
	case RGB24:
		return IRGB24;			
	break;
	case RGB565:
		return IRGB565;		
	break;
	case RGB32:
		return IRGB32;		
	break;
	default:
	return -1;
	break;
	}
 return 0;
}

__u8 SpcaGetBrightness (videoIn_t * vdin, __u8 param)
{
  if (GetVideoPict (vdin) < 0)
    {
      printf (" Error getBrightness \n");
      return 0;
    }
  return ((vdin->v4l1->videopict.brightness) >> 8);
}

__u8 SpcaSetBrightness (videoIn_t *vdin, __u8 bright)
{
  vdin->v4l1->videopict.brightness = bright << 8;
  if (SetVideoPict (vdin) < 0)
    {
      printf (" Error setBrightness \n");
    }
return (vdin->v4l1->videopict.brightness >> 8) ;
}

__u8 SpcaGetContrast (videoIn_t *vdin, __u8 param)
{
  if (GetVideoPict (vdin) < 0)
    {
      printf (" Error getContrast \n");
      return 0;
    }
  return ((vdin->v4l1->videopict.contrast) >> 8);
}

__u8 SpcaSetContrast (videoIn_t *vdin, __u8 contrast)
{
  vdin->v4l1->videopict.contrast = contrast << 8;
  if (SetVideoPict (vdin) < 0)
    {
      printf (" Error setContrast \n");
    }
    return vdin->v4l1->videopict.contrast >> 8;
}
__u8 SpcaGetColors (videoIn_t *vdin, __u8 param)
{
  if (GetVideoPict (vdin) < 0)
    {
      printf (" Error getColors \n");
      return 0;
    }
  return ((vdin->v4l1->videopict.colour) >> 8);
}

__u8 SpcaSetColors (videoIn_t *vdin, __u8 colors)
{
  vdin->v4l1->videopict.colour = colors << 8;
  if (SetVideoPict (vdin) < 0)
    {
      printf (" Error setColors \n");
    }
  return vdin->v4l1->videopict.colour >> 8; 
}

/* we assume that struct videopict is initialized */
__u8 upbright( videoIn_t *vdin , __u8 param)
{ 	
	unsigned short bright =0;
	
	bright = vdin->v4l1->videopict.brightness;
	if(( bright+0x200) < 0xffff){
	bright += 0x200;
	vdin->v4l1->videopict.brightness = bright;
	if (SetVideoPict (vdin) < 0)
    		{
      			printf (" Error setVideopict \n");
			return 0;
    		}	
	}
	return bright >> 8;	
}

__u8 downbright( videoIn_t *vdin, __u8 param)
{	
	unsigned short bright =0;
	
	bright = vdin->v4l1->videopict.brightness;
	if(( bright-0x200) > 0){
	bright -= 0x200;
	vdin->v4l1->videopict.brightness = bright;
	if (SetVideoPict (vdin) < 0)
    		{
      			printf (" Error setVideopict \n");
			return 0;
    		}	
	}
	return bright >> 8;	
}
__u8 upcontrast( videoIn_t *vdin, __u8 param)
{	
	unsigned short contrast =0;
	
	contrast = vdin->v4l1->videopict.contrast;
	if((contrast+0x200) < 0xffff){
	contrast += 0x200;
	vdin->v4l1->videopict.contrast = contrast;
	if (SetVideoPict (vdin) < 0)
    		{
      			printf (" Error setVideopict \n");
			return 0;
    		}	
	}
	return contrast >> 8;
}
__u8 downcontrast( videoIn_t *vdin, __u8 param)
{	
	unsigned short contrast =0;
	
	contrast = vdin->v4l1->videopict.contrast;
	if( (contrast-0x200) > 0){
	contrast -= 0x200;
	vdin->v4l1->videopict.contrast = contrast;
	if (SetVideoPict (vdin) < 0)
    		{
      			printf (" Error setVideopict \n");
			return 0;
    		}	
	}
	return contrast >> 8;	
}
__u8 qualityUp(videoIn_t *vdin, __u8 param)
{
	struct video_param *videoparam= &vdin->v4l1->videoparam;
	int fd = vdin->fd;
	unsigned char index = videoparam->quality;
	index += 1;
	spcaSetQuality(fd,videoparam,index);
	return index;
}
__u8 qualityDown(videoIn_t *vdin, __u8 param)
{
	struct video_param *videoparam= &vdin->v4l1->videoparam;
	int fd = vdin->fd;
	unsigned char index = videoparam->quality;
	if(index > 0) index--;
	spcaSetQuality(fd,videoparam,index);
	return index;
}
__u8 timeUp(videoIn_t *vdin, __u8 param)
{
	struct video_param *videoparam= &vdin->v4l1->videoparam;
	int fd = vdin->fd;
	unsigned short index = videoparam->time_interval;
	index+=10;
	spcaSetTimeInterval(fd,videoparam,index);
	return index >>8;
}
__u8 timeDown(videoIn_t *vdin, __u8 param)
{
	struct video_param *videoparam= &vdin->v4l1->videoparam;
	int fd = vdin->fd;
	unsigned short index = videoparam->time_interval;
	if(index > 0) index -=10;
	spcaSetTimeInterval(fd,videoparam,index);
	return index >>8;
}
__u8 spcaSetAutoExpo(videoIn_t *vdin, __u8 param)
{
	struct video_param *videoparam= &vdin->v4l1->videoparam;
	int fd = vdin->fd;
	videoparam->chg_para = CHGABRIGHT;
	videoparam->autobright = param;//!videoparam->autobright;
	if(ioctl(fd,SPCASVIDIOPARAM, videoparam) == -1){
		printf ("autobright error !!\n");
	} else
		spcaPrintParam (fd,videoparam);
	return videoparam->autobright;
}
static void spcaPrintParam (int fd, struct video_param *videoparam)
{
	if(ioctl(fd,SPCAGVIDIOPARAM, videoparam) == -1){
		printf ("wrong spca5xx device\n");
	} else 
		printf("quality %d autoexpo %d Timeframe %d \n",
			 videoparam->quality,videoparam->autobright,videoparam->time_interval);
}

static void spcaSetTimeInterval(int fd, struct video_param *videoparam, unsigned short time)
{
	if (time < 1000) {
	videoparam->chg_para = CHGTINTER;
	videoparam->time_interval = time;
	if(ioctl(fd,SPCASVIDIOPARAM, videoparam) == -1){
		printf ("frame_times error !!\n");
	} else
		spcaPrintParam (fd,videoparam);
	}

}
static void spcaSetQuality(int fd, struct video_param *videoparam, unsigned char index)
{
	if (index < 6) {
	videoparam->chg_para = CHGQUALITY;
	videoparam->quality = index;
	if(ioctl(fd,SPCASVIDIOPARAM, videoparam) == -1){
		printf ("quality error !!\n");
	} else
		spcaPrintParam (fd,videoparam);
	}
}

ctrlop opt[END_CTRL] ={
{GBRIGHTNESS,SpcaGetBrightness},
{SBRIGHTNESS,SpcaSetBrightness },
{GCONTRAST,SpcaGetContrast},
{SCONTRAST,SpcaSetContrast},
{GCOLORS,SpcaGetColors},
{SCOLORS,SpcaSetColors},
{UPBRIGHT, upbright},
{DWBRIGHT,downbright},
{UPCONTRAST,upcontrast},
{DWCONTRAST,downcontrast},
{UPQUAl,qualityUp},
{DWQUAL,qualityDown},
{UPDTIMES,timeUp},
{DWDTIMES,timeDown},
{AUTOONOFF,spcaSetAutoExpo},
};
