/****************************************************************************
#	 	GspcaGui:  Gspca/Spca5xx Grabber                            #
# 		Copyright (C) 2004 2005 2006 Michel Xhaard                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#                                                                           #
****************************************************************************/ 
#include <linux/videodev.h>
#include <pthread.h>
#define NBUFFER 4
#define NBUFFERI 2
/* ITU-R-BT.601 PAL/NTSC */
#include "spcav4l.h"
#include "uvc.h"

typedef enum {
	IXVGA =0,
	ISVGA,
	IVGA,
	IPAL,
	ISIF,
	ICIF,
	IQSIF,
	IQCIF,
	IMAXSIZE,
} ISIZE;

typedef enum {
	IJPG = 0,
	IYUV422,
	IYUV420P,
	IRGB24,
	IRGB565,
	IRGB32,
	IMAXPAL,
} IPALETTE;

#define MASQ 1
#define XVGA (MASQ << IXVGA)
#define SVGA (MASQ << ISVGA)
#define VGA  (MASQ << IVGA)
#define PAL (MASQ << IPAL)
#define SIF (MASQ << ISIF)
#define CIF (MASQ << ICIF)
#define QSIF (MASQ << IQSIF)
#define QCIF (MASQ << IQCIF)

/* fourcc palette check in preference order*/
#define JPG (MASQ << IJPG)
#define YUV422 (MASQ << IYUV422)
#define YUV420P (MASQ << IYUV420P)
#define RGB24 (MASQ << IRGB24)
#define RGB565 (MASQ << IRGB565)
#define RGB32 (MASQ << IRGB32)


enum {
	JPEG = 0,
	YUVY,
	YYUV,
//	YUYV,
	GREY,
	GBRG,
	SN9C,
	GBGR,
	UNOW,
};

typedef struct v4l1_t {
	struct video_capability videocap;
	struct video_picture videopict;
	struct video_window videowin;
	struct video_channel videochan;
	struct video_param videoparam;
	struct video_mmap vmmap;
	struct video_mbuf videombuf;	
} v4l1_t;
  
typedef struct v4l2_t {
	struct v4l2_capability cap;
	struct v4l2_cropcap cropcap;
        struct v4l2_crop crop;
    	struct v4l2_format fmt;
    	struct v4l2_buffer buf;
    	struct v4l2_requestbuffers rb;
	void *mem[NBUFFERI];
} v4l2_t; 
 
typedef union v4l { 
  	v4l1_t v4l1;
  	v4l2_t v4l2;
} v4l;



typedef struct framebuf {
	pthread_mutex_t grabmutex; // protect rw lock
	int readlck;
	unsigned char *data;
	int size;
	int formatIn;
	int w;
	int h;	
} framebuf;

struct videoIn;

typedef __u8 (*SGctrl) (struct videoIn* vd, int control,__u8 value);
typedef __u8 (*ctrlv4l1) (struct videoIn* vd,__u8 value);

typedef enum {
 GBRIGHTNESS = 0,
 SBRIGHTNESS,
 GCONTRAST,
 SCONTRAST,
 GCOLORS,
 SCOLORS,
 UPBRIGHT,
 DWBRIGHT,
 UPCONTRAST,
 DWCONTRAST,
 UPQUAl,
 DWQUAL,
 UPDTIMES,
 DWDTIMES,
 AUTOONOFF,
 END_CTRL,
} V4L1_CONTROL;

typedef struct ctrlop {
	int ctrl;
	ctrlv4l1 fctrl;
} ctrlop;

typedef struct videoIn {
	struct videoIn *this;
	int fd;
	char *videodevice;
	char *cameraname;
	int cameratype;
	char bridge[9];
	v4l indev;
	v4l1_t *v4l1; //pointer to union v4l1 set by init
	v4l2_t *v4l2; //pointer to union v4l2 set by init
	SGctrl fsgctrl;
	framebuf framebuffer[NBUFFER]; //queue frames
	int framecour; //index to the queue grabbing frames
	int mmapsize;
	unsigned char *pFramebuffer; // input frame
	int framesizeIn;	
	int grabmethod; // iomethod
	int sizenative[IMAXPAL]; // sizes f(palettes) available
	int cursize;
	int palette; //palettes availables
	int curpalette;
	int hdrwidth;
	int hdrheight;
	int formatIn;
	int bppIn;
	int streamon;
	int readon;
	int signalquit;
} videoIn_t;

videoIn_t * openvideo(const char *path);
int closevideo(videoIn_t *vd);
int grabone(videoIn_t *vd);
int setPalette (videoIn_t *vd, int palette);
int changeSize (videoIn_t *vd, int size);
int streamstop(videoIn_t *vd);
int streamstart (videoIn_t *vd);
int paletteMIconvert( int masq);
