
#ifndef ZC3XXUSB_H
#define ZC3XXUSB_H
/****************************************************************************
#	 	Z-star zc301 zc302 P library                                #
# 		Copyright (C) 2004 2005 2006 Michel Xhaard                  #
#               mxhaard@magic.fr                                            #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#                                                                           #
****************************************************************************/
#include "tas5130c.h"
#include "icm105a.h"
#include "hdcs2020.h"
#include "hv7131b.h"
#include "pb0330.h"
#include "hv7131c.h"
#include "cs2102.h"
#include "pas106b.h"
#include "ov7630c.h"
/*******************     Camera Interface   ***********************/

static int zc3xx_init(struct gspca_device *gspca_dev);
static int zc3xx_config(struct gspca_device *gspca_dev);
static void zc3xx_start(struct gspca_device *gspca_dev);
static void zc3xx_stop0(struct gspca_device *gspca_dev);
static void zc3xx_stopN(struct gspca_device *gspca_dev);
static void zc3xx_shutdown(struct gspca_device *gspca_dev);
static int zc3xx_sofdetect(struct gspca_device *gspca_dev,struct gspca_rawframe *frame,
			unsigned char *cdata,int *iPix, int seqnum, int *datalength);
static int zc3xx_set_control(struct gspca_device *gspca_dev,struct v4l2_control *ctrl);
static int zc3xx_get_control(struct gspca_device *gspca_dev,struct v4l2_control *ctrl);
static int zc3xx_query_control(struct gspca_device *gspca_dev,struct v4l2_queryctrl *q_ctrl);
static int
gspca_check_vimicro(struct gspca_device *gspca_dev, __u16 vendor, __u16 product);
static int 
gspca_attach_vimicro(struct gspca_device *gspca_dev);
/*******************     Camera Private     ***********************/
static void zc3xx_stopN(struct gspca_device *gspca_dev){}
static __u16 zc3xx_getbrightness(struct gspca_device *gspca_dev);
static __u16 zc3xx_getcontrast(struct gspca_device *gspca_dev);
static __u16 zc3xx_getcolors(struct gspca_device *gspca_dev);
static void zc3xx_setbrightness(struct gspca_device *gspca_dev);
static void zc3xx_setcontrast(struct gspca_device *gspca_dev);
static void zc3xx_setcolors(struct gspca_device *gspca_dev);
static void zc3xx_setAutobright(struct gspca_device *gspca_dev);
static void zc3xx_setquality(struct gspca_device *gspca_dev);
/******************************************************************/
static struct cam_operation fzc3xx = {
 	.initialize = zc3xx_init,
	.configure = zc3xx_config,
	.start = zc3xx_start,
	.stopN = zc3xx_stopN,
	.stop0 = zc3xx_stop0,
	.cam_shutdown = zc3xx_shutdown,
	.sof_detect = zc3xx_sofdetect,
	.set_ctrl = zc3xx_set_control,
	.get_ctrl = zc3xx_get_control,
	.query_ctrl = zc3xx_query_control,	
 };
static struct v4l2_queryctrl zc3xx_controls[GSPCATOT_CTRL] = {
        {
	    .id      = V4L2_CID_BRIGHTNESS,
	    .type    = V4L2_CTRL_TYPE_INTEGER,
	    .name    = "Brightness",
	    .minimum = 0,
	    .maximum = 255,
	    .step    = 1,
	    .default_value = 128,
	},
        {
	    .id      = V4L2_CID_CONTRAST,
	    .type    = V4L2_CTRL_TYPE_INTEGER,
	    .name    = "Contrast",
	    .minimum = 0,
	    .maximum = 8,
	    .step    = 1,
	    .default_value = 4,
	},
        {
	    .id      = V4L2_CID_SATURATION,
	    .type    = V4L2_CTRL_TYPE_INTEGER,
	    .name    = "Colors",
	    .minimum = 0,
	    .maximum = 255,
	    .step    = 1,
	    .default_value = 128,
	},
        {
	    .id      = V4L2_CID_AUTO_WHITE_BALANCE,
	    .type    = V4L2_CTRL_TYPE_BOOLEAN,
	    .name    = "Auto White Balance",
	    .minimum = 0,
	    .maximum = 1,
	    .step    = 1,
	    .default_value = 0,
	},
	{
	    .id      = V4L2_CID_AUTOGAIN,
	    .type    = V4L2_CTRL_TYPE_BOOLEAN,
	    .name    = "Auto Gain Enabled",
	    .minimum = 0,
	    .maximum = 1,
	    .step    = 1,
	    .default_value = 1,
	}
};

enum {
    SensorId = 0,
    reg8d,
    val8d,
    SensorReg1,
    valSreg1L,
    valSreg1H,
    SensorReg2,
    valSreg2L,
    valSreg2H,
    totval,
};
#define VGATOT 9
static __u8 zcxxi2cSensor[VGATOT][totval] = {
    {0x00, 0xff, 0xff, 0x01, 0xaa, 0x00, 0xff, 0xff, 0xff},	// HV7131B
    {0x04, 0xff, 0xff, 0x01, 0xaa, 0x00, 0xff, 0xff, 0xff},	// CS2102
    {0x06, 0x8d, 0x08, 0x11, 0xaa, 0x00, 0xff, 0xff, 0xff},
    {0x08, 0xff, 0xff, 0x1c, 0x00, 0x00, 0x15, 0xaa, 0x00},	// HDCS2020 ?
    {0x0a, 0xff, 0xff, 0x07, 0xaa, 0xaa, 0xff, 0xff, 0xff},	// MI330 PB330
    {0x0c, 0xff, 0xff, 0x01, 0xaa, 0x00, 0xff, 0xff, 0xff},	// ICM105
    {0x0e, 0x8d, 0x08, 0x03, 0xaa, 0x00, 0xff, 0xff, 0xff},	// pas102
    {0x02, 0xff, 0xff, 0x01, 0xaa, 0x00, 0xff, 0xff, 0xff},
    {0x06, 0x8b, 0xa1, 0x12, 0x80, 0x00, 0x0a, 0x0b, 0xff},	// OV7630c
};

#define SIFTOT 1
static __u8 zcxxi2cSensorSIF[SIFTOT][totval] = {
#if 0
    {0x01, 0xff, 0xff, 0x01, 0xaa, 0x00, 0xff, 0xff, 0xff},	// corrupt with 0x00 hv7131b reg 0 return 0x01 readonly
    {0x05, 0xff, 0xff, 0x01, 0xaa, 0x00, 0xff, 0xff, 0xff},
    {0x07, 0x8d, 0x08, 0x11, 0xaa, 0x00, 0xff, 0xff, 0xff},
    {0x09, 0xff, 0xff, 0x1c, 0x00, 0x00, 0x15, 0xaa, 0x00},	// corrupt with 0x08 hdcs2020 reg 0 return 0x18 readonly
    {0x0b, 0xff, 0xff, 0x07, 0xaa, 0xaa, 0xff, 0xff, 0xff},
    {0x0d, 0xff, 0xff, 0x01, 0x11, 0x00, 0xff, 0xff, 0xff},	// corrupt with 0x0c ICM105 reg 0 is writable
#endif
    {0x0f, 0x8d, 0x08, 0x03, 0xaa, 0x00, 0xff, 0xff, 0xff},	// PAS106 reg3 did not write with 0x0e !conflict PAS102 
};
static __u8 zcxx3wrSensor[][5] = {
    {0x8b, 0xb3, 0x11, 0x12, 0xff},	// HV7131R
    {0x8b, 0x91, 0x14, 0x15, 0x16},
    {0x8b, 0xe0, 0x14, 0x15, 0x16},
    {0, 0, 0, 0, 0}
};
static __u16 zcxx_i2cRead(struct usb_device *dev, __u8 reg)
{
    __u8 retbyte = 0;
    __u8 retval[] = { 0, 0 };

    spca5xxRegWrite(dev, 0xa0, reg, 0x92, NULL, 0);
    spca5xxRegWrite(dev, 0xa0, 0x02, 0x90, NULL, 0);	// <-read command
    spca5xxRegRead(dev, 0xa1, 0x01, 0x0091, &retbyte, 1);	// read status
    spca5xxRegRead(dev, 0xa1, 0x01, 0x0095, &retval[0], 1);	// read Lowbyte
    spca5xxRegRead(dev, 0xa1, 0x01, 0x0096, &retval[1], 1);	// read Hightbyte
    return ((retval[1] << 8) | retval[0]);
}
static __u8 zcxx_i2cWrite(struct usb_device *dev, __u8 reg, __u8 valL,
			  __u8 valH)
{
    __u8 retbyte = 0;
    spca5xxRegWrite(dev, 0xa0, reg, 0x92, NULL, 0);
    spca5xxRegWrite(dev, 0xa0, valL, 0x93, NULL, 0);
    spca5xxRegWrite(dev, 0xa0, valH, 0x94, NULL, 0);
    spca5xxRegWrite(dev, 0xa0, 0x01, 0x90, NULL, 0);	// <-write command
    spca5xxRegRead(dev, 0xa1, 0x01, 0x0091, &retbyte, 1);	// read status
    return (retbyte);
}
static int zcxx_probeSensor(struct gspca_device *gspca_dev)
{
    int i, j;
    struct usb_device *dev = gspca_dev->dev;
    __u8 retbyte = 0;
    __u16 checkword = 0;
   __u16 checkid = 0xffff;
/* check i2c */
/* check SIF */
    for (i = 0; i < SIFTOT; i++) {
        checkword = 0;
	spca5xxRegWrite(dev, 0xa0, 0x01, 0x0000, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, zcxxi2cSensorSIF[i][SensorId],
			0x0010, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x01, 0x0001, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x03, 0x0012, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x01, 0x0012, NULL, 0);
	//wait_ms(2);
	if (zcxxi2cSensorSIF[i][reg8d] == 0x8d){
	    spca5xxRegWrite(dev, 0xa0, zcxxi2cSensorSIF[i][val8d],
			    0x008d, NULL, 0);
	wait_ms(150);
	checkword = ((zcxx_i2cRead(dev,0x00) & 0x0f) << 4) 
	               | ((zcxx_i2cRead(dev,0x01) & 0xf0) >> 4 );
	
	    if (checkword == 0x07){
	        spca5xxRegWrite(dev, 0xa0, 0x01, 0x0000, NULL, 0);
		spca5xxRegWrite(dev, 0xa0, 0x03, 0x003a, NULL, 0);
		spca5xxRegWrite(dev, 0xa0, 0x0c, 0x003b, NULL, 0);
		spca5xxRegWrite(dev, 0xa0, 0x08, 0x0038, NULL, 0);
	    return zcxxi2cSensorSIF[i][SensorId];
	    } 	    
	}
	retbyte =
	    zcxx_i2cWrite(dev, zcxxi2cSensorSIF[i][SensorReg1],
			  zcxxi2cSensorSIF[i][valSreg1L],
			  zcxxi2cSensorSIF[i][valSreg1H]);
	//wait_ms(2);
	retbyte =
	    (zcxx_i2cRead(dev, zcxxi2cSensorSIF[i][SensorReg1])) &
	    0xff;
	//wait_ms(2);
	PDEBUG(1, "sensor answer1  %d ", retbyte);
	if (retbyte != zcxxi2cSensorSIF[i][valSreg1L])
	    continue;
	if (zcxxi2cSensorSIF[i][SensorReg2] != 0xff) {
	    retbyte =
		zcxx_i2cWrite(dev,
			      zcxxi2cSensorSIF[i][SensorReg2],
			      zcxxi2cSensorSIF[i][valSreg2L],
			      zcxxi2cSensorSIF[i][valSreg2H]);
	    retbyte =
		(zcxx_i2cRead
		 (dev, zcxxi2cSensorSIF[i][SensorReg2])) & 0xff;
	    PDEBUG(1, "sensor answer2  %d ", retbyte);
	    if (retbyte == zcxxi2cSensorSIF[i][valSreg2L])
		return zcxxi2cSensorSIF[i][SensorId];
	}
    }
/* check VGA */
    for (i = 0; i < VGATOT; i++) {
	if ((zcxxi2cSensor[i][reg8d] == 0x8b)
	    && (zcxxi2cSensor[i][val8d] == 0xa1)) {
	    /* code for stephane sensor ov7630c */
	    checkword = 0;
	    spca5xxRegWrite(dev, 0xa0, 0x01, 0x0000, NULL, 0);
	    spca5xxRegWrite(dev, 0xa0, 0x01, 0x0001, NULL, 0);
	    spca5xxRegWrite(dev, 0xa0, zcxxi2cSensor[i][SensorId],
			    0x0010, NULL, 0);
	    spca5xxRegWrite(dev, 0xa0, zcxxi2cSensor[i][val8d],
			    0x008b, NULL, 0);
	    spca5xxRegWrite(dev, 0xa0, 0x08, 0x008d, NULL, 0);
	    wait_ms(150);
	    spca5xxRegWrite(dev, 0xa0, 0x01, 0x0012, NULL, 0);
	    retbyte =
		zcxx_i2cWrite(dev, zcxxi2cSensor[i][SensorReg1],
			      zcxxi2cSensor[i][valSreg1L],
			      zcxxi2cSensor[i][valSreg1H]);

	    retbyte =
		(zcxx_i2cRead(dev, zcxxi2cSensor[i][SensorReg2]))
		& 0xff;
	    checkword = retbyte << 8;
	    checkword |= ((zcxx_i2cRead(dev, 0x0b)) & 0xff);
	    if (checkword == 0x7631) {
		PDEBUG(0, "sensor answervga  0x%04X ", checkword);
		spca5xxRegWrite(dev, 0xa0,
				zcxxi2cSensor[i][SensorId], 0x0010, NULL,
				0);
		return zcxxi2cSensor[i][SensorId];
	    }
	    continue;
	}
	spca5xxRegWrite(dev, 0xa0, 0x01, 0x0000, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, zcxxi2cSensor[i][SensorId],
			0x0010, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x01, 0x0001, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x03, 0x0012, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x01, 0x0012, NULL, 0);
	wait_ms(2);
	if (zcxxi2cSensor[i][reg8d] == 0x8d)
	    spca5xxRegWrite(dev, 0xa0, zcxxi2cSensor[i][val8d],
			    0x008d, NULL, 0);
	retbyte =
	    zcxx_i2cWrite(dev, zcxxi2cSensor[i][SensorReg1],
			  zcxxi2cSensor[i][valSreg1L],
			  zcxxi2cSensor[i][valSreg1H]);
	if (zcxxi2cSensor[i][SensorReg2] != 0xff) {
	    retbyte =
		zcxx_i2cWrite(dev, zcxxi2cSensor[i][SensorReg2],
			      zcxxi2cSensor[i][valSreg2L],
			      zcxxi2cSensor[i][valSreg2H]);
	    retbyte =
		(zcxx_i2cRead(dev, zcxxi2cSensor[i][SensorReg2]))
		& 0xff;
	} else {
	checkid = (zcxx_i2cRead(dev,0x00)) & 0xff;
	PDEBUG(1, "check sensor id  0x%04X ", checkid);
	    retbyte =
		(zcxx_i2cRead(dev, zcxxi2cSensor[i][SensorReg1]))
		& 0xff;
	if(checkid == 2) // Who is here, that is a surprised :)
	        continue ;
	}

	PDEBUG(1, "sensor answervga  %d ", retbyte);
	if (retbyte != 0)
	    return zcxxi2cSensor[i][SensorId];
    }
/* check 3 wires bus */
    i = 0;
    while (zcxx3wrSensor[i][0]) {
	spca5xxRegWrite(dev, 0xa0, 0x02, 0x0010, NULL, 0);
	spca5xxRegRead(dev, 0xa1, 0x01, 0x0010, &retbyte, 1);
	spca5xxRegWrite(dev, 0xa0, 0x01, 0x0000, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x00, 0x0010, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x01, 0x0001, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, zcxx3wrSensor[i][1],
			zcxx3wrSensor[i][0], NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x03, 0x0012, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x01, 0x0012, NULL, 0);
	spca5xxRegWrite(dev, 0xa0, 0x05, 0x0012, NULL, 0);
	for (j = 2; j < 5; j++) {
	    if (zcxx3wrSensor[i][j] != 0xff) {
		retbyte =
		    (zcxx_i2cRead(dev, zcxx3wrSensor[i][j])) &
		    0xff;
		if (retbyte != 0)
		    return (i | 0x10);
	    }
	}
	if (zcxx3wrSensor[i][1] == 0x91) {
	    /* check Aurelien Jurgen sensor */
	    checkword = 0;
	    spca5xxRegWrite(dev, 0xa0, 0x02, 0x0010, NULL, 0);
	    spca5xxRegRead(dev, 0xa1, 0x01, 0x000b, &retbyte, 1);
	    checkword = retbyte << 8;
	    spca5xxRegRead(dev, 0xa1, 0x01, 0x000a, &retbyte, 1);
	    checkword |= retbyte;
	    PDEBUG(1, "sensor 3w Vga ??? 0x%04X ", (checkword & 0xffff));
	    if (checkword == 0xc001 || checkword == 0xe001 || checkword == 0x8001) {
		gspca_dev->camera.chip_revision = checkword;
		spca5xxRegRead(dev, 0xa1, 0x01, 0x0010, &retbyte,
			       1);
		spca5xxRegWrite(dev, 0xa0, 0x01, 0x0000, NULL, 0);
		spca5xxRegWrite(dev, 0xa0, 0x0d, 0x003a, NULL, 0);
		spca5xxRegWrite(dev, 0xa0, 0x02, 0x003b, NULL, 0);
		spca5xxRegWrite(dev, 0xa0, 0x00, 0x0038, NULL, 0);
		return 0x13;
	    }

	}
	i++;
    }
    return (-1);
}
 

static __u16 zc3xxWriteVector(struct usb_device *dev, __u16 data[][3])
{
    int err = 0;
    int i = 0;
    __u8 buffread;
    while (data[i][0]) {
	if (data[i][0] == 0xa0) {
	    /* write registers */
	    spca5xxRegWrite(dev, data[i][0], data[i][1], data[i][2], NULL,
			    0);
	} else {
	    /* read status */
	    spca5xxRegRead(dev, data[i][0], data[i][1], data[i][2],
			   &buffread, 1);
	}
	i++;
	udelay(1000);
    }

    return err;
}

#define CLAMP(x) (unsigned char)(((x)>0xFF)?0xff:(((x)<1)?1:(x)))

static __u8 Tgamma[16] =
    { 0x13, 0x38, 0x59, 0x79, 0x92, 0xa7, 0xb9, 0xc8, 0xd4, 0xdf, 0xe7,
0xee, 0xf4, 0xf9, 0xfc, 0xff };
static __u8 Tgradient[16] =
    { 0x26, 0x22, 0x20, 0x1c, 0x16, 0x13, 0x10, 0x0d, 0x0b, 0x09, 0x07,
0x06, 0x05, 0x04, 0x03, 0x02 };
//static __u8 Tgamma[16]={0x24,0x44,0x64,0x84,0x9d,0xb2,0xc4,0xd3,0xe0,0xeb,0xf4,0xff,0xff,0xff,0xff,0xff}; //CS2102
//static __u8 Tgradient[16]={0x18,0x20,0x20,0x1c,0x16,0x13,0x10,0x0e,0x0b,0x09,0x07,0x00,0x00,0x00,0x00,0x01};

static __u16 zc3xx_initcontrol(struct gspca_device *gspca_dev)
{
	gspca_dev->stream_param.colors = zc3xx_controls[GSPCA_COLOR].default_value;
	gspca_dev->stream_param.brightness = zc3xx_controls[GSPCA_BRIGHT].default_value;
	gspca_dev->stream_param.contrast = zc3xx_controls[GSPCA_CONTRAST].default_value;
	gspca_dev->stream_param.autoexpo = zc3xx_controls[GSPCA_AUTOEXPO].default_value;
	return 0;
}
static __u16 zc3xx_getbrightness(struct gspca_device *gspca_dev)
{   
    return gspca_dev->stream_param.brightness;
}
static __u16 zc3xx_getcontrast(struct gspca_device *gspca_dev)
{
    return gspca_dev->stream_param.contrast ;
}
static void zc3xx_setbrightness(struct gspca_device *gspca_dev)
{
    __u8 brightness;
    brightness = (__u8) gspca_dev->stream_param.brightness;
    spca5xxRegWrite(gspca_dev->dev, 0xa0, brightness, 0x011d, NULL, 0);
    if (brightness < 0x70) {
	spca5xxRegWrite(gspca_dev->dev, 0xa0, brightness + 0x10, 0x018d,
			NULL, 0);
    } else {
	spca5xxRegWrite(gspca_dev->dev, 0xa0, 0x80, 0x018d, NULL, 0);
    }

}
static void zc3xx_setcontrast(struct gspca_device *gspca_dev)
{
    int gm0 = 0;
    int gr0 = 0;
    int index = 0;
    int i;
    /* now get the index of gamma table */
    index = zc3xx_getcontrast(gspca_dev);
    PDEBUG(2, "starting new table index %d ", index);
    for (i = 0; i < 16; i++) {
	gm0 = Tgamma[i] * index >> 2;
	gr0 = Tgradient[i] * index >> 2;
	spca5xxRegWrite(gspca_dev->dev, 0xa0, CLAMP(gm0), 0x0120 + i, NULL, 0);	//brightness
	spca5xxRegWrite(gspca_dev->dev, 0xa0, CLAMP(gr0), 0x0130 + i, NULL, 0);	// contrast
	//PDEBUG(0,"i %d gamma %d gradient %d",i ,Tgamma[i],Tgradient[i]);
    }
}

static __u16 zc3xx_getcolors(struct gspca_device *gspca_dev)
{	
return gspca_dev->stream_param.colors;
}

static void zc3xx_setcolors(struct gspca_device *gspca_dev)
{
	gspca_dev->stream_param.colors = zc3xx_controls[GSPCA_COLOR].default_value;
}
static void zc3xx_setAutobright(struct gspca_device *gspca_dev)
{
    __u8 autoval = 0;
    if (gspca_dev->stream_param.autoexpo)
	autoval = 0x42;
    else
	autoval = 0x02;
    spca5xxRegWrite(gspca_dev->dev, 0xa0, autoval, 0x0180, NULL, 0);

}
static int zc3xx_query_control(struct gspca_device *gspca_dev, struct v4l2_queryctrl *q_ctrl)
{
	int i;
	for (i = 0; i < GSPCATOT_CTRL; i++)
		if(q_ctrl->id == zc3xx_controls[i].id){
			memcpy(q_ctrl,&zc3xx_controls[i],sizeof(struct v4l2_queryctrl));
			return 0;
		}
	return -EINVAL;
}	
static int zc3xx_get_control(struct gspca_device *gspca_dev, struct v4l2_control *ctrl)
{
	switch(ctrl->id){
		case V4L2_CID_BRIGHTNESS:
			ctrl->value = zc3xx_getbrightness(gspca_dev);
		break;
		case V4L2_CID_CONTRAST:
			ctrl->value = zc3xx_getcontrast(gspca_dev);
		break;
		case V4L2_CID_SATURATION:
			ctrl->value = zc3xx_getcolors(gspca_dev);
		break;
		case V4L2_CID_AUTO_WHITE_BALANCE:
			ctrl->value = gspca_dev->stream_param.autoexpo;
		break;
		case V4L2_CID_AUTOGAIN:
			ctrl->value = gspca_dev->stream_param.autoexpo;
		break;
		default:
		return -EINVAL;
	}
	return 0;
}
static int zc3xx_set_control(struct gspca_device *gspca_dev, struct v4l2_control *ctrl)
{
	int i;
	for (i = 0; i < GSPCATOT_CTRL; i++)
		if(ctrl->id == zc3xx_controls[i].id){
			if ( ctrl->value >= zc3xx_controls[i].minimum &&
				ctrl->value <= zc3xx_controls[i].maximum ){
				PDEBUG(0, "Control value %d ",ctrl->value);
				switch(ctrl->id){
					case V4L2_CID_BRIGHTNESS:
						gspca_dev->stream_param.brightness = ctrl->value;
						zc3xx_setbrightness(gspca_dev);
					return 0;
					case V4L2_CID_CONTRAST:
						gspca_dev->stream_param.contrast = ctrl->value;
						zc3xx_setcontrast(gspca_dev);
					return 0;
					case V4L2_CID_SATURATION:
						gspca_dev->stream_param.colors = ctrl->value;
						zc3xx_setcolors(gspca_dev);
					return 0;
					case V4L2_CID_AUTO_WHITE_BALANCE:
						gspca_dev->stream_param.autoexpo = ctrl->value;
						zc3xx_setAutobright(gspca_dev);
					return 0;
					case V4L2_CID_AUTOGAIN:
						gspca_dev->stream_param.autoexpo = ctrl->value;
						zc3xx_setAutobright(gspca_dev);
					return 0;
					default:
						return -EINVAL;
				}
			} else
				return -ERANGE;
		}
	
	return -EINVAL;
}

static int zc3xx_init(struct gspca_device *gspca_dev)
{
    spca5xxRegWrite(gspca_dev->dev, 0xa0, 0x01, 0x0000, NULL, 0);
    return 0;
}
static void set_zc3xxVGA(struct gspca_device *gspca_dev)
{
    struct stream_camera *camera = &gspca_dev->camera;
    
    memset(camera->mode_cam, 0x00, TOTMODE * sizeof(struct mwebcam));
    camera->mode_cam[VGA].width = 640;
    camera->mode_cam[VGA].height = 480;
    camera->mode_cam[VGA].t_palette = P_JPEG;
    camera->mode_cam[VGA].mode = 0;
    camera->mode_cam[CIF].width = 320;
    camera->mode_cam[CIF].height = 240;
    camera->mode_cam[CIF].t_palette = P_JPEG; 
    camera->mode_cam[CIF].mode = 1;
 
}
static void set_zc3xxSIF(struct gspca_device *gspca_dev)
{
    struct stream_camera *camera = &gspca_dev->camera;
    
    memset(camera->mode_cam, 0x00, TOTMODE * sizeof(struct mwebcam));
    camera->mode_cam[SIF].width = 352;
    camera->mode_cam[SIF].height = 288;
    camera->mode_cam[SIF].t_palette = P_JPEG;
    camera->mode_cam[SIF].mode = 0;
    camera->mode_cam[QSIF].width = 176;
    camera->mode_cam[QSIF].height = 144;
    camera->mode_cam[QSIF].t_palette = P_JPEG;
    camera->mode_cam[QSIF].mode = 1;
}

static int zc3xx_config(struct gspca_device *gspca_dev)
{
    struct stream_camera *camera = &gspca_dev->camera;
    int sensor = 0;
    __u8 bsensor = 0;
    gspca_dev->stream_param.qindex = 2;
    camera->epaddr = 0x01;
    camera->transfert = USB_ENDPOINT_XFER_ISOC;
    camera->type = JPGH;
    
    sensor = zcxx_probeSensor(gspca_dev);
    switch (sensor) {
    case -1:
	PDEBUG(0, "Find Sensor UNKNOW_0 force Tas5130");
	gspca_dev->sensor = SENSOR_TAS5130C;
	set_zc3xxVGA(gspca_dev);
	break;
    case 0:
	PDEBUG(0, "Find Sensor HV7131B");
	gspca_dev->sensor = SENSOR_HV7131B;
	set_zc3xxVGA(gspca_dev);
	break;
    case 1:
	PDEBUG(0, "Find Sensor SIF UNKNOW_1");
	break;
    case 0x02:
	PDEBUG(0, "Find Sensor UNKNOW_2");
	break;
    case 0x04:
	PDEBUG(0, "Find Sensor CS2102");
	gspca_dev->sensor = SENSOR_CS2102;
	set_zc3xxVGA(gspca_dev);
	break;
    case 5:
	PDEBUG(0, "Find Sensor SIF UNKNOW_5");
	break;
    case 0x06:
	PDEBUG(0, "Find Sensor OV7630C");
	gspca_dev->sensor = SENSOR_OV7630C;
	set_zc3xxVGA(gspca_dev);
	break;
    case 7:
	PDEBUG(0, "Find Sensor SIF UNKNOW_7");
	break;
    case 0x08:
	PDEBUG(0, "Find Sensor HDCS2020(b)");
	gspca_dev->sensor = SENSOR_HDCS2020b;
	set_zc3xxVGA(gspca_dev);
	break;
    case 9:
	PDEBUG(0, "Find Sensor SIF UNKNOW_9");
	break;
    case 0x0a:
	PDEBUG(0, "Find Sensor PB0330");
	gspca_dev->sensor = SENSOR_PB0330;
	set_zc3xxVGA(gspca_dev);
	break;
    case 0x0b:
	PDEBUG(0, "Find Sensor SIF UNKNOW_b");
	break;
    case 0x0c:
	PDEBUG(0, "Find Sensor ICM105T");
	gspca_dev->sensor = SENSOR_ICM105A;
	set_zc3xxVGA(gspca_dev);
	break;
    case 0x0d:
	PDEBUG(0, "Find Sensor SIF UNKNOW_d");
	break;
    case 0x0e:
	PDEBUG(0, "Find Sensor PAS202BCB");
	gspca_dev->sensor = SENSOR_HDCS2020;
	set_zc3xxVGA(gspca_dev);
	break;
    case 0x0f:
	PDEBUG(0, "Find Sensor PAS106");
	gspca_dev->sensor = SENSOR_PAS106;
	set_zc3xxSIF(gspca_dev);
	break;
    case 0x10:
	PDEBUG(0, "Find Sensor TAS5130");
	gspca_dev->sensor = SENSOR_TAS5130C;
	set_zc3xxVGA(gspca_dev);
	break;
    case 0x11:
	PDEBUG(0, "Find Sensor HV7131R(c)");
	gspca_dev->sensor = SENSOR_HV7131C;
	set_zc3xxVGA(gspca_dev);
	break;
    case 0x12:
	PDEBUG(0, "Find Sensor TAS5130");
	gspca_dev->sensor = SENSOR_TAS5130C;
	set_zc3xxVGA(gspca_dev);
	break;
    case 0x13:
	PDEBUG(0, "Find Sensor MI0360");
	gspca_dev->sensor = SENSOR_PB0330;
	set_zc3xxVGA(gspca_dev);
	break;
    };
    if ((sensor == 0x02) || (sensor == 0x01) || (sensor == 0x05)
	|| (sensor == 0x07) || (sensor == 0x09) || (sensor == 0x0b)
	|| (sensor == 0x0d)) {
	PDEBUG(0,
	       "Our Sensor is unknow at the moment please report mxhaard@free.fr ");
	return -EINVAL;
    }
    if ((sensor == -1) || (sensor == 0x10) || (sensor == 0x12)) {
	spca5xxRegWrite(gspca_dev->dev, 0xa0, 0x02, 0x0010, NULL, 0);
	spca5xxRegRead(gspca_dev->dev, 0xa1, 0x01, 0x0010, &bsensor, 1);
    } else {
	sensor = sensor & 0x0f;
	spca5xxRegWrite(gspca_dev->dev, 0xa0, sensor, 0x0010, NULL, 0);
	spca5xxRegRead(gspca_dev->dev, 0xa1, 0x01, 0x0010, &bsensor, 1);
    }
    /* switch the led off */
    if( sensor == 0x06)
      spca5xxRegWrite(gspca_dev->dev,0xa0,0x01,0x0000,NULL,0);

    return zc3xx_initcontrol(gspca_dev);
}
static void zc3xx_setquality(struct gspca_device *gspca_dev)
{
	struct usb_device *dev = gspca_dev->dev;
    __u8 quality = 0;
    __u8 frxt = 0x30;
    quality = (gspca_dev->stream_param.qindex) & 0x0f;
    spca5xxRegWrite(dev, 0xa0, quality, 0x0008, NULL, 0);
    spca5xxRegWrite(dev, 0xa0, frxt, 0x0007, NULL, 0);
    switch (quality){
    case 0:
    case 1:
    case 2:
    	frxt = 0xff;
    break;
    case 3:
    	frxt = 0xf0;
    break;
    case 4:
    	frxt = 0xe0;
    break;
    case 5:
    	frxt = 0x20;
    break;
    }
    spca5xxRegWrite(gspca_dev->dev, 0xa0, frxt, 0x0018, NULL, 0);
    
}

static void zc3xx_start(struct gspca_device *gspca_dev)
{
    struct usb_device *dev = gspca_dev->dev;
    int err = 0;
    /* Assume start use the good resolution from gspca_dev->mode */
    switch (gspca_dev->sensor) {
    case SENSOR_TAS5130C:
	if (gspca_dev->mode) {
	    /* 320x240 */
	    err = zc3xxWriteVector(dev, tas5130cxx_start_data);
	} else {
	    /* 640x480 */
	    err = zc3xxWriteVector(dev, tas5130cxx_scale_data);
	}
	break;
    case SENSOR_ICM105A:
	if (gspca_dev->mode) {
	    /* 320x240 */
	    err = zc3xxWriteVector(dev, icm105axx_start_data);
	} else {
	    /* 640x480 */
	    err = zc3xxWriteVector(dev, icm105axx_scale_data);
	}
	break;
    case SENSOR_HDCS2020:
	if (gspca_dev->mode) {
	    /* 320x240 */
	    err = zc3xxWriteVector(dev, hdcs2020xx_start_data);
	} else {
	    /* 640x480 */
	    err = zc3xxWriteVector(dev, hdcs2020xx_scale_data);
	}
	break;
    case SENSOR_HDCS2020b:
	if (gspca_dev->mode) {
	    /* 320x240 */
	    err = zc3xxWriteVector(dev, hdcs2020xb_start_data);
	} else {
	    /* 640x480 */
	    err = zc3xxWriteVector(dev, hdcs2020xb_scale_data);
	}
	break;
    case SENSOR_HV7131B:
    err = zcxx_probeSensor(gspca_dev);
	if (gspca_dev->mode) {
	    /* 320x240 */
	    err = zc3xxWriteVector(dev, hv7131bxx_start_data);
	} else {
	    /* 640x480 */
	    err = zc3xxWriteVector(dev, hv7131bxx_scale_data);
	}
	break;
    case SENSOR_HV7131C:	//HV7131R
    err = zcxx_probeSensor(gspca_dev);
	if (gspca_dev->mode) {
	    /* 320x240 */
	    err = zc3xxWriteVector(dev, hv7131cxx_start_data);
	} else {
	    /* 640x480 */
	    err = zc3xxWriteVector(dev, hv7131cxx_scale_data);
	}
	break;
    case SENSOR_PB0330:
	if (gspca_dev->mode) {
	    /* 320x240 */
	    if ((gspca_dev->camera.chip_revision == 0xc001)
		|| (gspca_dev->camera.chip_revision == 0xe001)
		|| (gspca_dev->camera.chip_revision == 0x8001))
		err = zc3xxWriteVector(dev, pb03303x_start_data);
	    else
		err = zc3xxWriteVector(dev, pb0330xx_start_data);
	} else {
	    /* 640x480 */
	    if ((gspca_dev->camera.chip_revision == 0xc001)
		|| (gspca_dev->camera.chip_revision == 0xe001)
		|| (gspca_dev->camera.chip_revision == 0x8001))
		err = zc3xxWriteVector(dev, pb03303x_scale_data);
	    else
		err = zc3xxWriteVector(dev, pb0330xx_scale_data);
	}
	break;
    case SENSOR_CS2102:
	if (gspca_dev->mode) {
	    /* 320x240 */
	    err = zc3xxWriteVector(dev, cs2102_start_data);
	} else {
	    /* 640x480 */
	    err = zc3xxWriteVector(dev, cs2102_scale_data);
	}
	break;
    case SENSOR_OV7630C:
	if (gspca_dev->mode) {
	    /* 320x240 */
	    err = zc3xxWriteVector(dev, ov7630c_start_data);
	} else {
	    /* 640x480 */
	    err = zc3xxWriteVector(dev, ov7630c_scale_data);
	}
	break;
    case SENSOR_PAS106:
    	err= zc3xxWriteVector(dev, pas106b_com_data);
	if (gspca_dev->mode) {
	    /* 176x144 */
	    err = zc3xxWriteVector(dev, pas106b_start_data);	   
	} else {
	    /* 352x288 */
	    err = zc3xxWriteVector(dev, pas106b_scale_data);   
	}
	break;
    }
    zc3xx_setbrightness(gspca_dev);
    zc3xx_setquality(gspca_dev);
    zc3xx_setAutobright(gspca_dev);
}

static void zc3xx_stop0(struct gspca_device *gspca_dev)
{
struct usb_device *dev = gspca_dev->dev;
    switch (gspca_dev->sensor) {
     case SENSOR_PAS106:
                spca5xxRegWrite(dev, 0xa0, 0x01, 0x0000, NULL, 0);
		spca5xxRegWrite(dev, 0xa0, 0x03, 0x003a, NULL, 0);
		spca5xxRegWrite(dev, 0xa0, 0x0c, 0x003b, NULL, 0);
		spca5xxRegWrite(dev, 0xa0, 0x08, 0x0038, NULL, 0);
     break;
    }
}

static void zc3xx_shutdown(struct gspca_device *gspca_dev)
{
    struct usb_device *dev = gspca_dev->dev;
    __u8 buffread;
    spca5xxRegRead(dev, 0xa1, 0x01, 0x0180, &buffread, 1);
    spca5xxRegWrite(dev, 0xa0, 0x00, 0x0180, NULL, 0);
    spca5xxRegWrite(dev, 0xa0, 0x01, 0x0000, NULL, 0);
}

static int zc3xx_sofdetect(struct gspca_device *gspca_dev,struct gspca_rawframe *frame,
			unsigned char *cdata,int *iPix, int seqnum, int *datalength)
{
		
		if (cdata[0] == 0xFF && cdata[1] == 0xD8) {
		    *iPix = 2;	//18 remove 0xff 0xd8;
		    *datalength -= *iPix;
		    return 0;
		    PDEBUG(5,
			   "Zc301 header packet found datalength %d !!",
			   *datalength );
		} 
		*iPix = 0;
		return (seqnum +1);
}
static int 
gspca_attach_vimicro(struct gspca_device *gspca_dev)
{
	gspca_dev->camera.type = JPGH;
	info("USB SPCA5XX camera found.(ZC3XX) ");
	memcpy(&gspca_dev->funct, &fzc3xx, sizeof (struct cam_operation));
return 0;
}
static int
gspca_check_vimicro(struct gspca_device *gspca_dev, __u16 vendor, __u16 product)
{
	switch (vendor) {
	case VENDOR_VIMICRO:
		switch (product) {
		case 0x301b:	/* Wasam 350r */
			gspca_dev->desc = Vimicro;
			goto the_end;
		case 0x303b:	/* Wasam 350r */
			gspca_dev->desc = Vimicro303b;
			goto the_end;
		case 0x305b:	/* Generic */
			gspca_dev->desc = Zc0305b;
			goto the_end;
		case 0x0302:	/* Generic */
			gspca_dev->desc = Zc302;
			goto the_end;
		}
		break;
	case VENDOR_LOGITECH:
		switch (product) {
		case 0x08a0:
			gspca_dev->desc = QCim;
			goto the_end;
		case 0x08a1:
			gspca_dev->desc = QCimA1;
			goto the_end;
		case 0x08a2:	// zc302 chips 
			gspca_dev->desc = LabtecPro;
			goto the_end;
		case 0x08a3:
			gspca_dev->desc = QCchat;
			goto the_end;
		case 0x08a6:
			gspca_dev->desc = LogitechQCim;
			goto the_end;
		case 0x08a7:
			gspca_dev->desc = LogitechQCImage;
			goto the_end;
		case 0x08a9:
			gspca_dev->desc = LogitechNotebookDeluxe;
			goto the_end;
		case 0x08ae:
			gspca_dev->desc = QuickCamNB;
			goto the_end;
		case 0x08ac:
			gspca_dev->desc = LogitechQCCool;
			goto the_end;
		case 0x08ad:
			gspca_dev->desc = LogitechQCCommunicateSTX;
			goto the_end;
		case 0x08aa:
			gspca_dev->desc = LabtecNotebook;
			goto the_end;
		case 0x08b9:
			gspca_dev->desc = QCimB9;
			goto the_end;
		case 0x08d9:
			gspca_dev->desc = QCimconnect;
			goto the_end;
		case 0x08da:
			gspca_dev->desc = QCmessenger;
			goto the_end;
		}
		break;
	case VENDOR_CREATIVE:
		switch (product) {
		case 0x401c:	// zc301 chips 
			gspca_dev->desc = CreativeNX;
			goto the_end;
		case 0x401e:	// zc301 chips 
			gspca_dev->desc = CreativeNxPro;
			goto the_end;
		case 0x4034:	// zc301 chips 
			gspca_dev->desc = CreativeInstant1;
			goto the_end;
		case 0x4035:	// zc301 chips 
			gspca_dev->desc = CreativeInstant2;
			goto the_end;
		case 0x403a:
			gspca_dev->desc = CreativeNxPro2;
			goto the_end;
		case 0x4036:
			gspca_dev->desc = CreativeLive;
			goto the_end;
		case 0x401f:	// zc301 chips 
			gspca_dev->desc = CreativeNotebook;
			goto the_end;
		case 0x4017:	// zc301 chips 
			gspca_dev->desc = CreativeMobile;
			goto the_end;
		}
		break;
	case VENDOR_PHILIPS:
		switch (product) {
		case 0x0325:	/* Low cost Philips Webcam */
			gspca_dev->desc = PhilipsSPC200NC;
			goto the_end;
		case 0x0326:	/* Low cost Philips Webcam */
			gspca_dev->desc = PhilipsSPC300NC;
			goto the_end;
		}
		break;
	case VENDOR_MUSTEK:
		switch (product) {
		case 0xc005:	// zc302 chips 
			gspca_dev->desc = Wcam300A;
			goto the_end;
		case 0xd003:	// zc302 chips 
			gspca_dev->desc = MustekWcam300A;
			goto the_end;
		case 0xd004:	// zc302 chips 
			gspca_dev->desc = WCam300AN;
			goto the_end;
		}
		break;
	case VENDOR_GENIUS:
		switch (product) {
		case 0x7007:	// zc301 chips 
			gspca_dev->desc = GeniusVideoCamV2;
			goto the_end;
		case 0x700c:	// zc301 chips 
			gspca_dev->desc = GeniusVideoCamV3;
			goto the_end;
		case 0x700f:	// zc301 chips 
			gspca_dev->desc = GeniusVideoCamExpressV2b;
			goto the_end;
		}
		break;
	case VENDOR_UINOVATION:
		switch (product) {
		case 0x0a00:	// zc301 chips 
			gspca_dev->desc = WebCam320;
			goto the_end;
		}
		break;
	case VENDOR_FLYCAM:
		switch (product) {
		case 0x0128:
		case 0x8050:	// zc301 chips
			gspca_dev->desc = TyphoonWebshotIIUSB300k;
			goto the_end;
		}
		break;
	case VENDOR_CTX:
		switch (product) {
		case 0x2003:	/* The Webcam built in the CTX M730V TFT-Display, behind an USB-HUB */
			gspca_dev->desc = CTXM730VCam;
			goto the_end;
		}
		break;
	}
	return 0;
      the_end:
	gspca_dev->bridge = BRIDGE_ZC3XX;
	return product;
}

#endif				// ZC3XXUSB_H
