/*
* GSPCA based usb camera driver (currently supports
* yuv native stream spca501a, spca501c, spca505, spca508, spca506
* jpeg native stream spca500, spca551, spca504a, spca504b, spca533a, spca536a,
* 	zc0301, zc0302, zc0301p,
* 	vc301, vc301+, vc302, vc303,
* 	cx11646,
* 	sn9c102p, sn9c105, sn9c120
* bayer native stream spca561a, sn9c101, sn9c102, tv8532 ).
* Z-star Vimicro chips zc0301 zc0301P zc0302
* Sunplus spca501a, spca501c, spca505, spca508, spca506, spca500, spca551, spca504a, spca504b, spca533a, spca536a
* Sonix sn9c101, sn9c102, sn9c102p, sn9c105, sn9c120,
* Conexant cx11646,
* Transvision tv_8532 
* Etoms Et61x151 Et61x251
* Pixart Pac207-BCA-32
*
* Credit:  
* UVC Linux  driver (c) Laurent Pinchard:
* SPCA5xx version by Michel Xhaard <mxhaard@users.sourceforge.net>
* SPCA50x version by Joel Crisp <cydergoth@users.sourceforge.net>
* OmniVision OV511 Camera-to-USB Bridge Driver (c) 1999-2000 Mark W. McClelland
* Kernel 2.6.x port Michel Xhaard && Reza Jelveh (feb 2004)
* Based on the Linux CPiA driver written by Peter Pregler, Scott J. Bertin and Johannes Erdfelt.
* This program is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the License, or (at your
* option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation,
* Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/vmalloc.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/string.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/ctype.h>
#include <linux/pagemap.h>
#include <linux/usb.h>
#include <asm/io.h>
#include <asm/atomic.h>
#include <asm/semaphore.h>
#include <asm/page.h>
#include <asm/uaccess.h>
/* only on 2.6.x */
#include <linux/jiffies.h>
#include <linux/param.h>
#include <linux/moduleparam.h>

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,15)
#include <media/v4l2-common.h>
#endif

static const char version[] = GSPCA_VERSION;
#include "gspcaV2.h"
#define VIMICRO 1
#define DRIVER_VERSION_NUMBER	KERNEL_VERSION(1, 0, 0)

static int video_nr = -1;
static int debug = 0;
module_param(debug, int, 0644);
MODULE_PARM_DESC(debug,
		 "Debug level: 0=warning, 1=memory, 2=init/config,"
		 " 3=usb transfer, 4=v4l2 api, 5=packet transfet");
MODULE_AUTHOR
    ("Michel Xhaard <mxhaard@users.sourceforge.net>");
MODULE_DESCRIPTION("GSPCA USB Camera Driver");
MODULE_LICENSE("GPL");

static struct usb_driver gspca_driver;

static struct palette_list Plist[] = {
	{JPEG, "JPEG"},
	{JPGH, "JPEG"},
	{JPGC, "JPEG"},
	{JPGS, "JPEG"},
	{JPGM, "JPEG"},
	{YUVY, "YUVY"},
	{YYUV, "YYUV"},
	{YUYV, "YUYV"},
	{GREY, "GREY"},
	{GBRG, "GBRG"},
	{SN9C, "SN9C"},
	{GBGR, "GBGR"},
	{S561, "S561"},
	{PGBRG, "GBRG"},
	{-1, NULL}
};
static struct bridge_list Blist[] = {
	{BRIDGE_SPCA505, "SPCA505"},
	{BRIDGE_SPCA506, "SPCA506"},
	{BRIDGE_SPCA501, "SPCA501"},
	{BRIDGE_SPCA508, "SPCA508"},
	{BRIDGE_SPCA504, "SPCA504"},
	{BRIDGE_SPCA500, "SPCA500"},
	{BRIDGE_SPCA504B, "SPCA504B"},
	{BRIDGE_SPCA533, "SPCA533"},
	{BRIDGE_SPCA504C, "SPCA504C"},
	{BRIDGE_SPCA561, "SPCA561"},
	{BRIDGE_SPCA536, "SPCA536"},
	{BRIDGE_SONIX, "SN9C102"},
	{BRIDGE_ZC3XX, "ZC301-2"},
	{BRIDGE_CX11646, "CX11646"},
	{BRIDGE_TV8532, "TV8532"},
	{BRIDGE_ETOMS, "ET61XX51"},
	{BRIDGE_SN9CXXX, "SN9CXXX"},
	{BRIDGE_MR97311, "MR97311"},
	{BRIDGE_PAC207, "PAC207BCA"},
	{-1, NULL}
};
enum {
	UnknownCamera = 0,	// 0
	IntelPCCameraPro,
	IntelCreateAndShare,
	GrandtecVcap,
	ViewQuestM318B,
	ViewQuestVQ110,
	KodakDVC325,
	MustekGsmartMini2,
	MustekGsmartMini3,
	CreativePCCam300,
	DLinkDSC350,		// 10
	CreativePCCam600,
	IntelPocketPCCamera,
	IntelEasyPCCamera,
	ThreeComHomeConnectLite,
	KodakEZ200,
	MaxellMaxPocket,
	AiptekMiniPenCam2,
	AiptekPocketDVII,
	AiptekPenCamSD,
	AiptekMiniPenCam13,	// 20
	MustekGsmartLCD3,
	MustekMDC5500Z,
	MegapixV4,
	AiptekPocketDV,
	HamaUSBSightcam,
	Arowana300KCMOSCamera,
	MystFromOriUnknownCamera,
	AiptekPocketDV3100,
	AiptekPocketCam3M,
	GeniusVideoCAMExpressV2,	// 30
	Flexcam100Camera,
	MustekGsmartLCD2,
	PureDigitalDakota,
	PetCam,
	BenqDC1500,
	LogitechClickSmart420,
	LogitechClickSmart510,
	BenqDC1300,
	HamaUSBSightcam2,
	MustekDV3000,		// 40
	CreativePccam750,
	MaxellCompactPM3,
	BenqDC3410,
	BenqDC1016,
	MicroInnovationIC200,
	LogitechTraveler,
	Flycam100Camera,
	UsbGrabberPV321c,
	ADSInstantVCD,
	Gsmartmini,		// 50
	Jenoptikjdc21lcd,
	LogitechClickSmart310,
	Terratec2move13,
	MustekDV4000,
	AiptekDV3500,
	LogitechClickSmart820,
	Enigma13,
	Sonix6025,
	Epsilon13,
	Nxultra,		//60
	AiptekPocketCam2M,
	DeMonUSBCapture,
	CreativeVista,
	PolaroidPDC2030,
	CreativeNotebook,
	CreativeMobile,
	LabtecPro,
	MustekWcam300A,
	GeniusVideoCamV2,
	GeniusVideoCamV3,
	GeniusVideoCamExpressV2b,
	CreativeNxPro,
	Sonix6029,		//73 74 75
	Vimicro,
	Digitrex2110,
	GsmartD30,
	CreativeNxPro2,
	Bs888e,
	Zc302,
	CreativeNoteBook2,
	AiptekSlim3200,		/* 83 84 85 */
	LabtecWebcam,
	QCExpress,
	ICM532cam,
	MustekGsmart300,
	CreativeLive,		//90
	MercuryDigital,
	Wcam300A,
	CreativeVista3b,
	VeoStingray1,
	VeoStingray2,
	TyphoonWebshotIIUSB300k,	//96
	PolaroidPDC3070,
	QCExpressEtch2,
	QCforNotebook,
	QCim,			//100
	WebCam320,
	AiptekPocketCam4M,
	AiptekPocketDV5100,
	AiptekPocketDV5300,
	SunplusGeneric536,
	QCimA1,
	QCchat,
	QCimB9,
	Labtec929,		//109 110
	Etoms61x151,
	Etoms61x251,
	PalmPixDC85,
	Optimedia,
	ToptroIndus,
	AgfaCl20,
	LogitechQC92c,
	SonixWC311P,
	Concord3045,
	Mercury21,		//120
	CreativeNX,
	CreativeInstant1,
	CreativeInstant2,
	QuickCamNB,
	WCam300AN,
	LabtecWCPlus,
	GeniusVideoCamMessenger,
	Pcam,
	GeniusDsc13,
	MustekMDC4000,		//130
	LogitechQCCommunicateSTX,
	Lic200,
	SweexTas5110,
	Pccam168,
	Sn535,
	Pccam,
	Lic300,
	PolaroidIon80,
	Zc0305b,
	BtcPc380,		//140
	LogitechNotebookDeluxe,
	LabtecNotebook,
	JvcGcA50,
	SmileIntlCamera,
	PcCam350,
	PAC207,
	QtecWb100,
	GeniusGe111,
	Vimicro303b,
	CyberpixS550V,
	GeniusGF112,
	LogitechQCim,
	AiptekSlim3000F,
	CTXM730VCam,
	GeniusVideoCamNB,
	CreativeVistaPlus,
	PhilipsSPC200NC,
	PhilipsSPC700NC,
	SpeedNVC350K,
	Mustek330K,
	PhilipsSPC600NC,
	PhilipsSPC300NC,
	Sonix6019,
	LogitechQCImage,
	Sunplus500c,
	MustekMDC3500,
	LogitechQCCool,
	QCimconnect,
	QCmessenger,
	LastCamera
};
static struct cam_list clist[] = {
	{UnknownCamera, "Unknown"},
	{IntelPCCameraPro, "Intel PC Camera Pro"},
	{IntelCreateAndShare, "Intel Create and Share"},
	{GrandtecVcap, "Grandtec V.cap"},
	{ViewQuestM318B, "ViewQuest M318B"},
	{ViewQuestVQ110, "ViewQuest VQ110"},
	{KodakDVC325, "Kodak DVC-325"},
	{MustekGsmartMini2, "Mustek gSmart mini 2"},
	{MustekGsmartMini3, "Mustek gSmart mini 3"},
	{CreativePCCam300, "Creative PC-CAM 300"},
	{DLinkDSC350, "D-Link DSC-350"},
	{CreativePCCam600, "Creative PC-CAM 600"},
	{IntelPocketPCCamera, "Intel Pocket PC Camera"},
	{IntelEasyPCCamera, "Intel Easy PC Camera"},
	{ThreeComHomeConnectLite, "3Com Home Connect Lite"},
	{KodakEZ200, "Kodak EZ200"},
	{MaxellMaxPocket, "Maxell Max Pocket LEdit. 1.3 MPixels"},
	{AiptekMiniPenCam2, "Aiptek Mini PenCam  2 MPixels"},
	{AiptekPocketDVII, "Aiptek PocketDVII  1.3 MPixels"},
	{AiptekPenCamSD, "Aiptek Pencam SD  2 MPixels"},
	{AiptekMiniPenCam13, "Aiptek mini PenCam 1.3 MPixels"},
	{MustekGsmartLCD3, "Mustek Gsmart LCD 3"},
	{MustekMDC5500Z, "Mustek MDC5500Z"},
	{MegapixV4, "Megapix V4"},
	{AiptekPocketDV, "Aiptek PocketDV "},
	{HamaUSBSightcam, "Hama USB Sightcam 100"},
	{Arowana300KCMOSCamera, "Arowana 300K CMOS Camera"},
	{MystFromOriUnknownCamera, "Unknow Ori Camera"},
	{AiptekPocketDV3100, "Aiptek PocketDV3100+ "},
	{AiptekPocketCam3M, "Aiptek PocketCam  3 M "},
	{GeniusVideoCAMExpressV2, "Genius VideoCAM Express V2"},
	{Flexcam100Camera, "Flexcam 100 Camera"},
	{MustekGsmartLCD2, "Mustek Gsmart LCD 2"},
	{PureDigitalDakota, "Pure Digital Dakota"},
	{PetCam, "PetCam"},
	{BenqDC1500, "Benq DC1500"},
	{LogitechClickSmart420, "Logitech Inc. ClickSmart 420"},
	{LogitechClickSmart510, "Logitech Inc. ClickSmart 510"},
	{BenqDC1300, "Benq DC1300"},
	{HamaUSBSightcam2, "Hama USB Sightcam 100 (2)"},
	{MustekDV3000, "Mustek DV 3000"},
	{CreativePccam750, "Creative PCcam750"},
	{MaxellCompactPM3, "Maxell Compact PC PM3"},
	{BenqDC3410, "Benq DC3410"},
	{BenqDC1016, "Benq DC1016"},
	{MicroInnovationIC200, "Micro Innovation IC200"},
	{LogitechTraveler, "Logitech QuickCam Traveler"},
	{Flycam100Camera, "FlyCam Usb 100"},
	{UsbGrabberPV321c, "Usb Grabber PV321c"},
	{ADSInstantVCD, "ADS Instant VCD"},
	{Gsmartmini, "Mustek Gsmart Mini"},
	{Jenoptikjdc21lcd, "Jenoptik DC 21 LCD"},
	{LogitechClickSmart310, "Logitech ClickSmart 310"},
	{Terratec2move13, "Terratec 2 move 1.3"},
	{MustekDV4000, "Mustek DV4000 Mpeg4"},
	{AiptekDV3500, "Aiptek DV3500 Mpeg4"},
	{LogitechClickSmart820, "Logitech ClickSmart 820"},
	{Enigma13, "Digital Dream Enigma 1.3"},
	{Sonix6025, "Xcam Shanga"},
	{Epsilon13, "Digital Dream Epsilon 1.3"},
	{Nxultra, "Creative Webcam NX ULTRA"},
	{AiptekPocketCam2M, "Aiptek PocketCam 2Mega"},
	{DeMonUSBCapture, "3DeMON USB Capture"},
	{CreativeVista, "Creative Webcam Vista"},
	{PolaroidPDC2030, "Polaroid PDC2030"},
	{CreativeNotebook, "Creative Notebook PD1171"},
	{CreativeMobile, "Creative Mobile PD1090"},
	{LabtecPro, "Labtec Webcam Pro"},
	{MustekWcam300A, "Mustek Wcam300A"},
	{GeniusVideoCamV2, "Genius Videocam V2"},
	{GeniusVideoCamV3, "Genius Videocam V3"},
	{GeniusVideoCamExpressV2b, "Genius Videocam Express V2 Firmware 2"},
	{CreativeNxPro, "Creative Nx Pro"},
	{Sonix6029, "Sonix sn9c10x + Pas106 sensor"},
	{Vimicro, "Z-star Vimicro zc0301p"},
	{Digitrex2110, "ApexDigital Digitrex2110 spca533"},
	{GsmartD30, "Mustek Gsmart D30 spca533"},
	{CreativeNxPro2, "Creative NX Pro FW2"},
	{Bs888e, "Kowa Bs888e MicroCamera"},
	{Zc302, "Z-star Vimicro zc0302"},
	{CreativeNoteBook2, "Creative Notebook PD1170"},
	{AiptekSlim3200, "Aiptek Slim 3200"},
	{LabtecWebcam, "Labtec Webcam"},
	{QCExpress, "QC Express"},
	{ICM532cam, "ICM532 cam"},
	{MustekGsmart300, "Mustek Gsmart 300"},
	{CreativeLive, "Creative Live! "},
	{MercuryDigital, "Mercury Digital Pro 3.1Mp"},
	{Wcam300A, "Mustek Wcamm300A 2"},
	{CreativeVista3b, "Creative Webcam Vista 0x403b"},
	{VeoStingray1, "Veo Stingray 1"},
	{VeoStingray2, "Veo Stingray 2"},
	{TyphoonWebshotIIUSB300k, " Typhoon Webshot II"},
	{PolaroidPDC3070, " Polaroid PDC3070"},
	{QCExpressEtch2, "Logitech QuickCam Express II"},
	{QCforNotebook, "Logitech QuickCam for Notebook"},
	{QCim, "Logitech QuickCam IM"},
	{WebCam320, "Micro Innovation WebCam 320"},
	{AiptekPocketCam4M, "Aiptek Pocket Cam 4M"},
	{AiptekPocketDV5100, "Aiptek Pocket DV5100"},
	{AiptekPocketDV5300, "Aiptek Pocket DV5300"},
	{SunplusGeneric536, "Sunplus Generic spca536a"},
	{QCimA1, "Logitech QuickCam IM + sound"},
	{QCchat, "Logitech QuickCam chat"},
	{QCimB9, "Logitech QuickCam IM ???"},
	{Labtec929, "Labtec Webcam Elch2 "},
	{Etoms61x151, "QCam Sangha"},
	{Etoms61x251, "QCam xxxxxx"},
	{PalmPixDC85, "PalmPix DC85"},
	{Optimedia, "Optimedia TechnoAME"},
	{ToptroIndus, "Toptro Industrial"},
	{AgfaCl20, "Agfa ephoto CL20"},
	{LogitechQC92c, "Logitech QuickCam chat"},
	{SonixWC311P, "Sonix sn9c102P Hv7131R"},
	{Concord3045, "Concord 3045 spca536a"},
	{Mercury21, "Mercury Peripherals Inc."},
	{CreativeNX, "Creative NX"},
	{CreativeInstant1, "Creative Instant P0620"},
	{CreativeInstant2, "Creative Instant P0620D"},
	{QuickCamNB, "Logitech QuickCam for Notebooks"},
	{WCam300AN, "Mustek WCam300AN "},
	{LabtecWCPlus, "Labtec Webcam Plus"},
	{GeniusVideoCamMessenger, "VideoCam Messenger sn9c101 Ov7630"},
	{Pcam, "Mars-Semi Pc-Camera MR97311 MI0360"},
	{GeniusDsc13, "Genius Dsc 1.3 Smart spca504B-P3"},
	{MustekMDC4000, "Mustek MDC4000"},
	{LogitechQCCommunicateSTX, "Logitech QuickCam Communicate STX"},
	{Lic200, "LG LIC-200"},
	{SweexTas5110, "Sweex SIF webcam"},
	{Pccam168, "Sonix PcCam"},
	{Sn535, "Sangha 350k"},
	{Pccam, "Sonix Pccam +"},
	{Lic300, "LG Lic-300"},
	{PolaroidIon80, "Polaroid Ion 80"},
	{Zc0305b, "Generic Zc0305b"},
	{BtcPc380, "Sonix Btc PC380"},
	{LogitechNotebookDeluxe, "Logitech Notebook Deluxe"},
	{LabtecNotebook, "Labtec Webcam Notebook"},
	{JvcGcA50, "JVC GC-A50"},
	{SmileIntlCamera, "Smile International"},
	{PcCam350, "PC-Cam350"},
	{PAC207, "Pixart PAC207-BCA"},
	{QtecWb100, "Qtec Webcam 100"},
	{GeniusGe111, "Genius VideoCam Ge111"},
	{Vimicro303b, "Generic Vimicro 303b"},
	{CyberpixS550V, "Mercury Cyberpix S550V"},
	{GeniusGF112, "Genius GF112"},
	{LogitechQCim, "Logitech QCIM"},
	{AiptekSlim3000F, "Aiptek Slim3000F"},
	{CTXM730VCam, "CTX M730V built in Cam"},
	{GeniusVideoCamNB, "Genius VideoCAM NB"},
	{CreativeVistaPlus, "Creative Webcam Vista Plus"},
	{PhilipsSPC200NC, "Philips SPC200NC "},
	{PhilipsSPC700NC, "Philips SPC700NC "},
	{SpeedNVC350K, "Speed NVC 350K "},
	{Mustek330K, "Mustek Digicam 330K "},
	{PhilipsSPC600NC, "Philips SPC600NC "},
	{PhilipsSPC300NC, "Philips SPC300NC "},
	{Sonix6019, "Sonix VGA Ov7630 "},
	{LogitechQCImage, "Logitech QuickCam Image "},
	{Sunplus500c, "Sunplus CA500C "},
	{MustekMDC3500, "Mustek MDC3500"},
	{LogitechQCCool, "Logitech QuickCam Cool"},
	{QCimconnect, "Logitech QuickCam IM/Connect "},
	{QCmessenger, "Logitech QuickCam Messenger "},
	{-1, NULL}
};
static __devinitdata struct usb_device_id device_table[] = {
	{USB_DEVICE(0x0733, 0x0430)},	/* Intel PC Camera Pro */
	{USB_DEVICE(0x0733, 0x0401)},	/* Intel Create and Share */
	{USB_DEVICE(0x99FA, 0x8988)},	/* Grandtec V.cap */
	{USB_DEVICE(0x0733, 0x0402)},	/* ViewQuest M318B */
	{USB_DEVICE(0x0733, 0x0110)},	/* ViewQuest VQ110 */
	{USB_DEVICE(0x040A, 0x0002)},	/* Kodak DVC-325 */
	{USB_DEVICE(0x055f, 0xc420)},	/* Mustek gSmart Mini 2 */
	{USB_DEVICE(0x055f, 0xc520)},	/* Mustek gSmart Mini 3 */
	{USB_DEVICE(0x041E, 0x400A)},	/* Creative PC-CAM 300 */
	{USB_DEVICE(0x084D, 0x0003)},	/* D-Link DSC-350 */
	{USB_DEVICE(0x041E, 0x400B)},	/* Creative PC-CAM 600 */
	{USB_DEVICE(0x8086, 0x0630)},	/* Intel Pocket PC Camera */
	{USB_DEVICE(0x8086, 0x0110)},	/* Intel Easy PC Camera */
	{USB_DEVICE(0x0506, 0x00df)},	/* 3Com HomeConnect Lite */
	{USB_DEVICE(0x040a, 0x0300)},	/* Kodak EZ200 */
	{USB_DEVICE(0x04fc, 0x504b)},	/* Maxell MaxPocket LE 1.3 */
	{USB_DEVICE(0x08ca, 0x2008)},	/* Aiptek Mini PenCam 2 M */
	{USB_DEVICE(0x08ca, 0x0104)},	/* Aiptek PocketDVII 1.3 */
	{USB_DEVICE(0x08ca, 0x2018)},	/* Aiptek Pencam SD 2M */
	{USB_DEVICE(0x04fc, 0x504a)},	/* Aiptek Mini PenCam 1.3 */
	{USB_DEVICE(0x055f, 0xc530)},	/* Mustek Gsmart LCD 3 */
	{USB_DEVICE(0x055f, 0xc650)},	/* Mustek MDC5500Z */
	{USB_DEVICE(0x052b, 0x1513)},	/* Megapix V4 */
	{USB_DEVICE(0x08ca, 0x0103)},	/* Aiptek PocketDV */
	{USB_DEVICE(0x0af9, 0x0010)},	/* Hama USB Sightcam 100 */
	{USB_DEVICE(0x1776, 0x501c)},	/* Arowana 300K CMOS Camera */
	{USB_DEVICE(0x08ca, 0x0106)},	/* Aiptek Pocket DV3100+ */
	{USB_DEVICE(0x08ca, 0x2010)},	/* Aiptek PocketCam 3M */
	{USB_DEVICE(0x0458, 0x7004)},	/* Genius VideoCAM Express V2 */
	{USB_DEVICE(0x04fc, 0x0561)},	/* Flexcam 100 */
	{USB_DEVICE(0x055f, 0xc430)},	/* Mustek Gsmart LCD 2 */
	{USB_DEVICE(0x04fc, 0xffff)},	/* Pure DigitalDakota */
	{USB_DEVICE(0xabcd, 0xcdee)},	/* Petcam */
	{USB_DEVICE(0x04a5, 0x3008)},	/* Benq DC 1500 */
	{USB_DEVICE(0x046d, 0x0960)},	/* Logitech Inc. ClickSmart 420 */
	{USB_DEVICE(0x046d, 0x0901)},	/* Logitech Inc. ClickSmart 510 */
	{USB_DEVICE(0x04a5, 0x3003)},	/* Benq DC 1300 */
	{USB_DEVICE(0x0af9, 0x0011)},	/* Hama USB Sightcam 100 */
	{USB_DEVICE(0x055f, 0xc440)},	/* Mustek DV 3000 */
	{USB_DEVICE(0x041e, 0x4013)},	/* Creative Pccam750 */
	{USB_DEVICE(0x060b, 0xa001)},	/* Maxell Compact Pc PM3 */
	{USB_DEVICE(0x04a5, 0x300a)},	/* Benq DC3410 */
	{USB_DEVICE(0x04a5, 0x300c)},	/* Benq DC1016 */
	{USB_DEVICE(0x0461, 0x0815)},	/* Micro Innovation IC200 */
	{USB_DEVICE(0x046d, 0x0890)},	/* Logitech QuickCam traveler */
	{USB_DEVICE(0x10fd, 0x7e50)},	/* FlyCam Usb 100 */
	{USB_DEVICE(0x06e1, 0xa190)},	/* ADS Instant VCD */
	{USB_DEVICE(0x055f, 0xc220)},	/* Gsmart Mini */
	{USB_DEVICE(0x0733, 0x2211)},	/* Jenoptik jdc 21 LCD */
	{USB_DEVICE(0x046d, 0x0900)},	/* Logitech Inc. ClickSmart 310 */
	{USB_DEVICE(0x055f, 0xc360)},	/* Mustek DV4000 Mpeg4  */
	{USB_DEVICE(0x08ca, 0x2024)},	/* Aiptek DV3500 Mpeg4  */
	{USB_DEVICE(0x046d, 0x0905)},	/* Logitech ClickSmart820  */
	{USB_DEVICE(0x05da, 0x1018)},	/* Digital Dream Enigma 1.3 */
	{USB_DEVICE(0x0c45, 0x6025)},	/* Xcam Shanga */
	{USB_DEVICE(0x0733, 0x1311)},	/* Digital Dream Epsilon 1.3 */
	{USB_DEVICE(0x041e, 0x401d)},	/* Creative Webcam NX ULTRA */
	{USB_DEVICE(0x08ca, 0x2016)},	/* Aiptek PocketCam 2 Mega */
	{USB_DEVICE(0x0734, 0x043b)},	/* 3DeMon USB Capture aka */
	{USB_DEVICE(0x041E, 0x4018)},	/* Creative Webcam Vista (PD1100) */
	{USB_DEVICE(0x0546, 0x3273)},	/* Polaroid PDC2030 */
	{USB_DEVICE(0x041e, 0x401f)},	/* Creative Webcam Notebook PD1171 */
	{USB_DEVICE(0x041e, 0x4017)},	/* Creative Webcam Mobile PD1090 */
	{USB_DEVICE(0x046d, 0x08a2)},	/* Labtec Webcam Pro */
	{USB_DEVICE(0x055f, 0xd003)},	/* Mustek WCam300A */
	{USB_DEVICE(0x0458, 0x7007)},	/* Genius VideoCam V2 */
	{USB_DEVICE(0x0458, 0x700c)},	/* Genius VideoCam V3 */
	{USB_DEVICE(0x0458, 0x700f)},	/* Genius VideoCam Web V2 */
	{USB_DEVICE(0x041e, 0x401e)},	/* Creative Nx Pro */
	{USB_DEVICE(0x0c45, 0x6029)},	/* spcaCam@150 */
	{USB_DEVICE(0x0c45, 0x6009)},	/* spcaCam@120 */
	{USB_DEVICE(0x0c45, 0x600d)},	/* spcaCam@120 */
	{USB_DEVICE(0x04fc, 0x5330)},	/* Digitrex 2110 */
	{USB_DEVICE(0x055f, 0xc540)},	/* Gsmart D30 */
	{USB_DEVICE(0x0ac8, 0x301b)},	/* Asam Vimicro */
	{USB_DEVICE(0x041e, 0x403a)},	/* Creative Nx Pro 2 */
	{USB_DEVICE(0x055f, 0xc211)},	/* Kowa Bs888e Microcamera */
	{USB_DEVICE(0x0ac8, 0x0302)},	/* Z-star Vimicro zc0302 */
	{USB_DEVICE(0x0572, 0x0041)},	/* Creative Notebook cx11646 */
	{USB_DEVICE(0x08ca, 0x2022)},	/* Aiptek Slim 3200 */
	{USB_DEVICE(0x046d, 0x0921)},	/* Labtec Webcam */
	{USB_DEVICE(0x046d, 0x0920)},	/* QC Express */
	{USB_DEVICE(0x0923, 0x010f)},	/* ICM532 cams */
	{USB_DEVICE(0x055f, 0xc200)},	/* Mustek Gsmart 300 */
	{USB_DEVICE(0x0733, 0x2221)},	/* Mercury Digital Pro 3.1p */
	{USB_DEVICE(0x041e, 0x4036)},	/* Creative Live ! */
	{USB_DEVICE(0x055f, 0xc005)},	/* Mustek Wcam300A */
	{USB_DEVICE(0x041E, 0x403b)},	/* Creative Webcam Vista (VF0010) */
	{USB_DEVICE(0x0545, 0x8333)},	/* Veo Stingray */
	{USB_DEVICE(0x0545, 0x808b)},	/* Veo Stingray */
	{USB_DEVICE(0x10fd, 0x8050)},	/* Typhoon Webshot II USB 300k */
	{USB_DEVICE(0x0546, 0x3155)},	/* Polaroid PDC3070 */
	{USB_DEVICE(0x046d, 0x0928)},	/* Logitech QC Express Etch2 */
	{USB_DEVICE(0x046d, 0x092a)},	/* Logitech QC for Notebook */
	{USB_DEVICE(0x046d, 0x08a0)},	/* Logitech QC IM */
	{USB_DEVICE(0x0461, 0x0a00)},	/* MicroInnovation WebCam320 */
	{USB_DEVICE(0x08ca, 0x2028)},	/* Aiptek PocketCam4M */
	{USB_DEVICE(0x08ca, 0x2042)},	/* Aiptek PocketDV5100 */
	{USB_DEVICE(0x08ca, 0x2060)},	/* Aiptek PocketDV5300 */
	{USB_DEVICE(0x04fc, 0x5360)},	/* Sunplus Generic */
	{USB_DEVICE(0x046d, 0x08a1)},	/* Logitech QC IM 0x08A1 +sound */
	{USB_DEVICE(0x046d, 0x08a3)},	/* Logitech QC Chat */
	{USB_DEVICE(0x046d, 0x08b9)},	/* Logitech QC IM ??? */
	{USB_DEVICE(0x046d, 0x0929)},	/* Labtec Webcam Elch2 */
	{USB_DEVICE(0x10fd, 0x0128)},	/* Typhoon Webshot II USB 300k 0x0128 */
	{USB_DEVICE(0x102c, 0x6151)},	/* Qcam Sangha CIF */
	{USB_DEVICE(0x102c, 0x6251)},	/* Qcam xxxxxx VGA */
	{USB_DEVICE(0x04fc, 0x7333)},	/* PalmPixDC85 */
	{USB_DEVICE(0x06be, 0x0800)},	/* Optimedia */
	{USB_DEVICE(0x2899, 0x012c)},	/* Toptro Industrial */
	{USB_DEVICE(0x06bd, 0x0404)},	/* Agfa CL20 */
	{USB_DEVICE(0x046d, 0x092c)},	/* Logitech QC chat Elch2 */
	{USB_DEVICE(0x0c45, 0x607c)},	/* Sonix sn9c102p Hv7131R */
	{USB_DEVICE(0x0733, 0x3261)},	/* Concord 3045 spca536a */
	{USB_DEVICE(0x0733, 0x1314)},	/* Mercury 2.1MEG Deluxe Classic Cam */
	{USB_DEVICE(0x041e, 0x401c)},	/* Creative NX */
	{USB_DEVICE(0x041e, 0x4034)},	/* Creative Instant P0620 */
	{USB_DEVICE(0x041e, 0x4035)},	/* Creative Instant P0620D */
	{USB_DEVICE(0x046d, 0x08ae)},	/* Logitech QuickCam for Notebooks */
	{USB_DEVICE(0x055f, 0xd004)},	/* Mustek WCam300 AN */
	{USB_DEVICE(0x046d, 0x092b)},	/* Labtec Webcam Plus */
	{USB_DEVICE(0x0c45, 0x602e)},	/* Genius VideoCam Messenger */
	{USB_DEVICE(0x0c45, 0x602c)},	/* Generic Sonix OV7630 */
	{USB_DEVICE(0x093A, 0x050F)},	/* Mars-Semi Pc-Camera */
	{USB_DEVICE(0x0458, 0x7006)},	/* Genius Dsc 1.3 Smart */
	{USB_DEVICE(0x055f, 0xc630)},	/* Mustek MDC4000 */
	{USB_DEVICE(0x046d, 0x08ad)},	/* Logitech QCCommunicate STX */
	{USB_DEVICE(0x0c45, 0x602d)},	/* LIC-200 LG */
	{USB_DEVICE(0x0c45, 0x6005)},	/* Sweex Tas5110 */
	{USB_DEVICE(0x0c45, 0x613c)},	/* Sonix Pccam168 */
	{USB_DEVICE(0x0c45, 0x6130)},	/* Sonix Pccam */
	{USB_DEVICE(0x0c45, 0x60c0)},	/* Sangha Sn535 */
	{USB_DEVICE(0x0c45, 0x60fc)},	/* LG-LIC300 */
	{USB_DEVICE(0x0546, 0x3191)},	/* Polaroid Ion 80 */
	{USB_DEVICE(0x0ac8, 0x305b)},	/* Z-star Vimicro zc0305b */
	{USB_DEVICE(0x0c45, 0x6028)},	/* Sonix Btc Pc380 */
	{USB_DEVICE(0x046d, 0x08a9)},	/* Logitech Notebook Deluxe */
	{USB_DEVICE(0x046d, 0x08aa)},	/* Labtec Webcam  Notebook */
	{USB_DEVICE(0x04f1, 0x1001)},	/* JVC GC A50 */
	{USB_DEVICE(0x0497, 0xc001)},	/* Smile International */
	{USB_DEVICE(0x041e, 0x4012)},	/* PC-Cam350 */
	{USB_DEVICE(0x0ac8, 0x303b)},	/* Vimicro 0x303b */
	{USB_DEVICE(0x093a, 0x2468)},	/* PAC207 */
	{USB_DEVICE(0x093a, 0x2471)},	/* PAC207 Genius VideoCam ge111 */
	{USB_DEVICE(0x093a, 0x2460)},	/* PAC207 Qtec Webcam 100 */
	{USB_DEVICE(0x0733, 0x3281)},	/* Cyberpix S550V */
	{USB_DEVICE(0x093a, 0x2470)},	/* Genius GF112 */
	{USB_DEVICE(0x046d, 0x08a6)},	/* Logitech QCim */
	{USB_DEVICE(0x08ca, 0x2020)},	/* Aiptek Slim 3000F */
	{USB_DEVICE(0x0698, 0x2003)},	/* CTX M730V built in */
	{USB_DEVICE(0x0c45, 0x6001)},	/* Genius VideoCAM NB */
	{USB_DEVICE(0x041E, 0x4028)},	/* Creative Webcam Vista Plus */
	{USB_DEVICE(0x0471, 0x0325)},	/* Philips SPC 200 NC */
	{USB_DEVICE(0x0471, 0x0328)},	/* Philips SPC 700 NC */
	{USB_DEVICE(0x0c45, 0x6040)},	/* Speed NVC 350K */
	{USB_DEVICE(0x055f, 0xc230)},	/* Mustek Digicam 330K */
	{USB_DEVICE(0x0c45, 0x6007)},	/* Sonix sn9c101 + Tas5110D */
	{USB_DEVICE(0x0471, 0x0327)},	/* Philips SPC 600 NC */
	{USB_DEVICE(0x0471, 0x0326)},	/* Philips SPC 300 NC */
	{USB_DEVICE(0x0c45, 0x6019)},	/* Generic Sonix OV7630 */
	{USB_DEVICE(0x0c45, 0x6024)},	/* Generic Sonix Tas5130c */
	{USB_DEVICE(0x046d, 0x08a7)},	/* Logitech QuickCam Image */
	{USB_DEVICE(0x04fc, 0x500c)},	/* Sunplus CA500C */
	{USB_DEVICE(0x055f, 0xc232)},	/* Mustek MDC3500 */
	{USB_DEVICE(0x046d, 0x08ac)},	/* Logitech QuickCam Cool */
	{USB_DEVICE(0x046d, 0x08d9)},	/* Logitech QuickCam IM/Connect */
	{USB_DEVICE(0x046d, 0x08da)},	/* Logitech QuickCam Messenger */
	{USB_DEVICE(0x0000, 0x0000)},	/* MystFromOri Unknow Camera */
	{}			/* Terminating entry */
};

MODULE_DEVICE_TABLE(usb, device_table);

#include "utils/spcausb.h"
#ifdef VIMICRO	
#include "Vimicro/zc3xx.h" 
#endif
static void
gspca_transfert_complete(struct urb *urb, struct pt_regs *regs);
static int
gspca_kill_transfert(struct gspca_device *gspca_dev);
static int
gspca_fill_rawbuf(struct gspca_device *gspca_dev ,struct urb *urb);
/* ------------------------------------------------------------------------
 * Copy from the uvc Linux  driver (c) Laurent Pinchard
 * Video buffers queue management.
 *
 * Video queues is initialized by uvc_queue_init(). The function performs
 * basic initialization of the uvc_video_queue struct and never fails.
 *
 * Video buffer allocation and freeing are performed by uvc_alloc_buffers and
 * uvc_free_buffers respectively. The former acquires the video queue lock,
 * while the later must be called with the lock help (so that allocation can
 * free previously allocated buffers). Trying to free buffers that are mapped
 * to user space will return -EBUSY.
 *
 * Video buffers are managed using two queues. However, unlike most USB video
 * drivers which use an in queue and an out queue, we use a main queue which
 * holds all queued buffers (both 'empty' and 'done' buffers), and an irq
 * queue which holds empty buffers. This design (copied from video-buf)
 * minimizes locking in interrupt, as only one queue is shared between
 * interrupt and user contexts.
 *
 * Use cases
 * ---------
 *
 * Unless stated otherwise, all operations which modify the irq buffers queue
 * are protected by the irq spinlock.
 *
 * 1. The user queues the buffers, starts streaming and dequeues a buffer.
 *
 *    The buffers are added to the main and irq queues. Both operations are
 *    protected by the queue lock, and the latert is protected by the irq
 *    spinlock as well.
 *-------------------------------------------------------------------------
 *    The completion handler fetches a buffer from the irq queue and fills it
 *    with video data. If no buffer is available (irq queue empty), the handler
 *    returns immediately.
 *
 *    When the buffer is full, the completion handler removes it from the irq
 *    queue, marks it as ready (UVC_BUF_STATE_DONE) and wake its wait queue.
 *    At that point, any process waiting on the buffer will be woken up. If a
 *    process tries to dequeue a buffer after it has been marked ready, the
 *    dequeing will succeed immediately.
 *
 * 2. Buffers are queued, user is waiting on a buffer and the device gets
 *    disconnected.
 *
 *    When the device is disconnected, the kernel calls the completion handler
 *    with an appropriate status code. The handler marks all buffers in the
 *    irq queue as being erroneous (UVC_BUF_STATE_ERROR) and wakes them up so
 *    that any process waiting on a buffer gets woken up.
 *
 *    Waking up up the first buffer on the irq list is not enough, as the
 *    process waiting on the buffer might restart the dequeue operation
 *    immediately.
 *
 */

/*
 * Free the video buffers.
 *
 * This function must be called with the queue lock held.
 */
static int gspca_free_buffers(struct gspca_video_queue *queue)
{
	unsigned int i;

	for (i = 0; i < queue->count; ++i) {
		if (queue->buffer[i].vma_use_count != 0)
			return -EBUSY;
	}

	if (queue->count) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
		unsigned long addr = (unsigned long)queue->mem;
		size_t size = queue->count * queue->buffer[0].size;
		while (size > 0) {
			ClearPageReserved(vmalloc_to_page((void*)addr));
			addr += PAGE_SIZE;
			size -= PAGE_SIZE;
		}
#endif
		vfree(queue->mem);
		queue->count = 0;
	}

	return 0;
}

/*
 * Allocate the video buffers.
 *
 * Pages are reserved to make sure they will not be swaped, as they will be
 * filled in URB completion handler.
 * 
 * Buffers will be individually mapped, so they must all be page aligned.
 */
static int gspca_alloc_buffers(struct gspca_video_queue *queue, unsigned int nbuffers,
		unsigned int buflength)
{
	unsigned int bufsize = PAGE_ALIGN(buflength);
	unsigned int i;
	void *mem = NULL;
	int ret;

	if (nbuffers > GSPCA_MAX_VIDEO_BUFFERS)
		nbuffers = GSPCA_MAX_VIDEO_BUFFERS;

	down(&queue->lock);

	if ((ret = gspca_free_buffers(queue)) < 0)
		goto done;

	/* Decrement the number of buffers until allocation succeeds. */
	for (; nbuffers > 0; --nbuffers) {
		mem = vmalloc_32(nbuffers * bufsize);
		if (mem != NULL)
			break;
	}

	if (mem == NULL) {
		ret = -ENOMEM;
		goto done;
	}

	for (i = 0; i < nbuffers; ++i) {
		memset(&queue->buffer[i], 0, sizeof queue->buffer[i]);
		queue->buffer[i].size = bufsize;
		queue->buffer[i].buf.index = i;
		queue->buffer[i].buf.m.offset = i * bufsize;
		queue->buffer[i].buf.length = buflength;
		queue->buffer[i].buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		queue->buffer[i].buf.sequence = 0;
		queue->buffer[i].buf.field = V4L2_FIELD_NONE;
		queue->buffer[i].buf.memory = V4L2_MEMORY_MMAP;
		queue->buffer[i].buf.flags = 0;
		init_waitqueue_head(&queue->buffer[i].wait);
	}

	queue->mem = mem;
	queue->count = nbuffers;
	ret = nbuffers;

done:
	up(&queue->lock);
	return ret;
}

/*
 * VMA operations.
 */
static void gspca_vm_open(struct vm_area_struct *vma)
{
	struct gspca_buffer *buffer = vma->vm_private_data;
	buffer->vma_use_count++;
}

static void gspca_vm_close(struct vm_area_struct *vma)
{
	struct gspca_buffer *buffer = vma->vm_private_data;
	buffer->vma_use_count--;
}

static struct vm_operations_struct gspca_vm_ops = {
	.open		= gspca_vm_open,
	.close		= gspca_vm_close,
};

static int gspca_queue_waiton(struct gspca_buffer *buf, int nonblocking)
{
	if (nonblocking) {
		return (buf->state != GSPCA_BUF_STATE_QUEUED &&
			buf->state != GSPCA_BUF_STATE_ACTIVE)
			? 0 : -EAGAIN;
	}

	return wait_event_interruptible(buf->wait,
		buf->state != GSPCA_BUF_STATE_QUEUED &&
		buf->state != GSPCA_BUF_STATE_ACTIVE);
}

static void gspca_query_buffer(struct gspca_buffer *buf,
		struct v4l2_buffer *v4l2_buf)
{
	memcpy(v4l2_buf, &buf->buf, sizeof *v4l2_buf);

	if(buf->vma_use_count)
		v4l2_buf->flags |= V4L2_BUF_FLAG_MAPPED;
		
	switch (buf->state) {
	case GSPCA_BUF_STATE_ERROR:
	case GSPCA_BUF_STATE_DONE:
		v4l2_buf->flags |= V4L2_BUF_FLAG_DONE;
		break;
	case GSPCA_BUF_STATE_QUEUED:
	case GSPCA_BUF_STATE_ACTIVE:
		v4l2_buf->flags |= V4L2_BUF_FLAG_QUEUED;
		break;
	case GSPCA_BUF_STATE_IDLE:
	default:
		break;
	}
}

/*
 * Dequeue a video buffer. If nonblocking is false, block until a buffer is
 * available.
 */
static int gspca_dequeue_buffer(struct gspca_video_queue *queue,
		struct v4l2_buffer *v4l2_buf, int nonblocking)
{
	struct gspca_buffer *buf;
	int ret = 0;

	if (v4l2_buf->type != V4L2_BUF_TYPE_VIDEO_CAPTURE ||
	    v4l2_buf->memory != V4L2_MEMORY_MMAP) {
	//debug	
		return -EINVAL;
	}

	down(&queue->lock);
	if (list_empty(&queue->mainqueue)) {
	//debug
		ret = -EINVAL;
		goto done;
	}

	buf = list_entry(queue->mainqueue.next, struct gspca_buffer, stream);
	
	if ((ret = gspca_queue_waiton(buf, nonblocking)) < 0)
		goto done;

	switch (buf->state) {
	case GSPCA_BUF_STATE_ERROR:
		ret = -EIO;
	case GSPCA_BUF_STATE_DONE:
		buf->state = GSPCA_BUF_STATE_IDLE;
		break;

	case GSPCA_BUF_STATE_IDLE:
	case GSPCA_BUF_STATE_QUEUED:
	case GSPCA_BUF_STATE_ACTIVE:
	default:
		ret = -EINVAL;
		goto done;
	}

	list_del(&buf->stream);
	gspca_query_buffer(buf, v4l2_buf);

done:
	up(&queue->lock);
	return ret;
}

/*
 * Queue a video buffer. Attempting to queue a buffer that has already been
 * queued will return -EINVAL.
 */
static int gspca_queue_buffer(struct gspca_video_queue *queue, struct v4l2_buffer *v4l2_buf)
{
	struct gspca_buffer *buf;
	unsigned long flags;
	int ret = 0;

	
	if (v4l2_buf->type != V4L2_BUF_TYPE_VIDEO_CAPTURE ||
	    v4l2_buf->memory != V4L2_MEMORY_MMAP) {
		//debug
		return -EINVAL;
	}

	down(&queue->lock);
	if (v4l2_buf->index >= queue->count)  {
		//debug
		ret = -EINVAL;
		goto done;
	}

	buf = &queue->buffer[v4l2_buf->index];
	if (buf->state != GSPCA_BUF_STATE_IDLE) {
		//debug
		ret = -EINVAL;
		goto done;
	}

	buf->state = GSPCA_BUF_STATE_QUEUED;
	buf->buf.bytesused = 0;
	list_add_tail(&buf->stream, &queue->mainqueue);
	spin_lock_irqsave(&queue->irqlock, flags);
	list_add_tail(&buf->queue, &queue->irqqueue);
	spin_unlock_irqrestore(&queue->irqlock, flags);

done:
	up(&queue->lock);
	return ret;
}

static void gspca_queue_init(struct gspca_video_queue *queue)
{
	init_MUTEX(&queue->lock);
	spin_lock_init(&queue->irqlock);
	INIT_LIST_HEAD(&queue->mainqueue);
	INIT_LIST_HEAD(&queue->irqqueue);
}

/*
 * Cancel the video buffers queue.
 *
 * Cancelling the queue marks all buffers on the irq queue as erroneous,
 * wakes them up and remove them from the queue.
 *
 * This function acquires the irq spinlock and can be called from interrupt
 * context.
 */
static void gspca_queue_cancel(struct gspca_video_queue *queue)
{
	struct gspca_buffer *buf;
	unsigned long flags;

	spin_lock_irqsave(&queue->irqlock, flags);
	while (!list_empty(&queue->irqqueue)) {
		buf = list_entry(queue->irqqueue.next, struct gspca_buffer, queue);
		list_del(&buf->queue);
		buf->state = GSPCA_BUF_STATE_ERROR;
		wake_up(&buf->wait);
	}
	spin_unlock_irqrestore(&queue->irqlock, flags);
}

/*
 * Enable or disable the video buffers queue.
 *
 * The queue must be enabled before starting video acquisition and must be
 * disabled after stopping it. This ensures that the video buffers queue
 * state can be properly initialized before buffers are accessed from the
 * interrupt handler.
 *
 * Enabling the video queue initializes parameters (such as sequence number,
 * sync pattern, ...). If the queue is already enabled, return -EBUSY.
 *
 * Disabling the video queue cancels the queue and removes all buffers from
 * the main queue.
 *
 * This function can't be called from interrupt context. Use
 * uvc_queue_cancel() instead.
 */
static int gspca_queue_enable(struct gspca_video_queue *queue, int enable)
{
	unsigned int i;
	int ret = 0;

	down(&queue->lock);
	if (enable) {
		if (queue->streaming) {
			ret = -EBUSY;
			goto done;
		}
		queue->sequence = 0;
		queue->streaming = 1;
	}
	else {
		gspca_queue_cancel(queue);
		INIT_LIST_HEAD(&queue->mainqueue);

		for (i = 0; i < queue->count; ++i)
			queue->buffer[i].state = GSPCA_BUF_STATE_IDLE;

		queue->streaming = 0;
	}

done:
	up(&queue->lock);
	return ret;
}

static struct gspca_buffer *gspca_queue_next_buffer(struct gspca_video_queue *queue,
		struct gspca_buffer *buf)
{
	struct gspca_buffer *nextbuf;
	unsigned long flags;

	spin_lock_irqsave(&queue->irqlock, flags);
	//list_del(&buf->queue);
	if (!list_empty(&queue->irqqueue)){
		nextbuf = list_entry(queue->irqqueue.next, struct gspca_buffer,
					queue);
		list_del(queue->irqqueue.next);
	} else
		nextbuf = NULL;
	spin_unlock_irqrestore(&queue->irqlock, flags);

	//buf->buf.sequence = queue->sequence++;
	//do_gettimeofday(&buf->buf.timestamp);

	//wake_up(&buf->wait);
	return nextbuf;
}
/* ------------------------------------------------------------------------ */
/**
* Endpoint management we assume one isoc endpoint per device
* that is not true some Zoran 0x0572:0x0001 have two
* assume ep adresse is static know 
* FIXME as I don't know how to set the bandwith budget
* we allow the maximum
**/
struct usb_host_endpoint *
gspca_set_isoc_ep(struct gspca_device *gspca_dev)
{
	int nbalt, i, j;
	struct usb_interface *intf;
	struct usb_host_endpoint *ep;
	struct usb_device *dev = gspca_dev->dev;
	struct usb_host_interface *altsetting = NULL;
	int error = 0;
	PDEBUG(0, "enter get iso ep ");
	intf = usb_ifnum_to_if(dev, gspca_dev->iface);
	nbalt = intf->num_altsetting;
	PDEBUG(0, "get iso nbalt %d",nbalt);
/* bandwith budget can be set here */
	for (i = nbalt-1; i; i--) {
		altsetting = &intf->altsetting[i];
		for (j = 0; j < altsetting->desc.bNumEndpoints; ++j) {
			ep = &altsetting->endpoint[j];
			PDEBUG(0, "test ISO EndPoint  %d",ep->desc.bEndpointAddress);
			if ((ep->desc.bEndpointAddress ==
			     (gspca_dev->camera.epaddr | USB_DIR_IN))
			    &&
			    ((ep->desc.
			      bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) ==
			     USB_ENDPOINT_XFER_ISOC)) {
				PDEBUG(0, "ISO EndPoint found 0x%02X AlternateSet %d",
				       ep->desc.bEndpointAddress,i);
				if ((error =
				     usb_set_interface(dev, gspca_dev->iface,
						       i)) < 0) {
					PDEBUG(0, "Set interface err %d",
					       error);
					return NULL;
				}
				gspca_dev->alt = i;
				return ep;
			}
		}
	}
	PDEBUG(0, "FATAL ISO EndPoint not found ");
	return NULL;
}
struct usb_host_endpoint *
gspca_set_bulk_ep(struct gspca_device *gspca_dev)
{
	int j;
	struct usb_interface *intf;
	struct usb_host_endpoint *ep;
	struct usb_device *dev = gspca_dev->dev;
	struct usb_host_interface *altsetting = NULL;
	intf = usb_ifnum_to_if(dev, gspca_dev->iface);
	altsetting = intf->cur_altsetting;
	for (j = 0; j < altsetting->desc.bNumEndpoints; ++j) {
		ep = &altsetting->endpoint[j];
		if ((ep->desc.bEndpointAddress ==
		     (gspca_dev->camera.epaddr | USB_DIR_IN))
		    && ((ep->desc.bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) ==
			USB_ENDPOINT_XFER_BULK)) {
			PDEBUG(0, "BULK EndPoint found %d",
			       ep->desc.bEndpointAddress);
			gspca_dev->alt = altsetting->desc.bInterfaceNumber;
			return ep;
		}
	}
	PDEBUG(0, "FATAL BULK EndPoint not found ");
	return NULL;
}
static int
gspca_set_alt0(struct gspca_device *gspca_dev)
{
	int error = 0;
	if ((error = usb_set_interface(gspca_dev->dev, gspca_dev->iface, 0)) < 0) {
		PDEBUG(0, "Set interface err %d", error);
	}
	gspca_dev->alt = 0;
	return  error;
}
static int
gspca_init_transfert(struct gspca_device *gspca_dev)
{
	struct usb_host_endpoint *ep;
	struct urb *urb;
	__u16 psize;
	int n,fx;
	int error = 0;
	if (gspca_dev->streaming)
		return -EBUSY;
	gspca_dev->curframe = 0;
	switch (gspca_dev->camera.transfert) {
	case USB_ENDPOINT_XFER_ISOC:
		if ((ep = gspca_set_isoc_ep(gspca_dev)) == NULL)
			return -EIO;
		psize = le16_to_cpu(ep->desc.wMaxPacketSize);
		psize = (psize & 0x07ff) * (1 + ((psize >> 11) & 3));
		PDEBUG(0, "packet size %d", psize);
		for (n = 0; n < GSPCA_NUMSBUF; n++) {
			urb = usb_alloc_urb(ISO_FRAMES_PER_DESC, GFP_KERNEL);
			if (!urb) {
				err("init isoc: usb_alloc_urb ret. NULL");
				return -ENOMEM;
			}
			
			gspca_dev->sbuf[n].data = usb_buffer_alloc(gspca_dev->dev,
					psize * ISO_FRAMES_PER_DESC,
					GFP_KERNEL, &urb->transfer_dma);
			
			//gspca_dev->sbuf[n].data = kmalloc((psize+1)* ISO_FRAMES_PER_DESC,GFP_KERNEL);
			
			if(gspca_dev->sbuf[n].data == NULL){
				usb_free_urb(urb);
				gspca_kill_transfert(gspca_dev);
				return -ENOMEM;
			}
			gspca_dev->sbuf[n].urb = urb;
			urb->dev = gspca_dev->dev;
			urb->context = gspca_dev;
			urb->pipe =
			    usb_rcvisocpipe(gspca_dev->dev,
					    ep->desc.bEndpointAddress);
			 urb->transfer_flags = URB_ISO_ASAP | URB_NO_TRANSFER_DMA_MAP;
			//urb->transfer_flags = URB_ISO_ASAP;
			urb->interval = ep->desc.bInterval;
			urb->transfer_buffer = gspca_dev->sbuf[n].data;
			urb->complete = gspca_transfert_complete;
			urb->number_of_packets = ISO_FRAMES_PER_DESC;
			urb->transfer_buffer_length = psize * ISO_FRAMES_PER_DESC;
			for (fx = 0; fx < ISO_FRAMES_PER_DESC; fx++) {
				urb->iso_frame_desc[fx].offset = psize * fx;
				urb->iso_frame_desc[fx].length = psize;
			}
		}
		break;
	case USB_ENDPOINT_XFER_BULK:
		if ((ep = gspca_set_bulk_ep(gspca_dev)) == NULL)
			return -EIO;
		psize = le16_to_cpu(ep->desc.wMaxPacketSize);
		psize = (psize & 0x07ff) * (1 + ((psize >> 11) & 3));
		for (n = 0; n < GSPCA_NUMSBUF; n++) {
			urb = usb_alloc_urb(0, GFP_KERNEL);
			if (!urb) {
				err("init isoc: usb_alloc_urb ret. NULL");
				return -ENOMEM;
			}
			gspca_dev->sbuf[n].data = usb_buffer_alloc(gspca_dev->dev,
					psize * ISO_FRAMES_PER_DESC,
					GFP_KERNEL, &urb->transfer_dma);
			if(gspca_dev->sbuf[n].data == NULL){
				usb_free_urb(urb);
				gspca_kill_transfert(gspca_dev);
				return -ENOMEM;
			}
			gspca_dev->sbuf[n].urb = urb;
			usb_fill_bulk_urb(urb,gspca_dev->dev,
				usb_rcvbulkpipe(gspca_dev->dev, ep->desc.bEndpointAddress),
				gspca_dev->sbuf[n].data, psize * ISO_FRAMES_PER_DESC,
				gspca_transfert_complete,gspca_dev);
				
			}	
		break;
	default:
		PDEBUG(0, "Trying to init transfert without pipe IN");
		return -EIO;
		break;
	}
	/* mark all frame unused */
	for (n = 0; n <  GSPCA_RAWBUFF; n++) {
		gspca_dev->frame[n].grabstate = GSPCA_BUF_STATE_IDLE;
		gspca_dev->frame[n].scanstate = GSPCA_BUF_STATE_IDLE;
	}
	/* start the cam and initialize the tasklet lock */
	gspca_dev->funct.start(gspca_dev);
	gspca_dev->streaming = 1;
	atomic_set(&gspca_dev->in_use,0);
	PDEBUG(0, "Submit URB Now");
	/* Submit the URBs. */
	for (n = 0; n < GSPCA_NUMSBUF; ++n) {
	
		if ((error = usb_submit_urb(gspca_dev->sbuf[n].urb, GFP_KERNEL)) < 0) {
			err("init isoc: usb_submit_urb(%d) ret %d", n, error);
			gspca_kill_transfert(gspca_dev);	
			return error;
		}
	
	}
	return 0;
}
static int
gspca_kill_transfert(struct gspca_device *gspca_dev)
{
	struct urb *urb;
	unsigned int i;
	gspca_dev->streaming = 0;
	for (i = 0; i <  GSPCA_NUMSBUF; ++i) {
		if ((urb = gspca_dev->sbuf[i].urb) == NULL)
			continue;

		usb_kill_urb(urb);
		/* urb->transfer_buffer_length is not touched by USB core, so
		 * we can use it here as the buffer length.
		 */
		if (gspca_dev->sbuf[i].data) {
			// kfree(gspca_dev->sbuf[i].data);
		
			usb_buffer_free(gspca_dev->dev,
				urb->transfer_buffer_length,
				gspca_dev->sbuf[i].data, urb->transfer_dma);
		
			gspca_dev->sbuf[i].data = NULL;
		}

		usb_free_urb(urb);
		gspca_dev->sbuf[i].urb = NULL;
	}
	return 0;
}

/**********************************************************************
* 
* Function processes the finish of the USB transfer by calling 
* function to move data from USB buffer to internal raw buffer
***********************************************************************/
static void
gspca_transfert_complete(struct urb *urb, struct pt_regs *regs)
{
	int i;
	struct gspca_device *gspca_dev = (struct gspca_device *) urb->context;
	int len;
	switch (urb->status) {
	case 0:
		break;
	default:
		PDEBUG(0, "Non-zero status (%d) in isoc "
		       "completion handler.", urb->status);
	case -ENOENT:		/* usb_kill_urb() called. */
	case -ECONNRESET:	/* usb_unlink_urb() called. */
	case -ESHUTDOWN:	/* The endpoint is being disabled. */
		return;
	}
	if (!gspca_dev->dev) {
		PDEBUG(4, "no device ");
		return;
	}
	if (!gspca_dev->user) {
		PDEBUG(4, "device not open");
		return;
	}
	if (!gspca_dev->streaming) {
/* Always get some of these after close but before packet engine stops */
		PDEBUG(4, "hmmm... not streaming, but got interrupt");
		return;
	}
	if (!gspca_dev->present) {
/*  */
		PDEBUG(4, "device disconnected ..., but got interrupt !!");
		return;
	}
/* Copy the data received into our raw buffer */
	if (gspca_dev->curframe >= 0) {
		if((len = gspca_fill_rawbuf(gspca_dev, urb)) < 0){
			PDEBUG(0, "Fatal FillRawBuff error %d",len);
			return;
		}
	}

/*
	 else if (waitqueue_active(&gspca_dev->wq)) 
			wake_up_interruptible(&gspca_dev->wq);
*/
	urb->dev = gspca_dev->dev;
	urb->status = 0;
	if ((i = usb_submit_urb(urb, GFP_ATOMIC)) != 0)
		err("usb_submit_urb() ret %d", i);
	return;
}

static struct gspca_rawframe *
gspca_next_rawframe( struct gspca_device *gspca_dev )
{
	struct  gspca_rawframe *frame = NULL;

	PDEBUG(2, "Frame %d State %d", gspca_dev->curframe,
	       gspca_dev->frame[gspca_dev->curframe].grabstate);
	if (gspca_dev->frame[gspca_dev->curframe].grabstate != GSPCA_BUF_STATE_ERROR) 
		gspca_dev->curframe = (gspca_dev->curframe + 1) % GSPCA_RAWBUFF;
	frame = &gspca_dev->frame[gspca_dev->curframe];
/* Reset some per-rawframe variables used by sof detect*/
	frame->grabstate = GSPCA_BUF_STATE_ACTIVE;
	frame->scanstate = GSPCA_BUF_STATE_ACTIVE;
	frame->ptdata = frame->data;
	frame->scanlength = 0;
	frame->last_packet = -1;
	frame->totlength = 0;
	frame->packet = 0;
	return frame;
}

/* Tasklet function to rearrange pixels data */
#include "gspca_dec.c"

void
gspca_task (unsigned long ptr)
{
	int err;
	struct gspca_rawframe *taskletframe = (struct gspca_rawframe *) ptr;
	taskletframe->scanlength = taskletframe->ptdata - taskletframe->data;
	
	if ((err = gspca_outpicture(taskletframe)) < 0) {
		PDEBUG(2, "frame decoder failed (%d)", err);
		taskletframe->grabstate = GSPCA_BUF_STATE_ERROR;
	} else {
		taskletframe->grabstate = GSPCA_BUF_STATE_DONE;
		PDEBUG(2, "Decode framestate return %d",
		       taskletframe->grabstate);
	}
	
	atomic_set(&taskletframe->dev->in_use,0);
}

/*********************************************************************
time Helper function
**********************************************************************/
static inline unsigned long
gspca_gettimes(void)
{
	u64 times_now;
	times_now = get_jiffies_64();
	return jiffies_to_msecs(times_now);
}
static int
gspca_fill_rawbuf(struct gspca_device *gspca_dev ,struct urb *urb)
{
	unsigned char *cdata;	//Pointer to buffer where we do store next packet
	unsigned char *pData;	//Pointer to buffer where we do store next packet
	int i;
	for (i = 0; i < urb->number_of_packets; i++) {
		int datalength = urb->iso_frame_desc[i].actual_length;
		int st = urb->iso_frame_desc[i].status;
		unsigned long ms_times_now;
		unsigned long ms_times_before;
		struct gspca_rawframe *frame;	//Pointer to frame data
		int sequenceNumber;
		int sof;
		int iPix;	//Offset of pixel data in the ISO packet
		if (st) {
			PDEBUG(0, "ISOC data error: [%d] len=%d, status=%d \n",
			       i, datalength, st);
			continue;
		}
		cdata = ((unsigned char *) urb->transfer_buffer) +
		    urb->iso_frame_desc[i].offset;
/* Check for zero length block or no selected frame buffer */
		if (!datalength || gspca_dev->curframe == -1) {
			gspca_dev->synchro = 0;
			continue;
		}
		PDEBUG(5, "Packet data [%d,%d,%d] Status: %d", datalength, st,
		       urb->iso_frame_desc[i].offset, st);
		frame = &gspca_dev->frame[gspca_dev->curframe];
		if (frame->last_packet == -1) {
/*initialize a new frame */
			sequenceNumber = 0;
		} else {
			sequenceNumber = frame->last_packet;
		}
/* check frame start */
		if ((sof =
		     gspca_dev->funct.sof_detect(gspca_dev, frame, cdata, &iPix,
					       sequenceNumber,
					       &datalength)) < 0)
			continue;
		sequenceNumber = sof;
		PDEBUG(3, "gspca_dev: Packet seqnum = 0x%02x.  curframe=%2d",
		       sequenceNumber, gspca_dev->curframe);
		pData = cdata;
		if (sequenceNumber == 0) {
			PDEBUG(3, "Gspca: Found Frame Start!, framenum = %d",
			       gspca_dev->curframe);
			       if (frame->scanstate == GSPCA_BUF_STATE_ACTIVE) {
			       /* overflow ? */
					ms_times_now = gspca_gettimes();
					if (ms_times_now < gspca_dev->last_times)
						gspca_dev->last_times = 0;
					spin_lock(&gspca_dev->avg_colors.v4l_lock);
					ms_times_before =
					    gspca_dev->last_times +
					    gspca_dev->dtimes;
					spin_unlock(&gspca_dev->avg_colors.v4l_lock);
					if (ms_times_now >= ms_times_before) {
						PDEBUG(2,
						       "Decode frame last %d",
						       (int) (ms_times_now -
							      gspca_dev->
							      last_times));
						gspca_dev->last_times =
						    ms_times_now;
			       			if(!atomic_read(&gspca_dev->in_use)){
							atomic_set(&gspca_dev->in_use,1);
							tasklet_init(&gspca_dev->
							     gspca_tasklet,
							     gspca_task,
							     (unsigned long)
							     frame);
							tasklet_schedule(&gspca_dev->gspca_tasklet);
						} else
							frame->grabstate = GSPCA_BUF_STATE_ERROR;
					} else
						frame->grabstate = GSPCA_BUF_STATE_ERROR;
						
			       		if((frame = gspca_next_rawframe(gspca_dev)) == NULL)
						return -EIO;
			       } else
			       		frame->scanstate = GSPCA_BUF_STATE_ACTIVE;
		}
		if (frame == NULL || frame->scanstate != GSPCA_BUF_STATE_ACTIVE)
			continue;
		frame->last_packet = sequenceNumber;
		pData = cdata + iPix;	// Skip packet header (1 or 10 bytes)
// Consume data
		PDEBUG(5, "Processing packet seq  %d,length %d,totlength %d",
		       frame->last_packet, datalength, frame->totlength);
		/* this copy consume input data from the isoc stream usb2.0 packet can be up to 0x03ff*/
		if ((datalength > 0) && (datalength <= 0x7ff)) {
			memcpy(frame->ptdata, pData, datalength);
			frame->ptdata+= datalength;
			frame->totlength += datalength;
		}       
	}
		
return 0;
}
#if 0
static int
gspca_fill_rawbuf(struct gspca_device *gspca_dev ,struct urb *urb)
{
	unsigned char *cdata;	//Pointer to buffer where we do store next packet
	unsigned char *pData;	//Pointer to buffer where we do store next packet
	int i;
	for (i = 0; i < urb->number_of_packets; i++) {
		int datalength = urb->iso_frame_desc[i].actual_length;
		int st = urb->iso_frame_desc[i].status;
		unsigned long ms_times_now;
		unsigned long ms_times_before;
		struct gspca_rawframe *frame;	//Pointer to frame data
		int sequenceNumber;
		int sof;
		int iPix;	//Offset of pixel data in the ISO packet
		if (st) {
			PDEBUG(0, "ISOC data error: [%d] len=%d, status=%d \n",
			       i, datalength, st);
			continue;
		}
		cdata = ((unsigned char *) urb->transfer_buffer) +
		    urb->iso_frame_desc[i].offset;
/* Check for zero length block or no selected frame buffer */
		if (!datalength || gspca_dev->curframe == -1) {
			gspca_dev->synchro = 0;
			continue;
		}
		PDEBUG(5, "Packet data [%d,%d,%d] Status: %d", datalength, st,
		       urb->iso_frame_desc[i].offset, st);
		frame = &gspca_dev->frame[gspca_dev->curframe];
		if (frame->last_packet == -1) {
/*initialize a new frame */
			sequenceNumber = 0;
		} else {
			sequenceNumber = frame->last_packet;
		}
/* check frame start */
		if ((sof =
		     gspca_dev->funct.sof_detect(gspca_dev, frame, cdata, &iPix,
					       sequenceNumber,
					       &datalength)) < 0)
			continue;
		sequenceNumber = sof;
		PDEBUG(3, "gspca_dev: Packet seqnum = 0x%02x.  curframe=%2d",
		       sequenceNumber, gspca_dev->curframe);
		pData = cdata;
/* Can we find a frame start */
		if (sequenceNumber == 0) {
			PDEBUG(3, "Gspca: Found Frame Start!, framenum = %d",
			       gspca_dev->curframe);
// Start of frame is implicit end of previous frame
// Check for a previous frame and finish it off if one exists
			if (frame->scanstate == GSPCA_BUF_STATE_ACTIVE) {
/* overflow ? */
					ms_times_now = gspca_gettimes();
					if (ms_times_now < gspca_dev->last_times)
						gspca_dev->last_times = 0;
					spin_lock(&gspca_dev->avg_colors.v4l_lock);
					ms_times_before =
					    gspca_dev->last_times +
					    gspca_dev->dtimes;
					spin_unlock(&gspca_dev->avg_colors.v4l_lock);
					if (ms_times_now >= ms_times_before) {
						PDEBUG(2,
						       "Decode frame last %d",
						       (int) (ms_times_now -
							      gspca_dev->
							      last_times));
						gspca_dev->last_times =
						    ms_times_now;
/* Decode the frame */
						if(!atomic_read(&gspca_dev->in_use)){
						atomic_set(&gspca_dev->in_use,1);
						tasklet_init(&gspca_dev->
							     gspca_tasklet,
							     gspca_task,
							     (unsigned long)
							     frame);
						tasklet_schedule(&gspca_dev->
								 gspca_tasklet);
						} else
							frame->grabstate = GSPCA_BUF_STATE_ERROR;
					} else
						frame->grabstate = GSPCA_BUF_STATE_ERROR;
				
				if((frame = gspca_next_rawframe(gspca_dev)) == NULL)
					return -EIO;
			} else
				frame->scanstate = GSPCA_BUF_STATE_ACTIVE;
		}
/* Are we in a frame? */
		if (frame == NULL || frame->scanstate != GSPCA_BUF_STATE_ACTIVE)
			continue;
		frame->last_packet = sequenceNumber;
		pData = cdata + iPix;	// Skip packet header (1 or 10 bytes)
// Consume data
		PDEBUG(5, "Processing packet seq  %d,length %d,totlength %d",
		       frame->last_packet, datalength, frame->totlength);
/* this copy consume input data from the isoc stream usb2.0 packet can be up to 0x03ff*/
		if ((datalength > 0) && (datalength <= 0x7ff)) {
			memcpy(frame->ptdata, pData, datalength);
			frame->ptdata+= datalength;
			frame->totlength += datalength;
		}
	}
	return 0;
}
#endif
/****************************************************************************
*
* RawBuffer management
* Arrangement from spca5xx need to be rewrite 
***************************************************************************/
static void *
rvmalloc(unsigned long size)
{
	void *mem;
	unsigned long adr;
	size = PAGE_ALIGN(size);
	mem = vmalloc_32(size);
	if (!mem)
		return NULL;
	memset(mem, 0, size);	/* Clear the ram out, no junk to the user */
	adr = (unsigned long) mem;
	while ((long) size > 0) {
		SetPageReserved(vmalloc_to_page((void *) adr));
		adr += PAGE_SIZE;
		size -= PAGE_SIZE;
	}
	return mem;
}
static void
rvfree(void *mem, unsigned long size)
{
	unsigned long adr;
	if (!mem)
		return;
	adr = (unsigned long) mem;
	while ((long) size > 0) {
		ClearPageReserved(vmalloc_to_page((void *) adr));
		adr += PAGE_SIZE;
		size -= PAGE_SIZE;
	}
	vfree(mem);
}

static int
gspca_alloc_rawbuff(struct gspca_device *gspca_dev)
{
	int i;
	int err = 0;
	PDEBUG(2, "entered rawbuff alloc");
	mutex_lock(&gspca_dev->buf_lock);
	if (gspca_dev->buf_state == GSPCA_BUF_STATE_ACTIVE){
		PDEBUG(0, "try to alloc rawbuff in used ");
		err = -EBUSY;
		goto out;
	}
	gspca_dev->bufsize = PAGE_ALIGN(gspca_dev->camera.size_limit.maxwidth *
		gspca_dev->camera.size_limit.maxheight * 2);
	if (!gspca_dev->bufsize){
		PDEBUG(0, "try to alloc rawbuff without max height width ? ");
		err = -EINVAL;
		goto out;
	}
	gspca_dev->fbuf = rvmalloc(gspca_dev->bufsize*GSPCA_RAWBUFF);
	if (!gspca_dev->fbuf){
		gspca_dev->buf_state = GSPCA_BUF_STATE_NONE;
		PDEBUG(0, "errored rawbuff alloc ");
		err = -ENOMEM;
		goto out;
	}
	for (i = 0; i < GSPCA_RAWBUFF; i++) {
		gspca_dev->frame[i].dev = gspca_dev;	
		gspca_dev->frame[i].grabstate = GSPCA_BUF_STATE_IDLE;
		gspca_dev->frame[i].scanstate = GSPCA_BUF_STATE_IDLE;
		gspca_dev->frame[i].data = gspca_dev->fbuf + i * gspca_dev->bufsize;
		gspca_dev->frame[i].ptdata = gspca_dev->frame[i].data;	
	}
	
	gspca_dev->buf_state = GSPCA_BUF_STATE_ACTIVE;
      out:
	mutex_unlock(&gspca_dev->buf_lock);
	PDEBUG(2, "leaving rawbuff alloc (%d)",err);
	return err;
}
static void
gspca_dealloc_rawbuff(struct gspca_device *gspca_dev)
{
	int i;
	PDEBUG(2, "entered rawbuff dealloc");
	mutex_lock(&gspca_dev->buf_lock);
	if (gspca_dev->fbuf) {
		rvfree(gspca_dev->fbuf, GSPCA_RAWBUFF * gspca_dev->bufsize);
		gspca_dev->fbuf = NULL;
		for (i = 0; i < GSPCA_RAWBUFF; i++)
			gspca_dev->frame[i].data = NULL;
	}
	gspca_dev->buf_state = GSPCA_BUF_STATE_NONE;
	mutex_unlock(&gspca_dev->buf_lock);
	PDEBUG(2, " raw buff memory deallocated");
}

/****************************************************************************
*
* helper functions
*
****************************************************************************/
static int
gspca_get_capability(struct gspca_device *gspca_dev)
{
	int maxw, maxh, minw, minh;
	int i;
	minw = minh = 255 * 255;
	maxw = maxh = 0;
	for (i = QCIF; i < TOTMODE; i++) {
		if (gspca_dev->camera.mode_cam[i].width) {
			if (maxw < gspca_dev->camera.mode_cam[i].width
			    || maxh < gspca_dev->camera.mode_cam[i].height) {
				maxw = gspca_dev->camera.mode_cam[i].width;
				maxh = gspca_dev->camera.mode_cam[i].height;
			}
			if (minw > gspca_dev->camera.mode_cam[i].width
			    || minh > gspca_dev->camera.mode_cam[i].height) {
				minw = gspca_dev->camera.mode_cam[i].width;
				minh = gspca_dev->camera.mode_cam[i].height;
			}
		}
	}
	gspca_dev->camera.size_limit.maxwidth = maxw;
	gspca_dev->camera.size_limit.maxheight = maxh;
	gspca_dev->camera.size_limit.minwidth = minw;
	gspca_dev->camera.size_limit.minheight = minh;
	PDEBUG(0, "maxw %d maxh %d minw %d minh %d", maxw, maxh, minw, minh);
	return 0;
}
static int
gspca_get_default_mode(struct gspca_device *gspca_dev)
{
	int i,f;
	for (i = QCIF; i < TOTMODE; i++) {
		if (gspca_dev->camera.mode_cam[i].width) {
			gspca_dev->stream_param.width = gspca_dev->camera.mode_cam[i].width;
			gspca_dev->stream_param.height = gspca_dev->camera.mode_cam[i].height;
			for( f = P_BAYER;f; f >>= 1){
				if( gspca_dev->camera.mode_cam[i].t_palette & f){
					gspca_dev->stream_param.format = gspca_dev->camera.mode_cam[i].t_palette;
					break;
				}
			}
			if(f == 0){
				PDEBUG(0,"Fatal palette not found");
				goto err;
			}
			gspca_dev->mode = gspca_dev->camera.mode_cam[i].mode;
			return 0;
		}
	}
	err:
	return -EINVAL;
}

static int
wxh_to_size(int width, int height)
{
	switch (width) {
	case 1024:
		if(height == 768)
			return XVGA;
		break;
	case 800:
		if(height == 600)
			return SVGA;
		break;
	case 640:
		if (height == 480)
			return VGA;
		break;
	case 384:
		if (height == 288)
			return PAL;
		break;
	case 352:
		if (height == 288)
			return SIF;
		break;
	case 320:
		if (height == 240)
			return CIF;
		break;
	case 192:
		if (height == 144)
			return QPAL;
		break;
	case 176:
		if (height == 144)
			return QSIF;
		break;
	case 160:
		if (height == 120)
			return QCIF;
		break;
	}
	return CUSTOM;
}

static int
v4l_to_gspca(__u32 format)
{
	switch (format) {
	case V4L2_PIX_FMT_RGB565:
		return P_RGB16;
	case V4L2_PIX_FMT_RGB24:
	case V4L2_PIX_FMT_BGR24:
		return P_RGB24;
	case V4L2_PIX_FMT_BGR32:
	case V4L2_PIX_FMT_RGB32:
		return P_RGB32;
	case V4L2_PIX_FMT_YUYV: // packed 4.2.2
		return P_YUV422;
	case V4L2_PIX_FMT_YUV420: // planar 4.2.0
		return P_YUV420;
	case V4L2_PIX_FMT_RAW: // Maybe i am the only one using raw :)
		return P_RAW;
	case V4L2_PIX_FMT_MJPEG:
	case V4L2_PIX_FMT_JPEG:
		return P_JPEG;
	case V4L2_PIX_FMT_SBGGR8://every one describe GRBG exept v4l2 ???
		return P_BAYER;
	default:
		return -EINVAL;
	}
}
static __u32
gspca_to_v4l(int format)
{
	switch (format) {
	case P_RGB16:
		return V4L2_PIX_FMT_RGB565;
	case P_RGB24:
		return V4L2_PIX_FMT_RGB24;
	case P_RGB32:
		return V4L2_PIX_FMT_RGB32;
	case P_YUV422: // packed 4.2.2
		return V4L2_PIX_FMT_YUYV;
	case P_YUV420: // planar 4.2.0
		return V4L2_PIX_FMT_YUV420;
	case P_RAW: // Maybe i am the only one using raw :)
		return V4L2_PIX_FMT_RAW;
	case P_JPEG:
		return V4L2_PIX_FMT_MJPEG;
	case P_BAYER://every one describe GRBG exept v4l2 ???
		return V4L2_PIX_FMT_SBGGR8;
	default:
		return -EINVAL;
	}
}
static __u32
get_v4l2_depth ( __u32 format)
{
	switch (format) {
	case V4L2_PIX_FMT_RGB565:
		return 16;
	case V4L2_PIX_FMT_RGB24:
	case V4L2_PIX_FMT_BGR24:
		return 24;
	case V4L2_PIX_FMT_BGR32:
	case V4L2_PIX_FMT_RGB32:
		return 32;
	case V4L2_PIX_FMT_YUYV: // packed 4.2.2
		return 16;
	case V4L2_PIX_FMT_YUV420: // planar 4.2.0
		return 12;
	case V4L2_PIX_FMT_RAW: // Maybe i am the only one using raw :)
		return 8;
	case V4L2_PIX_FMT_MJPEG:
	case V4L2_PIX_FMT_JPEG:
		return 8;
	case V4L2_PIX_FMT_SBGGR8://every one describe GRBG exept v4l2 ???
		return 8;
	default:
		return -EINVAL;
	}
}

static int
gspca_is_compressed ( __u32 format)
{
	switch (format) {
	case V4L2_PIX_FMT_RGB565:	
	case V4L2_PIX_FMT_RGB24:
	case V4L2_PIX_FMT_BGR24:	
	case V4L2_PIX_FMT_BGR32:
	case V4L2_PIX_FMT_RGB32:	
	case V4L2_PIX_FMT_YUYV: // packed 4.2.2
	case V4L2_PIX_FMT_YUV420: // planar 4.2.0
	case V4L2_PIX_FMT_RAW: // Maybe i am the only one using raw :)
	case V4L2_PIX_FMT_SBGGR8://every one describe GRBG exept v4l2 ???
	return 0;
	case V4L2_PIX_FMT_MJPEG:
	case V4L2_PIX_FMT_JPEG:	
	return 1;
	default:
		return 0;
	}
}
static int
gspca_get_size(struct gspca_device *gspca_dev,struct v4l2_format *fmt,int size,int format)
{
	int sizeU,sizeD;
	sizeU = size;
	sizeD = size;
	while ((sizeU != TOTMODE) && sizeD){
		if(sizeD-- > 0){
			if(gspca_dev->camera.mode_cam[sizeD].width && 
			(gspca_dev->camera.mode_cam[sizeD].t_palette & format)){
			fmt->fmt.pix.width = (__u32) gspca_dev->camera.mode_cam[sizeD].width;
			fmt->fmt.pix.height= (__u32) gspca_dev->camera.mode_cam[sizeD].height;
			fmt->fmt.pix.bytesperline = (get_v4l2_depth(fmt->fmt.pix.pixelformat)*fmt->fmt.pix.width ) >> 3;
			fmt->fmt.pix.sizeimage = fmt->fmt.pix.bytesperline*fmt->fmt.pix.height;
			return 0;
			}
				
		} else
			sizeD = 0;
		if(sizeU++ < TOTMODE) {
			if(gspca_dev->camera.mode_cam[sizeU].width &&
			(gspca_dev->camera.mode_cam[sizeU].t_palette & format)){
			fmt->fmt.pix.width = (__u32) gspca_dev->camera.mode_cam[sizeU].width;
			fmt->fmt.pix.height = (__u32) gspca_dev->camera.mode_cam[sizeU].height;
			fmt->fmt.pix.bytesperline = (get_v4l2_depth(fmt->fmt.pix.pixelformat)*fmt->fmt.pix.width ) >> 3;
			fmt->fmt.pix.sizeimage = fmt->fmt.pix.bytesperline*fmt->fmt.pix.height;
			return 0;
			}
		} else
			sizeU = TOTMODE;
	}
	return -EINVAL;
}
static unsigned int
gspca_num_fmt(struct gspca_device *gspca_dev)
{
int f;
int size;
int numfmt = 0;
	/* return number of format for this size */
	size = wxh_to_size(gspca_dev->stream_param.width,gspca_dev->stream_param.height);	
	for( f = P_BAYER;f; f >>= 1)
		if(gspca_dev->camera.mode_cam[size].t_palette & f)
			numfmt++;
	return numfmt;
}
static int
gspca_enum_fmt(struct gspca_device *gspca_dev,struct v4l2_fmtdesc *fmtdesc)
{
	int size, index,f;
	index = 0;
	size = wxh_to_size(gspca_dev->stream_param.width,gspca_dev->stream_param.height);
	for( f = P_BAYER;f; f >>= 1){
		if(gspca_dev->camera.mode_cam[size].t_palette & f){
			if(index == fmtdesc->index){
				fmtdesc->pixelformat = gspca_to_v4l(f);
				if (gspca_is_compressed(f))
					fmtdesc->flags = V4L2_FMT_FLAG_COMPRESSED;
				strlcpy(fmtdesc->description,"Gspca describe",sizeof(fmtdesc->description));
				return 0;
			}
			index++;
		}
	}
return -EINVAL;	
}
static int
gspca_get_buff_size(struct gspca_device *gspca_dev)
{
unsigned int size;
int w,h,depth;
	depth = get_v4l2_depth(gspca_to_v4l(gspca_dev->stream_param.format));
	w=gspca_dev->stream_param.width;
	h=gspca_dev->stream_param.height;
	size = w*h*depth >> 3;
	if(!size)
		return -ENOMEM;
return size;
}
static int
gspca_try_fmt(struct gspca_device *gspca_dev,struct v4l2_format *fmt)
{
int w,h,size,format;
int err,f;
	if (fmt->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;
	w = (int) fmt->fmt.pix.width ;
	h = (int) fmt->fmt.pix.height;
	size = wxh_to_size(w,h);
	format = v4l_to_gspca(fmt->fmt.pix.pixelformat);
	/* all are fine format size exist */
	if((gspca_dev->camera.mode_cam[size].width == w) &&
		(gspca_dev->camera.mode_cam[size].height == h) &&
		(gspca_dev->camera.mode_cam[size].t_palette & format))
		/*the format and size exist*/
		return 0;
	/* is the palette exist ? */
	if(gspca_dev->camera.mode_cam[size].t_palette & format){
		if((err = gspca_get_size(gspca_dev,fmt,size,format)) < 0)
		/*same format nearest size*/
			return err;
	} else {
	/* not available try others ? */
	/*a new format for the same size */
		for( f = P_BAYER;f; f >>= 1){
			if( gspca_dev->camera.mode_cam[size].t_palette & f){
				fmt->fmt.pix.pixelformat = gspca_to_v4l(f);
				fmt->fmt.pix.bytesperline = (get_v4l2_depth(fmt->fmt.pix.pixelformat)*fmt->fmt.pix.width ) >> 3;
				fmt->fmt.pix.sizeimage = fmt->fmt.pix.bytesperline*fmt->fmt.pix.height;
				return 0;
			}
		}
		if(!f){
		/* a new format nearest size */
			for( f = P_BAYER;f; f >>= 1){
				if(gspca_get_size(gspca_dev,fmt,size,f) < 0)
			 		continue;
				else
			  		return 0;
			}
		}
	}
	/* Negociation are always hard !! why not enum capability */
return -EINVAL;
}
static int
gspca_get_fmt(struct gspca_device *gspca_dev,struct v4l2_format *fmt)
{
	if (fmt->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;
	fmt->fmt.pix.width = gspca_dev->stream_param.width;
	fmt->fmt.pix.height = gspca_dev->stream_param.height;
	if((fmt->fmt.pix.pixelformat = gspca_to_v4l(gspca_dev->stream_param.format)) < 0);
		return -EINVAL;
	fmt->fmt.pix.field = V4L2_FIELD_NONE;
	fmt->fmt.pix.bytesperline = (get_v4l2_depth(fmt->fmt.pix.pixelformat)*fmt->fmt.pix.width ) >> 3;
	fmt->fmt.pix.sizeimage = fmt->fmt.pix.bytesperline*fmt->fmt.pix.height;
	fmt->fmt.pix.colorspace = V4L2_COLORSPACE_SRGB;
	fmt->fmt.pix.priv = 0;
}
static int
gspca_set_fmt(struct gspca_device *gspca_dev,struct v4l2_format *fmt)
{
int size,err;
	if (fmt->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;
	if(gspca_dev->streaming)
		return -EBUSY;
	if((err = gspca_try_fmt(gspca_dev,fmt)) < 0)
		return err;
	gspca_dev->stream_param.width=(int) fmt->fmt.pix.width;
	gspca_dev->stream_param.height=(int) fmt->fmt.pix.height;
	size = wxh_to_size(gspca_dev->stream_param.width,gspca_dev->stream_param.height);
	gspca_dev->stream_param.format = v4l_to_gspca(fmt->fmt.pix.pixelformat);
	gspca_dev->mode = gspca_dev->camera.mode_cam[size].mode;
	return 0;
}
static int gspca_stream_on(struct gspca_device *gspca_dev)
{
	int ret;
	if(gspca_dev->present){
		if ((ret = gspca_queue_enable(&gspca_dev->queue, 1)) < 0) 
			return ret;
		if((ret = gspca_init_transfert(gspca_dev)) < 0)
			return ret;
		PDEBUG(0,"Stream ON OK");
		
	} else 
		return -ENODEV;
return 0;
}
static int gspca_stream_off(struct gspca_device *gspca_dev)
{
int err = 0;
	gspca_dev->curframe = -1;
	if (gspca_dev->present) {
		gspca_dev->funct.stopN(gspca_dev);
		gspca_kill_transfert(gspca_dev);
		err = gspca_set_alt0(gspca_dev);
		gspca_queue_enable(&gspca_dev->queue, 0);
		gspca_dev->funct.stop0(gspca_dev);
		PDEBUG(0,"Stream OFF OK");
	} else {
		gspca_kill_transfert(gspca_dev);
		gspca_queue_enable(&gspca_dev->queue, 0);
		err = -ENODEV;
		PDEBUG(0,"Stream OFF No device ??");
		}
return err;
}

static int
gspca_init_source(struct gspca_device *gspca_dev)
{
int err_code;
	if ((err_code = gspca_dev->funct.initialize(gspca_dev)) < 0)
		return -EINVAL;
return 0;
}
/****************************************************************************
*
* File operation
*
****************************************************************************/
static int gspca_open(struct inode *inode, struct file *file);
static int gspca_release(struct inode *inode, struct file *file);
static int gspca_ioctl(struct inode *inode, struct file *file,
		     unsigned int cmd, unsigned long arg);
static ssize_t gspca_read(struct file *file, char __user *data,
		    size_t count, loff_t *ppos);
static int gspca_mmap(struct file *file, struct vm_area_struct *vma);
static unsigned int gspca_poll(struct file *file, poll_table *wait);

static struct file_operations gspca_fops = {
	.owner = THIS_MODULE,
	.open = gspca_open,
	.release = gspca_release,
	.read = gspca_read,
	.mmap = gspca_mmap,
	.ioctl = gspca_ioctl,
	.llseek = no_llseek,
	.poll	= gspca_poll,
};

static int gspca_open(struct inode *inode, struct file *file)
{
	struct video_device *vdev = video_devdata(file);
	struct gspca_device *gspca_dev = video_get_drvdata(vdev);
	int err;
	PDEBUG(2, "opening");
	mutex_lock(&gspca_dev->dev_lock);
/* sanity check disconnect, in use, no memory available */
	err = -ENODEV;
	if (!gspca_dev->present)
		goto out;
	err = -EBUSY;
	if (gspca_dev->user)
		goto out;
	err = -ENOMEM;
	if (gspca_alloc_rawbuff(gspca_dev))
		goto out;
	/* initialize sensor and decoding */
	err = gspca_init_source(gspca_dev);
	if (err != 0) {
		PDEBUG(0, "DEALLOC error on gspca_init_source\n");
		mutex_unlock(&gspca_dev->dev_lock);
		gspca_dealloc_rawbuff(gspca_dev);
		goto out2;
	}
//spca5xx_initDecoder(spca50x);
	gspca_dev->user++;
	file->private_data = vdev;
	      out:
	mutex_unlock(&gspca_dev->dev_lock);
      out2:
	if (err) {
		PDEBUG(2, "Open failed");
	} else {
		PDEBUG(2, "Open done");
	}
	return err;
}
static int gspca_release(struct inode *inode, struct file *file)
{
	struct video_device *vdev = video_devdata(file);
	struct gspca_device *gspca_dev = video_get_drvdata(vdev);
	int err;
	PDEBUG(2, "closing");
	mutex_lock(&gspca_dev->dev_lock);
	gspca_dev->user--;	
	err = gspca_stream_off(gspca_dev);
	gspca_dev->funct.cam_shutdown(gspca_dev);
	down(&gspca_dev->queue.lock);
	if (gspca_free_buffers(&gspca_dev->queue) < 0)
		PDEBUG(0, "Unable to free v4l2 buffers");
	up(&gspca_dev->queue.lock);
	gspca_dealloc_rawbuff(gspca_dev);
	file->private_data = NULL;
	mutex_unlock(&gspca_dev->dev_lock);
	PDEBUG(2, "closed");
return err;		
}
static int
gspca_v4l2_do_ioctl(struct inode *inode, struct file *file,
		     unsigned int cmd, void *arg)
{
	struct video_device *vdev = video_devdata(file);
	struct gspca_device *gspca_dev = video_get_drvdata(vdev);
	int ret;

	switch (cmd) {
	/* Query capabilities */
	case VIDIOC_QUERYCAP:
	{
		struct v4l2_capability *cap = arg;

		memset(cap, 0, sizeof *cap);
		strncpy(cap->driver, "gspca", sizeof cap->driver);
		strncpy(cap->card, clist[gspca_dev->desc].description, 
				strlen(clist[gspca_dev->desc].description)+1);
		strncpy(cap->bus_info, gspca_dev->dev->bus->bus_name,
			sizeof cap->bus_info);
		cap->version = DRIVER_VERSION_NUMBER;
		cap->capabilities = V4L2_CAP_VIDEO_CAPTURE
				  | V4L2_CAP_STREAMING;
	}
		break;

	/* Get, Set & Query control */
	case VIDIOC_QUERYCTRL:
		return gspca_dev->funct.query_ctrl(gspca_dev, (struct v4l2_queryctrl*)arg);

	case VIDIOC_G_CTRL:
		return gspca_dev->funct.get_ctrl(gspca_dev, (struct v4l2_control*)arg);

	case VIDIOC_S_CTRL:
		return gspca_dev->funct.set_ctrl(gspca_dev, (struct v4l2_control*)arg);

	case VIDIOC_QUERYMENU:
		return -EINVAL;

	/* Get, Set & Enum input */
	case VIDIOC_ENUMINPUT:
	{
		struct v4l2_input *input = arg;
		if (input->index != 0)
			return -EINVAL;
		memset(input, 0, sizeof *input);
		strncpy(input->name, Blist[gspca_dev->bridge].name, sizeof input->name);
		input->type = V4L2_INPUT_TYPE_CAMERA;
	}
		break;

	case VIDIOC_G_INPUT:
		*(int*)arg = 0;
		break;

	case VIDIOC_S_INPUT:
		if (*(int*)arg != 0)
			return -EINVAL;

		break;

	/* Try, Get, Set & Enum format */
	case VIDIOC_ENUM_FMT:
	{
		struct v4l2_fmtdesc *fmt = arg;

		if (fmt->type != V4L2_BUF_TYPE_VIDEO_CAPTURE ||
		    fmt->index >= gspca_num_fmt(gspca_dev))
			return -EINVAL;

		return gspca_enum_fmt(gspca_dev,fmt);
	}
		break;

	case VIDIOC_TRY_FMT:
	{
		
		return gspca_try_fmt(gspca_dev, (struct v4l2_format*)arg);
	}

	case VIDIOC_S_FMT:
		return gspca_set_fmt(gspca_dev, (struct v4l2_format*)arg);

	case VIDIOC_G_FMT:
		return gspca_get_fmt(gspca_dev, (struct v4l2_format*)arg);

	/* Get & Set streaming parameters */
	case VIDIOC_G_PARM:
		return -EINVAL;

	case VIDIOC_S_PARM:
		return -EINVAL;
	/* Cropping and scaling */
	case VIDIOC_CROPCAP:
	{
		struct v4l2_cropcap *ccap = arg;
		
		if (ccap->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
			return -EINVAL;

		ccap->bounds.left = 0;
		ccap->bounds.top = 0;
		ccap->bounds.width = gspca_dev->stream_param.width;
		ccap->bounds.height = gspca_dev->stream_param.height;

		ccap->defrect = ccap->bounds;

		ccap->pixelaspect.numerator = 1;
		ccap->pixelaspect.denominator = 1;
		
		break ;
	}
	
	case VIDIOC_G_CROP:
	case VIDIOC_S_CROP:
		return -EINVAL;
	/* Buffers & streaming */
	case VIDIOC_REQBUFS:
	{
		struct v4l2_requestbuffers *rb = arg;
		int bufsize;

		if (rb->type != V4L2_BUF_TYPE_VIDEO_CAPTURE ||
		    rb->memory != V4L2_MEMORY_MMAP)
			return -EINVAL;
		if((bufsize = gspca_get_buff_size(gspca_dev)) < 0)
			return -EINVAL;
		if ((ret = gspca_alloc_buffers(&gspca_dev->queue, rb->count,(unsigned int) bufsize)) < 0)
			return ret;

		rb->count = ret;
	}
		break;

	case VIDIOC_QUERYBUF:
	{
		struct v4l2_buffer *buf = arg;
		struct gspca_buffer *ubuf;

		if (buf->type != V4L2_BUF_TYPE_VIDEO_CAPTURE ||
		    buf->index >= gspca_dev->queue.count)
			return -EINVAL;

		ubuf = &gspca_dev->queue.buffer[buf->index];
		gspca_query_buffer(ubuf, buf);
	}
		break;

	case VIDIOC_QBUF:
		return gspca_queue_buffer(&gspca_dev->queue,
			(struct v4l2_buffer*)arg);

	case VIDIOC_DQBUF:
		return gspca_dequeue_buffer(&gspca_dev->queue,
			(struct v4l2_buffer*)arg, file->f_flags & O_NONBLOCK);

	case VIDIOC_STREAMON:
	{
		int *type = arg;

		if (*type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
			return -EINVAL;

		return gspca_stream_on(gspca_dev);
	}

	case VIDIOC_STREAMOFF:
	{
		int *type = arg;

		if (*type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
			return -EINVAL;

		return gspca_stream_off(gspca_dev);
	}
//#if 0
	/* Analog video standards make no sense for digital cameras. */
	case VIDIOC_ENUMSTD:
		//struct v4l2_standard *std = arg;
		//	if (std->index != 0)
		//		return -EINVAL;
		//	std->id = V4L2_STD_UNKNOWN;
		//	strncpy(std->name, "Unknown", sizeof(std->name));
	break;
	
	case VIDIOC_QUERYSTD:
	case VIDIOC_G_STD:
		//v4l2_std_id *std = arg;
		//	*std = V4L2_STD_UNKNOWN;
	break;;
	case VIDIOC_S_STD:
		//v4l2_std_id *std = arg;
		//	if (*std != V4L2_STD_UNKNOWN)
		//		return -EINVAL;
	break;
//#endif
	case VIDIOC_OVERLAY:

	case VIDIOC_ENUMAUDIO:
	case VIDIOC_ENUMAUDOUT:
	case VIDIOC_ENUMOUTPUT:
		//debug
		return -EINVAL;

	default:
		if ((ret = v4l_compat_translate_ioctl(inode, file, cmd, arg,
			gspca_v4l2_do_ioctl)) == -ENOIOCTLCMD)
			//debug
		return ret;
	}

	return 0;
}

static int gspca_ioctl(struct inode *inode, struct file *file,
		     unsigned int cmd, unsigned long arg)
{
	
	return video_usercopy(inode, file, cmd, arg, gspca_v4l2_do_ioctl);
}

static int gspca_mmap(struct file *file, struct vm_area_struct *vma)
{
	struct video_device *vdev = video_devdata(file);
	struct gspca_device *gspca_dev = video_get_drvdata(vdev);
	struct gspca_buffer *buffer = NULL;
	struct page *page;
	unsigned long addr, start, size;
	unsigned int i;
	int ret = 0;

	start = vma->vm_start;
	size = vma->vm_end - vma->vm_start;

	down(&gspca_dev->queue.lock);

	for (i = 0; i < gspca_dev->queue.count; ++i) {
		if ((gspca_dev->queue.buffer[i].buf.m.offset >> PAGE_SHIFT) == vma->vm_pgoff) {
			buffer = &gspca_dev->queue.buffer[i];
			break;
		}
	}

	if (buffer == NULL || size != buffer->size) {
		ret = -EINVAL;
		goto done;
	}

	/*
	 * - VM_IO marks the area as being an mmaped region for I/O to a
	 *   device. It also prevents the region from being core dumped.
	 */
	vma->vm_flags |= VM_IO;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
	/* Not sure if this is needed. remap_pfn_range() sets VM_RESERVED
	 * in 2.6.14.
	 */
	vma->vm_flags |= VM_RESERVED;
#endif

	addr = (unsigned long)gspca_dev->queue.mem + buffer->buf.m.offset;
	while (size > 0) {
		page = vmalloc_to_page((void*)addr);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
                SetPageReserved(page);
		if ((ret = remap_pfn_range(vma, start, page_to_pfn(page),
					PAGE_SIZE, vma->vm_page_prot)) < 0)
#else
		if ((ret = vm_insert_page(vma, start, page)) < 0)
#endif
			goto done;

		start += PAGE_SIZE;
		addr += PAGE_SIZE;
		size -= PAGE_SIZE;
	}

	vma->vm_ops = &gspca_vm_ops;
	vma->vm_private_data = buffer;
	gspca_vm_open(vma);

done:
	up(&gspca_dev->queue.lock);
	return ret;
}

static unsigned int gspca_poll(struct file *file, poll_table *wait)
{
	struct video_device *vdev = video_devdata(file);
	struct gspca_device *gspca_dev = video_get_drvdata(vdev);
	struct gspca_buffer *buf;
	unsigned int mask = 0;

	down(&gspca_dev->queue.lock);
	if (list_empty(&gspca_dev->queue.mainqueue)) {
		mask |= POLLERR;
		goto done;
	}
	buf = list_entry(gspca_dev->queue.mainqueue.next, struct gspca_buffer, stream);

	poll_wait(file, &buf->wait, wait);
	if (buf->state == GSPCA_BUF_STATE_DONE ||
	    buf->state == GSPCA_BUF_STATE_ERROR)
		mask |= POLLIN | POLLRDNORM;

done:
	up(&gspca_dev->queue.lock);
	return mask;
}

static ssize_t gspca_read(struct file *file, char __user *data,
		    size_t count, loff_t *ppos)
		    {
		    PDEBUG(0, "Not yet implemented ");
		    return -EINVAL;
		    }
/****************************************************************************
*
*  Module routines
*
****************************************************************************/
static struct video_device gspca_template = {
	.owner = THIS_MODULE,
	.name = "GSPCA USB Camera",
	.type = VID_TYPE_CAPTURE,
	.hardware = VID_HARDWARE_GSPCA,
	.fops = &gspca_fops,
	.release = video_device_release,
	.minor = -1,
};

static int gspca_probe_camera(struct gspca_device *gspca_dev)
{
	struct usb_device *dev = gspca_dev->dev;
	__u16 vendor;
	__u16 product;
	
	vendor = le16_to_cpu(dev->descriptor.idVendor);
	product = le16_to_cpu(dev->descriptor.idProduct);
#ifdef VIMICRO	
	if(gspca_check_vimicro(gspca_dev,vendor,product))
		return gspca_attach_vimicro(gspca_dev);
#endif
#ifdef SONIX		
	if(gspca_check_sonix(gspca_dev,vendor,product))
		return gspca_attach_sonix(gspca_dev);
#endif
#if 0
	if(gspca_check_sunplus_jpeg(gspca_dev,vendor,product))
		return gspca_attach_spjepg(gspca_dev);
	if(gspca_check_sunplus(gspca_dev,vendor,product))
		return gspca_attach_sunplus(gspca_dev);
	if(gspca_check_conexant(gspca_dev,vendor,product))
		return gspca_attach_conexant(gspca_dev);
	if(gspca_check_etoms(gspca_dev,vendor,product))
		return gspca_atttach_etoms(gspca_dev);
	if(gspca_check_pixart(gspca_dev,vendor,product))
		return gspca_attach_pixart(gspca_dev);
	if(gspca_check_mars_semi(gspca_dev,vendor,product))
		return gspca_attach_mars(gspca_dev);
	if(gspca_check_transvision(gspca_dev,vendor,product))
		return gspca_attach_transvision(gspca_dev);
#endif
return -ENODEV;	
}
static int
gspca_configure(struct gspca_device *gspca_dev)
{
int err;	
/* Initialise the camera bridge */
	if (gspca_dev->funct.configure(gspca_dev) < 0)
		goto error;
	if((err = gspca_set_alt0(gspca_dev)) < 0)
		return err;
	if (gspca_get_default_mode(gspca_dev) < 0)
		return -EINVAL;
	if (gspca_get_capability(gspca_dev) < 0) {
		err("failed to configure");
		goto error;
	}
/* configure the frame detector with default parameters */
	PDEBUG(2, "gspca Configure done !!");
	return 0;
      error:
	return -EBUSY;
}
static int
gspca_probe(struct usb_interface *intf, const struct usb_device_id *id)
{
	struct usb_interface_descriptor *interface;
	struct gspca_device *gspca_dev;
	int err_probe;
	struct usb_device *dev = interface_to_usbdev(intf);
/* We don't handle multi-config cameras */
	if (dev->descriptor.bNumConfigurations != 1)
		goto nodevice;
	interface = &intf->cur_altsetting->desc;
	if (interface->bInterfaceNumber > 0)
		goto nodevice;
	if ((gspca_dev =
	     kzalloc(sizeof (struct gspca_device), GFP_KERNEL)) == NULL) {
		err("couldn't kzalloc gspca struct");
		goto error;
	}
	gspca_dev->dev = dev;
	gspca_dev->iface = interface->bInterfaceNumber;
	if ((err_probe = gspca_probe_camera(gspca_dev)) < 0) {
		err(" Devices not found !! ");
		goto error;
	}
	if(!gspca_dev->camera.type){
	 	err(" Devices as no stream type !! ");
		goto error;
	 }
	PDEBUG(0, "Camera type %s ", Plist[gspca_dev->camera.type].name);
	if (!gspca_configure(gspca_dev)) {
		gspca_dev->user = 0;
		gspca_dev->last_times = 0;
		gspca_dev->dtimes = 0;
		gspca_dev->stream_param.autoexpo = 1;
		mutex_init(&gspca_dev->dev_lock);	/* to 1 == available */
		mutex_init(&gspca_dev->buf_lock);
		spin_lock_init(&gspca_dev->avg_colors.v4l_lock);
		gspca_dev->buf_state = GSPCA_BUF_STATE_IDLE;
	} else {
		err("Failed to configure camera");
		goto error;
	}
/* Init video stuff */
	gspca_dev->vdev = video_device_alloc();
	if (!gspca_dev->vdev)
		goto error;
	memcpy(gspca_dev->vdev, &gspca_template, sizeof (gspca_template));
	gspca_dev->vdev->dev = &dev->dev;
	video_set_drvdata(gspca_dev->vdev, gspca_dev);
	PDEBUG(2, "setting video device = %p,gspca_dev = %p", gspca_dev->vdev,
	       gspca_dev);
	if (video_register_device(gspca_dev->vdev, VFL_TYPE_GRABBER, video_nr) <
	    0) {
		err("video_register_device failed");
		goto error;
	}
/* test on disconnect */
	gspca_queue_init(&gspca_dev->queue);
	gspca_dev->present = 1;
	usb_set_intfdata(intf, gspca_dev);
	PDEBUG(2, "video_register_device succeeded");
	return 0;
      error:
	if (gspca_dev->vdev) {
		if (gspca_dev->vdev->minor == -1)
			video_device_release(gspca_dev->vdev);
		else
			video_unregister_device(gspca_dev->vdev);
		gspca_dev->vdev = NULL;
	}
	if (gspca_dev) {
		kfree(gspca_dev);
		gspca_dev = NULL;
	}
	return -EIO;
      nodevice:
	return -ENODEV;
}
static void
gspca_disconnect(struct usb_interface *intf)
{
	struct gspca_device *gspca_dev  = usb_get_intfdata(intf);
	int n;
	if (!gspca_dev)
		return;
	mutex_lock(&gspca_dev->dev_lock);
	gspca_dev->present = 0;
	gspca_dev->curframe = -1;
	for (n = 0; n < GSPCA_RAWBUFF; n++)
		gspca_dev->frame[n].grabstate = GSPCA_BUF_STATE_IDLE;
	gspca_kill_transfert(gspca_dev);
	PDEBUG(3, "Disconnect Kill isoc done");
	mutex_unlock(&gspca_dev->dev_lock);
	while (gspca_dev->user) //wait until close
		schedule();
	mutex_lock(&gspca_dev->dev_lock);
	dev_set_drvdata(&intf->dev, NULL);
/* We don't want people trying to open up the device */
	if (gspca_dev->vdev)
		video_unregister_device(gspca_dev->vdev);
	gspca_dev->dev = NULL;
	mutex_unlock(&gspca_dev->dev_lock);
/* Free the memory */
	if (gspca_dev && !gspca_dev->user) {
	// gspca_dealloc_rawbuff(gspca_dev); //should be done by close
		kfree(gspca_dev);
		gspca_dev = NULL;
	}
	PDEBUG(3, "Disconnect complete");
}
static struct usb_driver gspca_driver = {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
	.owner = THIS_MODULE,
#endif
	.name = "gspca",
	.id_table = device_table,
	.probe = gspca_probe,
	.disconnect = gspca_disconnect,
};

static int __init
usb_gspca_init(void)
{
	if (usb_register(&gspca_driver) < 0)
		return -1;
	info("gspca driver %s registered", version);
	return 0;
}
static void __exit
usb_gspca_exit(void)
{
	usb_deregister(&gspca_driver);
	info("driver gspca deregistered");
}

module_init(usb_gspca_init);
module_exit(usb_gspca_exit);
