#ifndef GSPCAV2_H
#define GSPCAV2_H
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/usb.h>
#include <linux/videodev.h>
#include <linux/mutex.h>

/* GSPCA our debug messages */
#ifdef GSPCA_ENABLE_DEBUG
#  define PDEBUG(level, fmt, args...) \
if (debug >= level) info("[%s:%d] " fmt, __PRETTY_FUNCTION__, __LINE__ , ## args)
#else				/* GSPCA_ENABLE_DEBUG */
#  define PDEBUG(level, fmt, args...) do {} while(0)
#endif				/* GSPCA_ENABLE_DEBUG */
/* Maximum number of video buffers. */
#define GSPCA_MAX_VIDEO_BUFFERS	32
/* input ISO_RAWBUFFER */
#define GSPCA_RAWBUFF 2
/* iso buf submit for completion */
#define GSPCA_NUMSBUF 2
#define ISO_FRAMES_PER_DESC  20	/* Default value, should be reasonable */
#define V4L2_PIX_FMT_RAW     v4l2_fourcc('P','R','A','W') /* Damned no one used raw in v4l2 */
/* GSPCA Vid */
#define VENDOR_SONIX 0x0c45
#define VENDOR_ETOMS 0x102c
#define VENDOR_SUNPLUS 0x04fc
#define VENDOR_AIPTEK 0x08ca
#define VENDOR_LOGITECH 0x046d
#define VENDOR_CREATIVE 0x041e
#define VENDOR_KODAK 0x040a
#define VENDOR_POLAROID 0x0546
#define VENDOR_VIEWQUEST 0x0733
#define VENDOR_INTEL 0x8086
#define VENDOR_GRANDTECH 0x99FA
#define VENDOR_MUSTEK 0x055f
#define VENDOR_DLINK 0x084D
#define VENDOR_3COM 0x0506
#define VENDOR_MEGAPIX 0x052b
#define VENDOR_HAMA 0x0af9
#define VENDOR_AROWANA 0x1776
#define VENDOR_GENIUS 0x0458
#define VENDOR_PETCAM 0xabcd
#define VENDOR_BENQ 0x04a5
#define VENDOR_MAXELL 0x060b
#define VENDOR_UINOVATION 0x0461
#define VENDOR_FLYCAM 0x10fd
#define VENDOR_ADS 0x06e1
#define VENDOR_DIGIDREAM 0x05da
#define VENDOR_VIMICRO 0x0ac8
#define VENDOR_CONEXANT 0x0572
#define VENDOR_ICMEDIA 0x0923
#define VENDOR_VEOSTINGRAY 0x0545
#define VENDOR_OPTIMEDIA 0x06be
#define VENDOR_TOPTRO 0x2899
#define VENDOR_AGFA 0x06bd
#define VENDOR_PIXART 0x093A
#define VENDOR_JVC 0x04f1
#define VENDOR_SMILE 0x0497
#define VENDOR_CTX 0x0698
#define VENDOR_PHILIPS 0x0471
/* GSPCA Bridge list */
#define BRIDGE_SPCA505 0
#define BRIDGE_SPCA506 1
#define BRIDGE_SPCA501 2
#define BRIDGE_SPCA508 3
#define BRIDGE_SPCA504 4
#define BRIDGE_SPCA500 5
#define BRIDGE_SPCA504B 6
#define BRIDGE_SPCA533 7
#define BRIDGE_SPCA504C 8
#define BRIDGE_SPCA561 9
#define BRIDGE_SPCA536 10
#define BRIDGE_SONIX 11
#define BRIDGE_ZC3XX 12
#define BRIDGE_CX11646 13
#define BRIDGE_TV8532 14
#define BRIDGE_ETOMS 15
#define BRIDGE_SN9CXXX 16
#define BRIDGE_MR97311 17
#define BRIDGE_PAC207 18
#define SENSOR_SAA7113 0
#define SENSOR_INTERNAL 1
#define SENSOR_HV7131B  2
#define SENSOR_HDCS1020 3
#define SENSOR_PB100_BA 4
#define SENSOR_PB100_92 5
#define SENSOR_PAS106_80 6
#define SENSOR_TAS5130C 7
#define SENSOR_ICM105A 8
#define SENSOR_HDCS2020 9
#define SENSOR_PAS106 10
#define SENSOR_PB0330 11
#define SENSOR_HV7131C 12
#define SENSOR_CS2102 13
#define SENSOR_HDCS2020b 14
#define SENSOR_HV7131R 15
#define SENSOR_OV7630 16
#define SENSOR_MI0360 17
#define SENSOR_TAS5110 18
#define SENSOR_PAS202 19
#define SENSOR_PAC207 20
#define SENSOR_OV7630C 21
/* Sequence packet identifier for a dropped packet */
#define SPCA50X_SEQUENCE_DROP 0xFF
/* Offsets into the 10 byte header on the first ISO packet */
#define SPCA50X_OFFSET_SEQUENCE 0
#define SPCA50X_OFFSET_FRAMSEQ 6
#define SPCA50X_OFFSET_DATA 10
#define SPCA50X_REG_USB 0x2	// spca505 501
/* I2C interface on an SPCA505, SPCA506, SPCA508 */
#define SPCA50X_REG_I2C_CTRL 0x7
#define SPCA50X_I2C_DEVICE 0x4
#define SPCA50X_I2C_SUBADDR 0x1
#define SPCA50X_I2C_VALUE 0x0
#define SPCA50X_I2C_TRIGGER 0x2
#define SPCA50X_I2C_TRIGGER_BIT 0x1
#define SPCA50X_I2C_READ 0x0
#define SPCA50X_I2C_STATUS 0x3

enum gspca_stream_state {
	GSPCA_STREAM_OFF = 0,
	GSPCA_STREAM_INTERRUPT = 1,
	GSPCA_STREAM_ON = 2,
};
enum gspca_buffer_state {
	GSPCA_BUF_STATE_IDLE       = 0,
	GSPCA_BUF_STATE_QUEUED     = 1,
	GSPCA_BUF_STATE_ACTIVE     = 2,
	GSPCA_BUF_STATE_DONE       = 3,
	GSPCA_BUF_STATE_ERROR      = 4,
	GSPCA_BUF_STATE_NONE       = 5,
};
enum gspca_io {
	IO_NONE =0,
	IO_MMAP,
	IO_READ,
};
enum { QCIF = 1,
	QSIF,
	QPAL,
	CIF,
	SIF,
	PAL,
	VGA,
	SVGA,
	XVGA,
	CUSTOM,
	TOTMODE,
};
enum camera_t {
	JPEG = 0,		//Jpeg 4.1.1 Sunplus
	JPGH,			//jpeg 4.2.2 Zstar
	JPGC,			//jpeg 4.2.2 Conexant
	JPGS,			//jpeg 4.2.2 Sonix
	JPGM,			//jpeg 4.2.2 Mars-Semi
	YUVY,
	YYUV,
	YUYV,
	GREY,
	GBRG,
	SN9C,			// Sonix compressed stream
	GBGR,
	S561,			// Sunplus compressed stream
	PGBRG,			// Pixart compressed RGGB bayer
};

/* available gspca palette */
#define P_RGB16  1
#define P_RGB24  (1 << 1)
#define P_RGB32  (1 << 2)
#define P_YUV420  (1 << 3)
#define P_YUV422 ( 1 << 4)
#define P_RAW  (1 << 5)
#define P_JPEG  (1 << 6)
#define P_BAYER (1 << 7)

enum control_cid {
	GSPCA_BRIGHT = 0,
	GSPCA_CONTRAST,
	GSPCA_COLOR,
	GSPCA_AUTOEXPO,
	GSPCA_AUTOWB,
	GSPCATOT_CTRL,
};
struct cam_list {
	int id;
	const char *description;
};
struct palette_list {
	int num;
	const char *name;
};
struct bridge_list {
	int num;
	const char *name;
};
struct mwebcam {
	int width;
	int height;
	__u16 t_palette;
	int method;
	int mode;
};
struct size_li {
	int maxwidth;
	int maxheight;
	int minwidth;
	int minheight;
};

struct stream_camera {
	__u8 epaddr;
	int transfert;
	enum camera_t type;
	__u16 customid;
	__u16 chip_revision;
	__u16 i2c_ctrl_reg;	// Camera I2C control register
	__u8 i2c_base;		// Camera I2C address base
	__u8 i2c_trigger_on_write;	//do trigger bit on write
	struct size_li size_limit;
	struct mwebcam mode_cam[TOTMODE];
};

struct gspca_buffer {
	unsigned int size;
	unsigned long vma_use_count;
	struct list_head stream;
	/* Touched by interrupt handler. */
	struct v4l2_buffer buf;
	struct list_head queue;
	wait_queue_head_t wait;
	enum gspca_buffer_state state;
};
struct gspca_video_queue {
	void *mem;
	unsigned int streaming;
	/* this data is set for video decoding in the frame buffer acquisition
	process */
	__u32 sequence;
	unsigned int count; //buffer index
	struct gspca_buffer buffer[GSPCA_MAX_VIDEO_BUFFERS];
	struct semaphore lock;
	spinlock_t irqlock;
	
	struct list_head mainqueue;
	struct list_head irqqueue;
};
struct gspca_isobuf {
	char *data;
	struct urb *urb;
};
struct gspca_rawframe {
	unsigned char *data;	/* RawFrame buffer */
	unsigned char *ptdata;	/* used for debugging */
	struct gspca_device *dev;
	volatile int grabstate;	/* State of grabbing */
	int scanstate;		/* State of scanning */
	long scanlength;	/* uncompressed, raw data length of frame */
	int totlength;		/* length of the current reading byte in the Iso stream */
	int last_packet;
	int packet;
};
struct gspca_statcolors{
	int avg_lum;		//The average luminance (if available from theframe header)
	int avg_bg;
	int avg_rg;		//The average B-G and R-G for white balancing 
	spinlock_t v4l_lock;	/* lock to protect shared data between isoc and process context */
};
struct gspca_cur_param{
	int width;
	int height;
	int format;
	__u16 frameinterval; 
	__u16 contrast;
	__u16 brightness;
	__u16 colors;
	__u8 autoexpo;
	__u8 autowb;
	__u8 qindex;
};
struct gspca_device; 
typedef void (*cam_ops) (struct gspca_device *);
typedef int (*initcam_ops) (struct gspca_device *);
typedef int (*cam_set_ctrl) (struct gspca_device *,struct v4l2_control *);
typedef int (*cam_get_ctrl) (struct gspca_device *,struct v4l2_control *);
typedef int (*cam_query_ctrl) (struct gspca_device *,struct v4l2_queryctrl *);
typedef int (*cam_detect) (struct gspca_device *, struct gspca_rawframe *,
			      unsigned char *, int *, int, int *);
struct cam_operation {
	initcam_ops initialize;
	initcam_ops configure;
	cam_ops start;
	cam_ops stopN;
	cam_ops stop0;
	cam_ops cam_shutdown;
	cam_detect sof_detect;
	cam_set_ctrl set_ctrl;
	cam_get_ctrl get_ctrl;
	cam_query_ctrl query_ctrl;
};

struct gspca_device{
	struct video_device *vdev;
	struct usb_device *dev;
	struct stream_camera camera;
	struct tasklet_struct gspca_tasklet;
	atomic_t in_use; /*tasklet list protect */
	struct gspca_video_queue queue;
	struct gspca_statcolors avg_colors;
	struct gspca_cur_param stream_param; //courant control per webcam
	struct gspca_isobuf sbuf[GSPCA_NUMSBUF];
	__u8 *fbuf; //raw_buffer
	struct gspca_rawframe frame[GSPCA_RAWBUFF];
	int buf_state;
	unsigned int bufsize;
	int curframe;
	int bridge;		/* Type of bridge */
	int sensor;		/* Type of image sensor chip */
	int desc;		/* enum camera name */
	__u8 iface;
	__u8 alt;
	int mode; // set frame streaming size
	unsigned long last_times;	//timestamp
	unsigned int dtimes;	//nexttimes to acquire
	int streaming;
	int synchro;
	struct mutex dev_lock;
	struct mutex buf_lock;
	struct cam_operation funct;
	int user; //set on open
	int present; //set on probe
}; 
#endif /* GSPCAV2_H  */
